
;****************************************************************************
;****************************************************************************
; Process_Dial -- Handle display and adjustment of LCD_Dat
;

Process_DialAF:

m1cAF:  jsr     write_dataAF            ;send LCD_dat_n digits to LCD

m2AF:   lda     #$0E                    ;cursor on
        jsr     cmnd2LCD
        lda     #$C5                    ;init cursor to 10's position
        jsr     cmnd2LCD

m2aAF:  lda     #6
        sta     sens_sav                ;save entered encoder Sensitivity value
        sta     sens_cw                 ;set CW sensitivity control
        sta     sens_ccw                ;set CCW sensitivity control

             ;Check pushbuttons for exit

m3AF:   brset   PB1,portH,m3aAF      ;if PB1 pressed
        brclr   PB1,portH,*          ;wait for release
        jmp     pd_exitAF            ;  then exit Process_Dial
m3aAF:  brset   PB4,portG,m3cAF      ;if PB4 pressed
        jmp     pd_exitAF            ;  then exit Process_Dial
m3cAF:  brset   PB3,portG,m3dAF      ;if PB3 pressed
        jmp     pd_exitAF
m3dAF:  brset   PB2,portH,m3bAF      ;if PB2 pressed
        jmp     pd_exitAF            ;  then exit Process_Dial

             ;Check the encoder

m3bAF:  mov     portf,temp2             ;else buffer the input
        lda     temp2
        and     #$30                    ;isolate the encoder bits
        sta     enc_new
        eor     enc_old
        bne     m4AF                    ;change detected
        bra     m3AF

             ;Change Detected

m4AF:   jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        bclr    3,enc_old               ;change detected ... which way?
        lda     enc_old
        rola
        eor     enc_new
        sta     enc_old
        brclr   5,enc_old,upAF

             ;encoder turned down (ccw)
downAF:
        lda     sens_sav
        sta     sens_cw                 ;Reset CW Sensitivity cntr
        dec     sens_ccw
        beq     dwn05AF                 ;sensitivity timer to zero, process turn
        mov     enc_new,enc_old         ;else, wait for SENS_CCW transitions to occur before continuing
        bra     m3AF

dwn05AF: brset   3,temp2,dwn1AF
        ldhx    cursor_index            ;Enc_PB pressed ... adjust cursor over digit
        txa                             ;index in A
        cmp     #$0                     ;already at list end? (i.e., under 1MHz digit?)
        beq     dwn2AF                  ;yes, don't move it any further
        aix     #-1                     ;else move it left one position
dwn2AF: sthx    cursor_index
        lda     digit_posnAF,X          ;get LCD digit address
        jsr     Cmnd2LCD                ;put cursor there
        mov     enc_new,enc_old
        jmp     m2aAF                   ;go wait for encoder change

dwn1AF: ldhx    cursor_index            ;point to digit to decrement
        jsr     Bcd_DecAF               ;decrement the BCD digit currently pointed to
        jsr     Write_DataAF            ;update LCD

down3AF: mov     enc_new,enc_old
        jmp     m2aAF                   ;go wait for encoder change

             ;encoder turned up (cw)
upAF:
        lda     sens_sav
        sta     sens_ccw                ;Reset CCW Sensitivity cntr
        dec     sens_cw
        beq     up05AF                  ;sensitivity timer to zero, process turn
        mov     enc_new,enc_old         ;else, wait for SENS_CCW transitions to occur before continuing
        jmp     m3AF

up05AF: brset   3,temp2,up1AF
        ldhx    cursor_index            ;Enc_PB pressed ... adjust cursor over digit
        aix     #1
        txa                             ;index in A
        cmp     AFCursor_End              ;at list end?
        tax
        bne     up2AF                   ;no, continue
        aix     #-1                     ;yes, keep cursor at right extreme
up2AF:  sthx    cursor_index
        lda     digit_posnAF,X          ;get LCD digit address
        jsr     Cmnd2LCD                ;put cursor there
        mov     enc_new,enc_old
        jmp     m2aAF                   ;go wait for encoder change

up1AF:  ldhx    cursor_index            ;point to digit to decrement
        jsr     Bcd_IncAF               ;increment the BCD digit currently pointed to
        jsr     Write_DataAF            ;update LCD
up3AF:  mov     enc_new,enc_old
        jmp     m2aAF                   ;go wait for encoder change

             ; Exit Process_Dial (one of the pushbuttons pressed)
pd_exitAF:

        rts


digit_posnAF:      db $C4,$C5,$C6,$C7  ;digits 1000'sMHz to 1's

;****************************************************************************
; Bcd_Inc -- BCD increment subroutine
;
; This routine does a binary coded decimal increment to the values in LCD_Dat.
; On entry, the X register contains the offset (0-4) of the LCD_Dat digit to be incremented.


Bcd_IncAF:
        sthx    start_index    ;save the starting index (points to selected digit)
BCD_IAF:
        lda     LCD_dat,X      ;get the digit to increment
        add     #1
        sta     temp1
        add     #6
        bhcs    carryAF        ;the add went past 9
        lda     temp1          ;else just store char and exit
        and     #$0F
        sta     LCD_dat,X
        bra     exit2AF

carryAF: clr    LCD_dat,X      ;clear the current digit
        aix     #-1            ;point to next higher digit
        tstx                   ;pointing to ms digit?
        bpl     BCD_IAF        ;no, just increment & test again
        aix     #1             ; stay here we are
        bra     exit2AF        ; yes just exit


exit2AF:  rts


;***************************************************************************
; BCD_DEC -- BCD decrement subroutine
;
; This routine does a binary coded decimal decrement to the values in LCD_Dat.
; On entry the INDEX variable must point to the LCD_Dat digit to be decremented,
; INDEX variable = 0 (1000's) through 3 (1's)

BCD_DECAF:
        sthx    start_index      ;save the starting index (points to selected digit)
BCD_d1AF: lda     LCD_dat,X      ;get the digit to increment
        sub     #1
        bcs     carry2AF         ;borrow occured
        sta     LCD_dat,X        ;no borrow, just store & continue
        bra     exitAF

carry2AF: lda     #9
        sta     LCD_dat,X        ;set current digit = 9
        aix     #-1              ;point to next higher digit
        tstx                     ;pointing to ms digit?
        bne     BCD_D1AF         ;no, ripple upward through next digit
        lda     LCD_dat,X        ;yes, is ms digit already 0?
        tsta
        bne     BCD_D1AF         ;no, decrement it like others

        ldhx    start_index      ;else, set the display to min value
set_minAF:                         ;starting at initial digit & going upward
        clr     LCD_dat,X
        tstx                     ;at ms digit? (i.e., pointer X = 0?)
        beq     exitAF
        aix     #-1              ;point to next higher digit
        bra     set_minAF

exitAF:   rts


;***************************************************************************
; WRITE_DATA -- Write contents of LCD_dat+0 through LCD_dat+4 to the LCD

Write_DataAF:
        clra
        sta        AFleading_zero
        jsr        Busy_Check
        lda        #$C4               ;LCD position for first Digit
        jsr        cmnd2LCD

        jsr        Busy_check
        lda        #$20
        ldx        LCD_Dat+0
        cpx        #$00               ;is digit a 0?
        beq        wd2AF              ;yes, leave A as space char and go display
        txa                           ; no move to A
        ora        #$30               ; and force ascii
        sta        AFleading_zero     ; and now print any further 0's
wd2AF:  jsr        data2LCD           ;Send 1000's digit

        jsr        Busy_check
        lda        AFTwo_Digits
        bne        wd5aAF
        lda        #$20
        ldx        LCD_Dat+1
        cpx        #$00               ;is digit a 0?
        beq        wd3AF              ;yes, leave A as space char if leading 0
        txa                           ; no move to A
        ora        #$30               ; and force ascii
        sta        AFleading_zero     ; and now print any further 0's
        bra        wd3aAF
wd3AF:  txa                           ; save the zero
        ora        #$30               ; and force ascii
        ldx        AFleading_zero     ; do we print a zero
        bne        wd3aAF             ; yes do it
        lda        $20                ; no still a leading zero so print space
wd3aAF: jsr        data2LCD           ;Send 100's digit

        jsr        Busy_check
        lda        #$20
        ldx        LCD_Dat+2
        cpx        #$00               ;is digit a 0?
        beq        wd4AF              ;yes, leave A as space char if leading 0
        txa                           ; no move to A
        ora        #$30               ; and force ascii
        sta        AFleading_zero     ; and now print any further 0's
        bra        wd4aAF
wd4AF:  txa                           ; save the zero
        ora        #$30               ; and force ascii
        ldx        AFleading_zero     ; do we print a zero
        bne        wd4aAF             ; yes do it
        lda        $20                ; no still a leading zero so print space
wd4aAF: jsr        data2LCD           ;Send 10's digit

        jsr        Busy_check
        lda        LCD_Dat+3
wd5AF:  ora        #$30
        jsr        data2LCD           ;Send 1's digit

        ldhx       cursor_index
        lda        digit_posnAF,X     ;put cursor back under proper digit
        jsr        cmnd2LCD

        rts

wd5aAF: lda        LCD_Dat+1
        bra        wd5AF

