$BASE 10T
;*************************************************************************************
;                              "Micro908 AF-908"
;                               DSP Audio Filter
;
;  George Heron, N2APB       Steve Holton, N1NB             Lyle Johnson, KK7P
;  2419 Feather Mae Ct.      9 Alewives Rd                  18345 Osprey Court
;  Forest Hill, MD  21050    Norwalk CT 06850-2201          MT. VERNON WA 98274
;  email: n2apb@amsat.org    email: sholton@optonline.net   email: kk7p@wavecable.com
;
;
; Revision History:
; ----------------
; 0.1    12/31/2004     Initial shell includes pass-thru ability to load DSP code
;                       to DSPx card from serial port as hooked through PB5.
; 1.0    2/15/2005      Complete initial version including dsp908 and ASP support
;
;**************************************************************************
; Copyright 2004 by G. Heron, N2APB.  All rights reserved.
;**************************************************************************

$IFNOT AA_AF_both      ; Already included in combined version
        include "ramdefs.asm"	      ;module defines all RAM and port usage

;**************************************************************************
;**********************      USER APP ENTRY POINT     *********************
;**************************************************************************
;        This is the point ($8040) where User Code starts executing       *
;        after coming from the Monitor "X" command (eXecute User App)     *
;        or after board gets a RESET (when Monitor jumper is off)         *
;**************************************************************************

        org     $8040
$ENDIF

StartAF:
        sei                            ;disable all interrupts
$IF FULL_DEBUGGER                      ; if true then full debugger capability
        ldhx    #$01ff
$ELSEIF
        ldhx    #$03FF                 ;initialize
$ENDIF

        txs                            ;  the stack pointer

RestartAF:
        jsr     Init_allAF             ;initialize system variables and devices

        cli                            ;allow interrupts to happen

;**************************************************************************
;**************************************************************************
;*****************      USER APPPLICATION MAINLINE      *******************
;**************************************************************************
;**************************************************************************

User_MainAF:
        mov     #3,timeout1            ;I'm not sure why this is needed (but it is!)
        jsr     LCD_init
        sta     copctl                 ;clear the COP counter
        lda     #100
        jsr     delay

             ;display battery voltage
        jsr     LCD_Clear
        ldhx    #Batt_msg               ;"Battery Voltage:"
        jsr     Disp_Msg
        jsr     Calc_Batt
        lda     #$89                    ;point to mid-1st line
        jsr     Cmnd2LCD
        jsr     dispb2
        jsr     wait500ms
        clra
        sta     DSP_offset              ; DSP program location in SEEPROM not known
        sta     DSP_offset+1
        sta     MSG_instant             ; set instant process dial flag off
        sta     MSG_one                 ; set single command flag off
        sta     ManualDL                ; set manualDL flag off

        jsr     chkDSP_EEPROMsig        ; SMH check for EEPROM initialized for DSP
                                        ; SMH if not this routine will set defaults
        lda     DSPdefault_load         ; if default pgm to be loaded
        beq     do_PB1                  ; if no go to "run" menu item
        jmp     PB1_Run                 ; otherwise go do load now
do_PB1:
        jmp     PB1_func                ; SMH start in PB1 function
startmainAF:
        jsr     LCD_Clear
        ldhx    #AF1_msg
        jsr     disp_msg                ;display banner on 1st line
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #AF2_Msg
        jsr     disp_msg                ;"Select Command"

st0:    brset   PB1,porth,st1
        jmp     PB1_func                ;go process PB1 function
st1:    brset   PB2,porth,st2
        jmp     PB2_func                ;go process PB2 function
st2:    brset   PB3,portG,st3
        jmp     PB3_func                ;go process PB3 function
st3:    brset   PB4,portG,st4
        jmp     PB4_func                ;go process PB4 function (CONFIG)
st4:    brset   PB5,portG,st0
        jmp     PB5_func                ;go process PB5 function (DSP Load)

AF1_msg: fcb     '  AF908 v1.2b',0
AF2_msg: fcb     'Select Function',0

;**************************************************************
;  INCLUDED MODULES
;**************************************************************

      include "pb1.asm"
      include "pb2.asm"
      include "pb3.asm"
      include "pb4-config.asm"
      include "pb5.asm"
      include "DSP_run.asm"             ; SMH load and start DSP pgm
      include "rundsp908.asm"           ; SMH explicit dsp908 run support
      include "runASP.asm"              ; SMH explicit ASP run support
      include "DSPSoftwareUART.asm"     ;SMH
      include "AF_dial.asm"             ; SMH DSP process dial
      include "init_af.asm"

$IFNOT AA_AF_Both                       ; Already included in combined version
      include "SPI_FileXfer.asm"        ;SMH
      include "SPI_Functions.asm"       ;SMH
      include "timer.asm"
      include "drivers.asm"
      include "math.asm"
      include "misc.asm"
$ENDIF
