;=======================================================================
;
; The following are Bit Banging Serial routines
;
;=======================================================================
;
BIT_PORT equ portA  ; Bit bang port
;;BIT_DDR  equ ddr??   ; Bit bang data direction for port Uncomment for single Rx/Tx line
nBTX     equ 5      ; Bit Bang Serial Transmit pin
nBRX     equ 4      ; Bit Bang Serial Receive pin
;
;     _putch(a-reg) Send a byte via bit-bang serial routine
;
;        export _putch
DSPputch:
        psha
;;--    bset nBTX,BIT_DDR       ;    uncomment if RX/TX on single line
        bclr nBTX,BIT_PORT      ;
        bsr _bit_dly            ;              890
        nsa                     ;  filler        3
        nop                     ;  filler        1
        lda #9T                 ;                2
        sec                     ;                1
_ser_o_lp:
        ror 1,sp                ;    5     5     5
        bcs one_bit             ;    3     3     3
        bclr nBTX,BIT_PORT      ;    4           4
        bra zero_bit            ;    3         ---
one_bit:                        ;              909
        bset nBTX,BIT_PORT      ;          4
        nop                     ;          1
        nop                     ;          1
        nop                     ;          1
zero_bit:
        bsr _bit_dly            ;  890   890
        deca                    ;    1     1
        bne _ser_o_lp           ;    3     3
;;--    bclr nBTX,BIT_DDR       ;  ---   ---   uncomment if RX/TX on single line
        pula                    ;  909   909
        rts
;
;     _getch() receive a byte via bit-bang serial routine
;               waits forever, returns byte in a-reg
;
;        export _getch
;       export _getchne


DSPgetch:                  ; echo to serial terminal
        bsr  DSPgetchne
        jsr  _putch
        rts
DSPgetchne:
        lda #8
        psha
wt_ser_rdy:
        brclr nBRX,BIT_PORT,wt_ser_rdy     ; Wait for previous rx done
wt_ser_in:
        sta copctl
        brset nBRX,BIT_PORT,wt_ser_in     ;          5    Wait for start bit
        brset nBRX,BIT_PORT,wt_ser_in     ;          5    Make double sure
        bsr _hbit_dly                     ;        445
                                          ;        ---
                                          ;        455
ser_in_lp:
        nsa                               ;    3          Filler
        nsa                               ;    3          Filler
        bsr _bit_dly                      ;  890
        brclr nBRX,BIT_PORT,rx_bit_zero   ;    5          Read bit into carry
rx_bit_zero:
        ror 1,sp                          ;    5          put rx bit in byte
        dbnza ser_in_lp                   ;    3          loop 8 times
        pula                              ;  ---          retrieve rx byte
        bsr _bit_dly                      ;  909
        rts
;
;       _clr_rx          Receive until input chars  stop
;
_clr_rx:
       clrx
clr_rx_lp1:
       clra
clr_rx_lp2:
       brset nBRX,BIT_PORT,no_start_bit  ;          check for stop bit
       bra _clr_rx
no_start_bit:
       dbnza clr_rx_lp2
       dbnzx clr_rx_lp1
       rts
;
;     1 cycle = .11454545 US
;   for 9600 baud
;     1/9600       = 104.1666us / bit
;
;     1   bit time = 909 cycles   (104.1666us/.11454545 us = 909.33)
;     1/2 bit time = 455 cycles
;
;     _hbit_dly Delay on half a bit time for 9600 baud,
;
;    jsr  to invoke                            5 cycles
;
_hbit_dly:
      psha            ;                        2 cycles
      nop             ;  filler                1 cycle
      lda #142T       ;                        2 cycles
      bra bd_lp       ;                        3 cycles
;
;     _bit_dly Delay on bit time for 9600 baud
;
;              Total - Other delay
;       Full:    890  -   17        =  873
;       Half:    445  -   19        =  426
;
;    jsr  to invoke       5 cycles
;
_bit_dly:
      psha            ;   2 cycles
      lda #200T       ;   2 cycles
bd_lp1:
      dbnza bd_lp1    ; 600 (3*200) cycles
      lda #91T        ;   2 cycles
bd_lp:
      dbnza bd_lp     ; 273 (3*91) cycles    426 (3*142) cycles
      pula            ;   2 cycles             2 cycles
      rts             ;   4 cycles             4 cycles
                      ; ----                 ---
                      ; 890                  445
;
;       _puts(h:x-reg)  Puts a string out the bit-bang serial port
;
;        export _puts
DSPputs:
puts_lp:
        lda ,x
        beq end_puts
        jsr DSPputch
        aix #1
        bra puts_lp
end_puts:
        rts

DSPputn:
        lda ,x
        jsr DSPputch
        aix #1
        deca
        bne DSPputn
        rts
