;***************************************************************************
;  MODULE:  RamDefs.asm
;
;  Contains definitions of all RAM usage and port definitions.
;  Common module to all main programs (AA, AF, AA+AF, etc.)
;
;***************************************************************************

        include "ab_regs.asm"            ;MC68HC908AB32 register definitions
        include "macros.asm"             ;Macro instruction definition
        include "local.asm"              ;Local system variables

;**************************************************************************
;  EQUATES
;**************************************************************************

RAMStart     equ  $0053         ;User App RAM is from 53 to 3FF. (HCmon RAM from 400-44F)
EEPROM       equ  $800          ;EEPROM mem is 800-9FF (512 bytes)
RomStart     equ  $8000         ;Flash mem is 8000 to FFFF (32 MB)

                                ;Port A pin usage
KbdData      equ  7               ;Keyboard data line
DSP_RST      equ  6               ;DSPx Reset
DSP_RXD      equ  5               ;DSPx RxD data line
DSP_TXD      equ  4               ;DSPx TxD data line
LCD_e        equ  3               ;LCD enable = bit 3 of porta
LCD_rw       equ  2               ;LCD read/write. 0=write, 1=read
LCD_rs       equ  1               ;LCD register select. 0=instruction, 1=data
KbdClk       equ  0               ;Keyboard clock line

                                ;Port B pin usage
CHAN_Va      equ  4               ;Reflectometer Va voltage
CHAN_BATT    equ  3               ;Battery voltage
CHAN_Vf      equ  2               ;Reflectometer Forward Voltage
CHAN_Vz      equ  1               ;Reflectometer Impedance Voltage
CHAN_Vr      equ  0               ;Reflectometer Reflected Voltage

                                ;Port C Usage
Keyline      equ  5               ;Keyline
BusyLED      equ  4               ;Busy LED
Dah          equ  3               ;Dah paddle input
Backlite     equ  2               ;LCD Backlight Control bit
Dit          equ  1               ;Dit paddle input

                                ;Port D pin usage  (LCD data lines)

                                ;Port E Usage
SEE_SCK      equ  7               ;SEEPROM SCK
SEE_SI       equ  6               ;SEEPROM SI
SEE_SO       equ  5               ;SEEPROM SO
SEE_CS       equ  4               ;SEEPROM CS
Piezo        equ  2               ;Piezo sounder
RxData       equ  1               ;serial port receive
TxData       equ  0               ;serial port transmit

                                ;Port F pin usage
Mon_Jumper   equ  7               ;Monitor jumper
HeartbeatLED equ  6               ;Heartbeat LED
ENC_A        equ  5               ;Encoder-A pin
ENC_B        equ  4               ;Encoder-B pin
ENC_pb       equ  3               ;Encoder-Pushbutton pin
DDS_load     equ  2               ;DDS load pin
DDS_clk      equ  1               ;DDS clock pin
DDS_data     equ  0               ;DDS data pin

                                 ;Port H Usage

ModePB       equ  1               ;Push Button 1
BandPB       equ  0               ;Push Button 2

PB1          equ  1               ;Push Button 1
PB2          equ  0               ;Push Button 2

                                 ;Port G Usage
ScanPB       equ  2               ;Push Button 3
ConfigPB     equ  1               ;Push Button 4
SleepPB      equ  0               ;Push Button 5

PB3          equ  2               ;Push Button 3
PB4          equ  1               ;Push Button 4
PB5          equ  0               ;Push Button 5

                                ;Measurement Modes
Impedance_Mode       equ  0
VFO_Mode             equ  1
Link908_Mode         equ  2
Capacitance_Mode     equ  3
Inductance_Mode      equ  4

                                ;Data Mode ... how to handle data during scan:
NoData               equ  0     ;  Save data from Scan
Data2Mem             equ  1     ;  Save data from Scan to SEEPROM
SendDuringScan       equ  2     ;  Send data during Scan
SendFromMem          equ  3     ;  Send data from SEEPROM *now* (this is actually a command from Config)

;**************************************************************************
;  PAGE 0 VARIABLE MEMORY ALLOCATION
;**************************************************************************

    org RamStart                ;$0053

_hbufpos     rmb  2             ;buffer position temp storage

sens_sav     ds   1             ;save for entered encoder Sensitivity value
sens_cw      ds   1             ;CW sensitivity control
sens_ccw     ds   1             ;CCW sensitivity control

DDS_active   ds   1             ;flag used in Process_Dial to include DDS ctl & SWR calcs

                                ;1ms timeout counters used in Timer ISR
Timeout1     ds   1             ;8-bit timeout counter used for HeartBeat LED
Timeout2     ds   2             ;16-bit timeout counter

LCD_timer    ds   1             ;counter for timeout check in LCD busy_check
LCD_read     ds   1             ;temp store for status read in LCD busy_check
LCD_char     ds   1             ;temp for LCD data written (cmnd2LCD & data2LCD)
temp1        ds   1             ;temp store used in write_LCD and DDS mpy + add routine
temp2        ds   1

temp         ds   1             ;temp used in EEPROM driver

enc_new      ds   1             ;new reading of encoder
enc_old      ds   1             ;last reading of encoder
enc_temp     ds   1             ;temp location for encoder bits
A_flag       ds   1             ;enc line A hi/lo status
B_flag       ds   1             ;enc line B hi/lo status

mult_count   ds   1             ;Used in calc_dds_word
bit_count    ds   1             ;   "
byte2send    ds   1             ;
byte_count   ds   1             ;DDS byte counter

DDS_w4       ds   1             ;DDS programming register W4 (freq-b7  ...  freq-b0) [LSB]
DDS_w3       ds   1             ;DDS programming register W3 (freq-b15 ...  freq-b8)
DDS_w2       ds   1             ;DDS programming register W2 (freq-b23 ...  freq-b16)
DDS_w1       ds   1             ;DDS programming register W1 (freq-b31 ...  freq-b24)
DDS_w0       ds   1             ;DDS programming register W0 (phase-b4 ...  control-0) [MSB]

LCD_dat      ds   7             ;LCD data display for frequency: XX,XXX.XX  [LCD_dat+0 is MSD]
Scan_Start   ds   7             ;Scan START freq, placed into LCD_dat in BAND processing, used in Scan
Scan_End     ds   7             ;Scan END freq, placed into LCD_dat in BAND processing, used in Scan

;-----------------------------------------------------------------------------
; START of "Page 0 data" saved to & restored from EEPROM
;      Freq Dial display array:
;            LCD_Dat+0, LCD_Dat+1, LCD_Dat+2, LCD_Dat+3, LCD_Dat+4, LCD_Dat+5, LCD_Dat+6
;              10 MHz     1 MHz     100 kHz     10 kHz     1 kHz     100 Hz      10 Hz
;     StepPtr =  00        01         02         03         04         05          06
;
NV_STRT      equ  *		;start of Non Volitile (EEPROM) variables
Custom_Start ds   7             ;holds Custom Start frequency, as set in Config/Start_Freq
Custom_End   ds   7             ;holds Custom End frequency, as set in Config/End_Freq
StepPtr      ds   2             ;holds step increment, used in Scan routine, set in Config/ScanStepSize
Tone_Flag    ds   1             ;flag turns tone on during manual scan
DataMode     ds   1             ;indicates how to handle data during scan:
                                ;          0 = NoData = Save data from Scan
                                ;          1 = Data2Mem = Save data from Scan to SEEPROM
                                ;          2 = SendDuringScan = Send data during Scan
                                ;          3 = SendFromMem = Send data from SEEPROM *now*
lite_tens    ds   1             ;backlight off display digit (tens)
lite_units   ds   1             ;backlight off display digit (units)
LiteOffMins  ds   1
LiteOnFlag   ds   1             ;indicates if LCD backlight was manually turned on (even if timed out)
NV_SIZ       equ  *-NV_STRT	;Size of NV data
;
; END of "Page 0" data saved/restored from EEPROM
;------------------------------------------------------------------------------

op2          ds   1             ;operand 2 for DDS multiply+add calculation

start_index  ds   2
cursor_index ds   2             ;variable for LCD digit address used in setting cursor
step_count   ds   2             ;variable for counting scan steps

;---------------------keep the next 8 bytes contiguous for SPI usage  SMH
;---------------------    AND in this order for SPI usage             SMH
;            ---------  these  2 bytes used only by SPI and wait500ms SMH
SPI_Args     equ  *             ; 2 x 16 bit argument list            SMH
SPI_RamAddr  equ  *             ;16 bit RAM addr                      SMH
SPI_toAddr   equ  *             ;

count_lo     ds   1
count_hi     ds   1
;               ---------  these  6 bytes used only by SPI and EEPROM SMH
SPI_XferArgs equ  *
SPI_Offset   equ  *             ;16 bit SEEPROM offset                SMH

to_addr      ds   2             ;index pointer storage locs

SPI_Count    equ  *             ; SPI 16 bit byte count               SMH
SPI_CountHi  equ  *             ; SPI  count HI byte                  SMH
SPI_CountLo  equ  *+1           ; SPI  count LO byte                  SMH

from_addr    ds   2

SPI_Ptr      equ  *             ;16 bit buffer pointer                SMH

SPI_Count2   equ  *             ; SPI 16 bit byte count2              SMH
EEbyte_count ds   1             ;EEPROM byte counter
;;drop EEline_count ds   1             ;EEPROM line counter
;-----------------------end contiguous -------------------            SMH

msg_cntr     ds   1             ;counter used as index into Msg Table
msg_pointer  ds   2             ;storage for message pointer
msg_callback ds   2             ; Address of immediate dial callback    SMH
                                ; Used when msg_instant non-zero used when AF908
                                ;  doing Dial Selection processing
                                ;  available at other times
;;drop CmndTabPntr  ds   2
CmndTabStart ds   2             ; start of command list in get dial selection
                                ; if MSG_one non-zero -> only one command address
                                ;    use it in all cases
MsgTabStart  ds   2
;;drop MsgTabPntr   ds   2

intacc1      ds   4             ;32-bit integer accumulator #1
intacc2      ds   4             ;32-bit integer accumulator #2

z            ds   4             ;32-bit integer store for Z
z_squared    ds   4             ;32-bit integer store for Z^2
r_temp       ds   4             ;32-bit integer store for (2500+Z^2) in Calc_R
SWR          ds   4             ;32-bit integer store for SWR
SWRx100      ds   4             ;32-bit integer store for SWRx100
SWR_temp     ds   4             ;32-bit integer store for 50 * (SWR^2 +1)
R            ds   4             ;32-bit integer store for R

spval        ds   2             ;storage for stack pointer value

WW           ds   1             ;SWR calculation whole number
DD           ds   1             ;SWR calculation decimal remainder

WW_min       ds   1             ;Min SWR - whole number
DD_min       ds   1             ;Min SWR - decimal number
F_min        ds   7             ;Min frequency (LCD_dat digits)

bcd100000    ds   1
bcd10000     ds   1
bcd1000      ds   1             ;results of Bin2BCD
bcd100       ds   1
bcd10        ds   1
bcd1         ds   1

sqrt         ds   2             ;used in imath.asm
PwrDwnRegs   ds   2             ;two byte register for Power Down control, decrements in ISR
Reactance    ds   2             ;binary Reactance value
Current_React ds   2             ;Current Reactance value
Freq         ds   2             ;binary frequency (10M+1M+100K+10K+1K) from LCD_Dat(0:6) Freq registers
lchar        ds   1             ;Link char save area

;**************************************************************************
;  EXTENDED VARIABLE MEMORY ALLOCATION (i.e. addr $0100 and upward)
;**************************************************************************

Vf           ds   1             ;ADC reading for Vf
Vr           ds   1             ;ADC reading for Vr
Vz           ds   1             ;ADC reading for Vz
Va           ds   1             ;ADC reading for Va
EE_dirty     ds   1             ;dirty flag used in saving Config settings
enc_units    ds   1             ;variables used to display encoder counter
enc_tens     ds   1             ;  in PwrDwn and LiteOff option
Tone_freq    ds   1             ;8-bit value used for piezo tone (5=500Hz...10=1kHz...100=10kHz)
Tone_Freq_min ds  1             ;saved min Tone_Freq
debug_flag   ds   1             ;set=1 to see debug print statements to terminal
iCalc_flag   ds   1             ;flag set during dial idle times to perform Calc/Display in heartbeat ISR
Va_acc       ds   4
Vfx100       ds   4             ;accumulators (x100) for Vf, Vr, Vz and Va ADC readings
Vr_acc       ds   4
Vz_acc       ds   4
Z_maxed      ds   1             ;flag indicates when Z computation exceeds 600

temp1_acc    ds   4             ;temporary accumulators
temp2_acc    ds   4
PwrDwnMinCtr ds   1             ;countdown variable for Power Down Minutes
LiteOffMinCtr ds  1             ;countdown variable for Backlight Off Minutes
sign         ds   1             ;sign of the reactance.  Either '+' (inductive) or '-' (capacitive)
No_Disp_Flag ds   1             ;set flag to have a silent computation of SWR, R and X (used in X_sign.asm)
FreqSave     ds   7             ;save locations for frequency dial LCD_Dat(0:6)
SWR_Disp_Flag ds  1             ;allows display of SWR label in Process_Dial (Mode.asm)
R_Disp_Flag  ds   1             ;allows display of R in Calcs
X_Disp_Flag  ds   1             ;allows display of X in Calcs
Mode_nr      ds   1             ;Mode number (0=imp, 1=VFO, 2=cap, 3=ind)

DSP_offset    ds 2              ; SMH offset in SEEPROM where DSP programs start
DSP_bytecount ds 2              ; SMH 16 bit bytecount for all data for a DSP pgm
DSP_currentCnt ds 2             ; SMH 16 bit bytecount for current download
DSP_status_msg ds 17            ; SMH msg buffer with DSP status
DSP_cmd2       ds 2             ; SMH buffer for 2 byte commands
DSP_fil_nibble ds 1             ; SMH 0=update low 1=update hi nibble filterparms
PB5_reset      ds 1             ; SMH 0= reset 1=no reset when in direct mode
DSPisdsp908    ds 1             ; SMH =1 if dsp908 is running
DSPisASP       ds 1             ; SMH =1 if ASP is running
DSPfilter_next ds   1           ; 0,1,2 = next filter is narrow, mid, wide
DSPwidth_set   ds   1           ; offset from DSP_cwfil to width being set
DSPtoggle_next ds   1           ; next NR & Notch
                                ; $00= both off, $04 =Notch, $08= NR, $0C=Both on
MSG_instant    ds   1           ; Set to 1 when process dial callback active
MSG_one        ds   1           ; set to 1 when only one command address used in
                                ;   get dial selection
ManualDL       ds   1           ; set when manual download

AFleading_zero ds   1           ; SMH clear until first non zero digit in LCD AF_DIA
AFcursor_end   ds   1           ; SMH # of digits = right end for cursor
AFtwo_Digits   ds   1           ; SMH if non-zero only 2 digits display
PB_turned      ds   1           ;flag to indicate that cursor was ajusted during Config "Change_Freq" rtn
PB_pressed     ds   1           ;flag to indicate ENC_PB was pressed during freq adjust (in Config "Change_Freq) rtn

;-----------------------------------------------------------------------------
; START of "Page 1 data" saved to & restored from EEPROM (calibration data)
;
Start_P1_data:
Vf_offset    ds   1             ;op amp offsets with no signal present
Va_offset    ds   1
Vr_offset    ds   1
Vz_offset    ds   1
Va_corrfact  ds   2             ;Va correction factor ... first byte + remainder byte
Vr_corrfact  ds   2             ;Vr correction factor ... first byte + remainder byte
Vz_corrfact  ds   2             ;Vz correction factor ... first byte + remainder byte

End_P1_data:
; END of "Page 1 data" saved/restored from EEPROM (calibration data)
;------------------------------------------------------------------------------


;-----------------------------------------------------------------------------
; START of "Page 2 data" saved to & restored from EEPROM (DSP data)
;
; Do NOT change the order of these.  Groups of related data are accessed by indices!
; Signature and global values

Start_P2_data:
DSP_EEPROMsig ds 4              ; signature tocheck if EEPROM valid "%dSp"
DSPdefault_load ds 1            ; if not 0 load this value-1 image from SEEPROM
DSPdefault_name ds 8            ; name of pgm including terminating null

;;----------- dsp908 section
DSP_response:                   ; SMH 2 byte reponse to commands go here
DSP_statusbyte: ds 1            ; SMH Status byte maintains current status
DSP_filterparms: ds 1           ; SMH filter parameters
DSPcw_fil      ds   1           ; current filter parameters by mode CW
DSPcw_narrow   ds   1           ; CW narrow filter parms
DSPcw_mid      ds   1           ; CW medium filter parms
DSPcw_wide     ds   1           ; CW wide filter parms
DSPssb_fil     ds   1           ; current filter parameters by mode SSB
DSPssb_narrow  ds   1           ; ssb narrow filter parms
DSPssb_mid     ds   1           ; ssb medium filter parms
DSPssb_wide    ds   1           ; ssb wide filter parms
DSPrtty_fil    ds   1           ; current filter parameters by mode rtty
DSPrtty_narrow ds   1           ; rtty narrow filter parms
DSPrtty_mid    ds   1           ; rtty medium filter parms
DSPrtty_wide   ds   1           ; rtty wide filter parms
DSPcw_mode     ds   1           ; current mode parameters by mode
DSPssb_mode    ds   1
DSPrtty_mode   ds   1
                                ; Command
DeNoisergain   ds   1           ;  (D0)   gains save to intialize DSP on startup
AutoNotchgain  ds   1           ;  (D1)     these are in order of commands
CWgain         ds   1           ;  (D2)     so that low nibble can be used as
SSBgain        ds   1           ;  (D3)     an index
RTTYgain       ds   1           ;  (D4)
CODECgain      ds   1           ;  (D5)
DSPfill        ds   1           ; was DSPtaps placeholder to preserve other offsets

DenoiserBeta   ds   1           ; Values stored as number 0-99
DenoiserDecay  ds   1           ; This group accessed by index 0-3
NotchBeta      ds   1
NotchDecay     ds   1

;;--------- ASP section         ;   low nibble
ASP_status     ds   1           ; 1=700 Hz 100 Hz wide CW filter
                                ; 2=700 Hz 200 Hz wide CW filter
                                ; 3=700 Hz 300 Hz wide CW filter
                                ; 4=700 Hz 400 Hz wide CW filter
                                ; 5=700 Hz 500 Hz wide CW filter
                                ; 6=1500 Hz 1200 Hz wide SSB filter
                                ; 7=1500 Hz 1800 Hz wide SSB filter
                                ; 8=1500 Hz 2400 Hz wide SSB filter
                                ; 9=2210 Hz 200 Hz wide RTTY filter

ASP_status2    ds   1           ; $00= both off, $01 =Notch, $02= NR, $03=Both on
ASPcwStatus    ds   1
ASPssbStatus   ds   1
ASPrttyStatus  ds   1
ASPcwStatus2   ds   1
ASPssbStatus2  ds   1
ASPrttyStatus2 ds   1
ASPTaps        ds   1           ; as command 'q', 'w', 'a', 's'
ASP_BFO        ds   2           ; current BFO (actual value - not offset)
ASPcwBFO       ds   2
ASPssbBFO      ds   2
ASPrttyBFO     ds   2

End_P2_data:
; ------------- Don't change order of these  -----------------
; END of "Page 2 data" saved/restored from EEPROM (DSP data)
;------------------------------------------------------------------------------

$IF FULL_DEBUGGER
          org   $200         ;SMH  this will make room
                             ;     for stack that will enable full debugger
$ENDIF
                                ; SMH SPI Contiguous 256 byte buffer
SPI_buffer1 ds 128              ; SMH 1st 1/2 of SPI Contiguous 256 byte buffer
SPI_buffer2 ds 128              ; SMH 2nd 1/2 of SPI Contiguous 256 byte buffer

    org $03FF
;      (Stack starts here and works down toward $0053)

    org $0400
;      ($0400 - $04FF used by HCmon)

    org $040A
_inbuf       rmb  RXBLEN        ;common "_inbuf" buffer with HCmon

;**************************************************************************
;   Vector Jump Table (located at start of User Area = 8000)
;**************************************************************************

    org RomStart

_user_adc         jmp    Dummy_ISR      ; Insert jump to ADC vector code
_user_keyboard    jmp    Dummy_ISR      ; Insert jump to Keyboard vector code
_user_scitx       jmp    Dummy_ISR      ; Insert jump to SCI transmit vector code
_user_scirx       jmp    Dummy_ISR      ; Insert jump to SCI receive vector code
_user_scierr      jmp    Dummy_ISR      ; Insert jump to SCI error vector code
_user_timbch3     jmp    Dummy_ISR      ; Insert jump to TIMB Channel 3 Vector code
_user_timbch2     jmp    Dummy_ISR      ; Insert jump to TIMB Channel 2 Vector code
_user_spitx       jmp    Dummy_ISR      ; Insert jump to SPI transmit vector code
_user_spirx       jmp    Dummy_ISR      ; Insert jump to SPI receive vector code
_user_timb_of     jmp    Dummy_ISR      ; Insert jump to TIMB Overflow Vector code
_user_timb_ch1    jmp    Dummy_ISR      ; Insert jump to TIMB Channel 1 Vector code
_user_timb_ch0    jmp    Dummy_ISR      ; Insert jump to TIMB Channel 0 Vector code
_user_tima_of     jmp    Dummy_ISR      ; Insert jump to TIMA Overflow Vector code
_user_tima_ch3    jmp    Dummy_ISR      ; Insert jump to TIMA Channel 3 Vector code
_user_tima_ch2    jmp    Dummy_ISR      ; Insert jump to TIMA Channel 2 Vector code
_user_tima_ch1    jmp    T_ISR_A1       ; Insert jump to TIMA Channel 1 Vector code
_user_tima_ch0    jmp    T_ISR_A0       ; Insert jump to TIMA Channel 0 Vector code
_user_pit         jmp    Dummy_ISR      ; Insert jump to PIT Vector code
_user_pll         jmp    Dummy_ISR      ; Insert jump to PLL Vector code
_user_irq         jmp    Dummy_ISR      ; Insert jump to ~IRQ1 Vector code

_HCmon_LCD_disp   jmp    HCmon_LCD_disp ; vector (at $803C) for HCmon
                                        ; to display message to LCD

