;*********************************************************************************
;*********************************************************************************
;                       Config Process for Micro908
;*********************************************************************************
;*********************************************************************************

Config_Process:
        brclr   ConfigPB,portG,*        ;wait until Config PB released
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        tone_off                        ;turn tone off while in Config menu
        jsr     LCD_Clear
        ldhx    #Config_msg
        jsr     disp_msg                ;display "Setup Selection" on 1st line

        ldhx    #ConfigCmndTable        ;set up pointer to ConfigCmndTable
;;drop        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #ConfigMsgTable         ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
;;drop        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

             ;Save all Config settings in EEPROM if EE_dirty flag is set, then exit
exitconfig:
        lda     EE_dirty
        beq     cp_exit                 ;just exit if no settings changes made
        bclr    BusyLED,portc           ;turn on Busy LED
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #Save_Settings
        jsr     disp_msg                ;"Saving settings.."
        jsr     SaveUserSettings
        bset    BusyLED,portc           ;turn off Busy LED
        brclr   ENC_PB,portf,*          ;wait here until Encoder PB released
cp_exit:
        clra
        sta     EE_dirty
        jmp     Impedance

Config_msg:     fcb     'Config Selection',0
SetStep_msg:    fcb     'Select step size:',0
SetCustom_msg:  fcb     '- Custom Band -',0
Save_Settings:  fcb     '    (saving)   ',0

;-----------------------------
; SaveUserSettings -- Saves users settings to EEPROM.

SaveUserSettings:
        bclr    BusyLED,portc           ;turn on Busy LED
        jsr     Put_EEPROM_P0           ;Save Config Data to EEPROM Page 0
                                        ;(routine located in Drivers_m.asm)
        bset    BusyLED,portc           ;turn off Busy LED
        rts

;--------------------------
;   Config JUMP TABLE

ConfigCmndTable:
        fdb     SetStepSize
        fdb     ExitConfig
        fdb     Calibrate
        fdb     Debug_Mode
        fdb     SW_Load
$IF  AA_AF_Both
        fdb     RunAF908
$ENDIF
        fdb     Monitor
        fdb     DispBatt
        fdb     Default_Settings
        fdb     DataOptions
        fdb     TurnLightOn
        fdb     TurnLightOff
        fdb     LightOffTime
        fdb     ViewSettings
        fdb     ToneOff
        fdb     ToneOn
        fdb     SetCustomEnd
        fdb     SetCustomStart

;---------------------------
;   Config MESSAGE TABLE

ConfigMsgTable:
        fcb     'Step Size',0           ;CW direction
        fcb     'Exit',0                ;1st message displayed
        fcb     'Calibrate',0           ;CCW direction
        fcb     'DEBUG Mode',0
        fcb     'Software Load',0
$IF  AA_AF_Both
        fcb     'Run AF908',0
$ENDIF
        fcb     'Debug Monitor',0
        fcb     'Battery Voltage',0
        fcb     'Default_Settings',0
        fcb     'Data Options',0
        fcb     'Turn Lite On',0
        fcb     'Turn Lite Off',0
        fcb     'Light Off Time',0
        fcb     'View Settings',0
        fcb     'Disable Tones',0
        fcb     'Enable Tones',0
        fcb     'End Freq',0
        fcb     'Start Freq',0
        fcb     0

$IF  AA_AF_Both
;******************************************************************************
; RUN AF908 -- This code vectors the user over to the AF908 program

RunAF908:
        sei             ; mask interrupts
        jmp     StartAF
$ENDIF

;******************************************************************************
; DispBatt -- Display the battery voltage

DispBatt:
        jsr     LCD_Clear
        ldhx    #Batt_msg               ;"Battery Voltage:"
        jsr     Disp_Msg

             ;Read battery voltage and display it
dispb1: jsr     Calc_Batt
        lda     #$89                    ;point to mid-1st line
        jsr     Cmnd2LCD
        jsr     dispb2
        brset   ENC_PB,portf,dispb1     ;do again if Encoder PB not pressed
        brclr   ENC_PB,portf,*          ;wait here until Encoder PB released
        jmp     Config_Process          ;then exit

             ;Display Battery Voltage
dispb2:
        lda     bcd100000
        add     #$30
        jsr     data2lcd
        lda     bcd10000
        add     #$30
        jsr     data2lcd
        lda     #'.'
        jsr     data2lcd
        lda     bcd1000
        add     #$30
        jsr     data2lcd
        lda     bcd100
        add     #$30
        jsr     data2lcd
        rts

Batt_msg:  fcb  'Battery:',0

;*********************************************************************************
; SetStepSize --
;      Sets 'StepPtr" to one of the four valid offsets into the LCD_Dat display array:
;            LCD_Dat+0, LCD_Dat+1, LCD_Dat+2, LCD_Dat+3, LCD_Dat+4, LCD_Dat+5, LCD_Dat+6
;              10 MHz     1 MHz     100 kHz     10 kHz     1 kHz     100 Hz      10 Hz
;                           StepPtr =  02        03         04        05         06

SetStepSize:
        jsr     LCD_Clear
        ldhx    #SetStep_msg
        jsr     disp_msg                ;display "Select step size:" on 1st line

        ldhx    #StepCmndTable          ;set up pointer to ConfigCmndTable
;;drop        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #StepMsgTable           ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
;;drop        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jsr     get_dial_selection
        jmp     Config_Process

Sel_100KHz:
        ldhx    #$02
        bra     sss_exit

Sel_10KHz:
        ldhx    #$03
        bra     sss_exit

Sel_1KHz:
        ldhx    #$04
        bra     sss_exit

Sel_100Hz:
        ldhx    #$05
        bra     sss_exit

Sel_10Hz:
        ldhx    #$06

sss_exit:
        sthx    StepPtr
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;--------------------------
;   Step JUMP TABLE

StepCmndTable:
        fdb     Sel_100KHz
        fdb     Sel_10KHz
        fdb     Sel_1KHz
        fdb     Sel_100Hz
        fdb     Sel_10Hz

;---------------------------
;   Step MESSAGE TABLE

StepMsgTable:
s2:     fcb     '100 kHz',0
s3:     fcb     '10 kHz',0
s4:     fcb     '1 kHz',0
s5:     fcb     '100 Hz',0
s6:     fcb     '10 Hz',0
        fcb     0

;*********************************************************************************
SetCustomStart:
        jsr     LCD_Clear
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Start_msg               ;"START frequency:"
        jsr     Disp_Msg
        clr     DDS_active               ;clear flag to not use DDS or SWR calcs

        mov     Custom_Start+0,LCD_dat+0 ;start the display at previous (or default)
        mov     Custom_Start+1,LCD_dat+1 ;  custom start setting
        mov     Custom_Start+2,LCD_dat+2
        mov     Custom_Start+3,LCD_dat+3
        mov     Custom_Start+4,LCD_dat+4
        mov     Custom_Start+5,LCD_dat+5
        mov     Custom_Start+6,LCD_dat+6

        ldhx    #2
        sthx    cursor_index             ;digit address for cursor

        jsr     Process_Dial             ;put current Custom_Start to the LCD
                                         ; and allow user to modify it

        mov     LCD_dat+0,Custom_Start+0 ;save selected start frequency
        mov     LCD_dat+1,Custom_Start+1
        mov     LCD_dat+2,Custom_Start+2
        mov     LCD_dat+3,Custom_Start+3
        mov     LCD_dat+4,Custom_Start+4
        mov     LCD_dat+5,Custom_Start+5
        mov     LCD_dat+6,Custom_Start+6
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

Start_msg: fcb  '(exit = <Config>)',0

;*********************************************************************************
SetCustomEnd:
        jsr     LCD_Clear
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #End_msg                 ;"END:"
        jsr     Disp_Msg
        clr     DDS_active               ;clear flag to not use DDS or SWR calcs

        mov     Custom_End+0,LCD_dat+0   ;start the display at previous (or default)
        mov     Custom_End+1,LCD_dat+1   ;  custom end setting
        mov     Custom_End+2,LCD_dat+2
        mov     Custom_End+3,LCD_dat+3
        mov     Custom_End+4,LCD_dat+4
        mov     Custom_End+5,LCD_dat+5
        mov     Custom_End+6,LCD_dat+6

        ldhx    #2
        sthx    cursor_index             ;digit address for cursor

        jsr     Process_Dial             ;put current Custom_End to the LCD
                                         ; and allow user to modify it

        mov     LCD_dat+0,Custom_End+0   ;save selected start frequency
        mov     LCD_dat+1,Custom_End+1
        mov     LCD_dat+2,Custom_End+2
        mov     LCD_dat+3,Custom_End+3
        mov     LCD_dat+4,Custom_End+4
        mov     LCD_dat+5,Custom_End+5
        mov     LCD_dat+6,Custom_End+6

        lda     LCD_dat+0
        cmp     #3                       ;is max end set to 3?  (i.e., 30 MHz?)
        bne     sce_1                    ;no, just leave dial setting as-is
        clra                             ;yes, ensure that max freq doesn't exceed 30 MHz
        sta     LCD_dat+1
        sta     Custom_End+1
        sta     LCD_dat+2
        sta     Custom_End+2
        sta     LCD_dat+3
        sta     Custom_End+3
        sta     LCD_dat+4
        sta     Custom_End+4
        sta     LCD_dat+5
        sta     Custom_End+5
        sta     LCD_dat+6
        sta     Custom_End+6

sce_1:  lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

End_msg:   fcb  '(exit = <Config>)',0

;********************************************************************************
ToneOn:
        lda     #1
        sta     Tone_Flag
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;********************************************************************************
ToneOff:
        clra
        sta     Tone_Flag
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;*********************************************************************************
; DataOptions --
;      Allows user to:
;             Not save data from Scan
;             Save data from Scan to SEEPROM
;             Send data during Scan
;             Send data from SEEPROM now

DataOptions:
        jsr     LCD_Clear
        ldhx    #Data_msg
        jsr     disp_msg                ;display "Data Options:" on 1st line

        ldhx    #DataCmndTable          ;set up pointer to ConfigCmndTable
;;drop        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #DataMsgTable           ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
;;drop        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jsr     get_dial_selection
        jmp     Config_Process

nd:     lda     #NoData
        bra     do_exit

sds:    lda     #SendDuringScan
        bra     do_exit

sfm:    lda     #SendFromMem
        bra     do_exit

            ; Send data from SEEPROM to PC
d2m:    nop                             ;INSERT SEEPROM USAGE CODE HERE
        lda     #Data2Mem

do_exit:
        sta     DataMode
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;--------------------------
;   Data JUMP TABLE

DataCmndTable:
        fdb     sfm                     ;SendFromMem
        fdb     nd                      ;NoData
        fdb     d2m                     ;Data2Mem
        fdb     sds                     ;SendDuringScan

;---------------------------
;   Data MESSAGE TABLE

DataMsgTable:
        fcb     'Send from Mem',0       ;CW
        fcb     'No Data Save',0        ;start
        fcb     'Data to Mem',0         ;CCW
        fcb     'Send during Scan',0
        fcb     0

Data_msg:   fcb  'Data Options:',0

;********************************************************************************
;TurnLightOn

TurnLightOn:
        jsr     TurnLiteOn
        lda     #$FF
        sta     LiteOnFlag              ;indicates that LCD backlight should be turned on when checked
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;********************************************************************************
;TurnLightOff

TurnLightOff:
        jsr     TurnLiteOff
        clra
        sta     LiteOnFlag              ;indicates that LCD backlight should be turned off when checked
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process

;********************************************************************************
; Light Off Time -- Set the number of minutes until backlight turned off

LightOffTime:
        jsr     LCD_clear
        ldhx    #LiteOffMsg1             ;"Set LiteOff Time"
        jsr     Disp_Msg
        lda     lite_units
        sta     enc_units
        lda     lite_tens
        sta     enc_tens
        jsr     GetDialReading
        sta     LiteOffMins              ;save binary result
        lda     enc_units                ;and save ascii results
        sta     lite_units
        lda     enc_tens
        sta     lite_tens
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     Config_Process           ;and exit

LiteOffMsg1:
        fcb     'LiteOff Time',0

             ;----------------------------------------------------------------
             ; GetDialNumbers -- Get dial reading 00-99
             ;                   return binary in A
             ;                   and ascii in enc_tens, enc_units
             ;----------------------------------------------------------------

GetDialReading:
        jsr     ClearLCDLine2
        lda     #$C0                     ;point to 2nd line on LCD
        jsr     Cmnd2LCD
        ldhx    #EncoderMsg
        jsr     Disp_Msg
        jsr     Update_Digits            ;display current numbers for enc_tens,enc_units
se1:    lda     #4                       ;dial sensitivity
        jsr     Read_Encoder             ;process the encoder turns
        bcs     se3                      ;exit if ENC PB pressed
        bne     se2
        jsr     increase_digit_display   ;CW turn detected
        bra     se1
se2:    jsr     decrease_digit_display   ;CCW turn detected
        bra     se1

             ;Convert (enc_tens:enc_units) to binary in A

se3:    lda     enc_tens
        and     #$0F                     ;strip off upper ascii nibble
        sta     intacc1+1                ;and place into intacc1(0:1)
        clr     intacc1+0
        mov     #10,intacc2+1            ;load 10 into intacc2(0:1)
        clr     intacc2+0
        jsr     umult16                  ;multiply tens by 10
        lda     enc_units
        and     #$0F                     ;strip off upper ascii nibble
        add     intacc1+3                ;add to multiplication result and return in A
        brclr   ENC_PB,portf,*           ;wait here until Encoder PB released
        rts

EncoderMsg:
        fcb     'Minutes = 00',0

             ;-------------------------
             ;encoder turned down (ccw)

Decrease_Digit_Display:
        lda     enc_units
        deca                          ;decrement units digit
        sta     enc_units
        cmp     #$2F                  ;units at min?
        beq     dwn1a                 ;yes, roll down
        bra     dwn1b                 ;no,just continue

dwn1a:  lda     #$39
        sta     enc_units             ;roll down units to 9
        lda     enc_tens
        deca                          ;and decr tens
        sta     enc_tens
        cmp     #$2F                  ;tens at min?
        beq     dwn1c                 ;yes, reset cntr to 00
        bra     dwn1b                 ;no, just continue

dwn1c:  lda     #$39
        sta     enc_units             ;reset tens to zero
        sta     enc_tens              ; and units to zero

dwn1b:  jsr     update_digits         ;update counter digits
        rts
             ;----------------------
             ;encoder turned up (cw)

Increase_Digit_Display:
        lda     enc_units             ;increment units digit
        inca
        sta     enc_units
        cmp     #$3A                  ;units at max?
        beq     up1a                  ;yes, roll over
        bra     up1b                  ;no, just continue

up1a:   lda     #$30
        sta     enc_units             ;roll over units to zero
        lda     enc_tens              ;and bump tens
        inca
        sta     enc_tens
        cmp     #$3A                  ;tens at max?
        beq     up1c                  ;yes, reset cntr to 00
        bra     up1b                  ;no, just continue

up1c:   lda     #$30
        sta     enc_units             ;reset tens to zero
        sta     enc_tens              ; and units to zero

up1b:   jsr     update_digits         ;update counter digits
        rts
             ;--------------
             ;update Digits
update_digits:
        lda     #$CA                  ;point to tens digit on 2nd line
        jsr     Cmnd2LCD
        lda     enc_tens              ;send the tens digit there
        jsr     Data2LCD
        lda     #$CB                  ;point to units digit on 2nd line
        jsr     Cmnd2LCD
        lda     enc_units             ;send the units digit there
        jsr     Data2LCD
        rts

;********************************************************************************
; Default Settings
; Returns user-customizable settings to default values in EEPROM

Default_Settings:
        jsr     LCD_clear
        ldhx    #Default_Msg1         ;"Restore default?"
        jsr     Disp_Msg
        jsr     Get_YN                ;get Yes/No answer
        bcs     ds1                   ;CY=1=Yes
        clra
        sta     debug_flag            ;CY=0=No, so turn off flag
        clr     EE_dirty              ;don't save settings upon exiting Config
        jmp     Config_Process        ;and exit

            ;Clear EEPROM
ds1:    jsr     LCD_clear
        ldhx    #Default_Msg2         ;'(please wait)'
        jsr     Disp_Msg
        bclr    BusyLED,portc         ;turn on Busy LED
        jsr     Erase_EEPROM_P0       ;erase EEPROM Page 0 locations
        jsr     Init_EEPROM_vars      ;set nonvolatile vars to default settings
        jsr     Put_EEPROM_P0a        ;write nonvolatile vars to EEPROM Page 0
        bset    BusyLED,portc         ;turn off Busy LED
        jmp     Config_Process

Default_Msg1:  fcb  'Restore default?',0
Default_Msg2:  fcb  ' (please wait) ',0

;********************************************************************************
; Debug Mode
; Turns Debug_Flag on/off to allow/disallow diagnostic print statements going
; to terminal

Debug_Mode:
        jsr     LCD_clear
        ldhx    #Debug_Msg1          ;"Debug Mode?"
        jsr     Disp_Msg
        jsr     Get_YN                ;get Yes/No answer
        bcs     dm1                   ;CY=1=Yes
        clra
        sta     debug_flag            ;CY=0=No, so turn off flag
        clr     EE_dirty              ;don't save settings upon exiting Config
        jmp     Config_Process        ;and exit

            ;Turn on Debug_Flag
dm1:    lda     #1
        sta     debug_flag
        clr     EE_dirty              ;don't save settings upon exiting Config
        jmp     Config_Process        ;and exit

Debug_Msg1:
        fcb     'Debug Mode?',0

;********************************************************************************
; Software Load -- Produce a jump into the HCmon so the user can issue the C and L
;                  commands from TerarTerm to invoke the sending of new software
;                  to the Micro908.
;                  We basically simulate a power on reset and execute the exact
;                  same instructions here as would be done in HCMON with the P3
;                  jumper in place - except we don't test for P3.  We just jump
;                  into HCMON as if P3 is in. The code and the locations are the
;                  same per the listings of V3 and V2. This ensures that no code
;                  is different between the 2 cases.

SW_Load:
        sei                             ; disable all interupts
        ldhx    #$450                   ;;;  <===> ldhx  #_mon_stack+1      ; initialize
        txs                             ;  the stack pointer
        fcb     $CD,$f2,$00             ;;; <===> jsr     PLLset ; change bus speed

             ; Initialize the SCI.
        mov      #init_scbr,scbr        ; initialize baud rate
        mov      #init_scc1,scc1        ; initialize SCI Control Register 1
        mov      #init_scc2,scc2        ; initialize SCI Control Register 2
        mov      #init_config1,config1  ; init Configuration Register 1

             ; Test Port F7 for "Monitor Jumper"
             ;   Entry to Monitor  = jumper OFF ==> F7 = 1
             ;   Entry to User Pgm = jumper ON  ==> F7 = 0
        mov     #$40,ddrf               ;set portf bits 7,5,4,3,2,1,0 as inputs, bit 6 (LED) as output
        mov     #$B8,prtfpu             ;assign pullups to Port F bits 7,5,4,3
        bclr    6,portf                 ;turn LED on continuously while in monitor
                                        ;;;    brclr   7,portf,MonStart  ;go start Monitor if "Monitor Jumper" is in place
        jmp     $ef2a                   ; $EF2A == MonStart  DON'T Test it just do it!!!

;********************************************************************************
; Debug Monitor

Monitor:
        swi                            ;Software Interrupt enters HCmon
        rts

;*********************************************************************************
; Get Yes/No Response -- Presents Yes/No message selection to the LCD.
;                        Return with CY=1 if Yes
;                                    CY=0 if No

Get_YN:
        ldhx    #YNCmndTable            ;set up pointer to ConfigCmndTable
;;drop        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #YNMsgTable             ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
 ;;drop       sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

Do_Yes: sec                             ;set CY = 1 = Yes
        rts                             ;this returns to the caller of Get_YN

Do_No:  clc                             ;clear CY = No
        rts                             ;this returns to the caller of Get_YN

;--------------------------
;   Yes/No JUMP TABLE

YNCmndTable:
        fdb     Do_no
        fdb     Do_Yes


;---------------------------
;   Yes/No MESSAGE TABLE

YNMsgTable:
No_ssg:  fcb     '  No',0
Yes_msg: fcb     '  Yes',0
         fcb     0

;********************************************************************************
ViewSettings:
        jsr     LCD_Clear
        ldhx    #UserSettings_msg       ;"User Settings:"
        jsr     Disp_Msg

             ;Display Custom_Start frequency

vs0:    jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Startf_msg             ;"Start:"
        jsr     Disp_Msg
        lda     #$C7
        ldhx    #Custom_Start
        jsr     Disp_Ufreq

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs05                    ;contnue if Enc not pressed
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Custom_End frequency

vs05:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Endf_msg               ;"End:"
        jsr     Disp_Msg
        lda     #$C5
        ldhx    #Custom_End
        jsr     Disp_Ufreq

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs3
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Step Size

vs3:    jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Stepf_msg              ;"Step:"
        jsr     Disp_Msg
        lda     #$C6
        jsr     Cmnd2LCD
        lda     StepPtr+1
        cmp     #2
        bne     vs3a
        ldhx    #s2                     ;"100 kHz"
        bra     vs7
vs3a:   cmp     #3
        bne     vs4
        ldhx    #s3                     ;"10 kHz"
        bra     vs7
vs4:    cmp     #4
        bne     vs5
        ldhx    #s4                     ;"1 kHz"
        bra     vs7
vs5:    cmp     #5
        bne     vs6
        ldhx    #s5                     ;"100 Hz"
        bra     vs7
vs6:    cmp     #6
        beq     vs6a
        jmp     vs_err
vs6a:   ldhx    #s6                     ;"10 Hz"

vs7:    jsr     disp_msg                ;display the Step size

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs7a                    ;continue
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display Tone setting

vs7a:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Tone_msg               ;"Tone:"
        jsr     Disp_Msg
        lda     #$C6
        jsr     Cmnd2LCD
        tst     Tone_Flag
        beq     vs7b
        ldhx    #ToneOn_msg             ;'enabled'
        bra     vs8
vs7b:   ldhx    #ToneOff_msg            ;'disabled'
vs8:    jsr     Disp_Msg

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs9                     ;continue
        jmp     vs_exit                 ;exit if Encoder PB pressed

             ;Display the DataMode

vs9:    jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #DataMode_msg           ;"DataMode:"
        jsr     Disp_Msg
        lda     #$CA
        jsr     Cmnd2LCD
        lda     DataMode
        cmp     #NoData
        bne     vs9a
        ldhx    #nd_msg                 ;"NoSend"
        bra     vs9c
vs9a:   cmp     #Data2Mem
        bne     vs9b
        ldhx    #d2m_msg                ;"To Mem"
        bra     vs9c
vs9b:   ldhx    #sds_msg                ;"To PC"
vs9c:   jsr     Disp_Msg
        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcc     vs13                    ;continue
        jmp     vs_exit                 ;exit if Encoder PB pressed

            ;Display Backlight setting

vs13:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #LiteFlagMsg            ;"Backlight:"
        jsr     Disp_Msg
        lda     #$CB
        jsr     Cmnd2LCD
        lda     LiteOnFlag
        beq     vs13a
        ldhx    #On_msg                 ;'on'
        bra     vs13b
vs13a:  ldhx    #Off_msg                ;'off'
vs13b:  jsr     Disp_Msg
        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcs     vs_exit                 ;exit if Encoder PB pressed

             ;Display LiteOffMins setting

        jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #LiteOffMsg2             ;"LiteOff Timer: "
        jsr     Disp_Msg
        lda     #$CE
        jsr     Cmnd2LCD
        lda     lite_tens
        jsr     Data2LCD
        lda     #$CF
        jsr     Cmnd2LCD
        lda     lite_units
        jsr     Data2LCD
        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcs     vs_exit                 ;exit if Encoder PB pressed

             ;Display Debug setting

vs12:   jsr     ClearLCDLine2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Debug_msg               ;"Debug:"
        jsr     Disp_Msg
        lda     #$CD
        jsr     Cmnd2LCD
        tst     Tone_Flag
        beq     vs10
        ldhx    #On_msg                 ;'on'
        bra     vs11
vs10:   ldhx    #Off_msg                ;'off'
vs11:   jsr     Disp_Msg

        lda     #5                      ;encoder sensitivity
        jsr     Read_Encoder            ;wait for encoder to move
        bcs     vs_exit                 ;exit if Encoder PB pressed

        jmp     vs0                     ;go loop again

vs_exit:
        brclr   Enc_PB,portf,*          ;await Encoder PB release
        jmp     Config_Process

vs_err: swi                             ;error if get here

;--------------------------
Disp_Ufreq:
        jsr     Cmnd2LCD                ;put cursor to selected line 2 spot
        lda     ,x                      ;display 10M digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 1M digit
        ora     #$30
        jsr     Data2LCD
        lda     #','                    ;display comma
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 100K digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 10K digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 1K digit
        ora     #$30
        jsr     Data2LCD
        lda     #'.'                    ;display decimal point
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 100 Hz digit
        ora     #$30
        jsr     Data2LCD
        aix     #1
        lda     ,x                      ;display 10 Hz digit
        ora     #$30
        jsr     Data2LCD
        rts

UserSettings_msg:  fcb  'User Settings:',0
Startf_msg:        fcb  'Start: ',0
Endf_msg:          fcb  'End: ',0
Stepf_msg:         fcb  'Step: ',0
Tone_msg:          fcb  'Tone: ',0
ToneOn_msg:        fcb  'enabled',0
ToneOff_msg:       fcb  'disabled',0
Debug_msg:         fcb  'Debug: ',0
On_msg:            fcb  'on',0
Off_msg:           fcb  'off',0
DataMode_msg:      fcb  'DataMode:',0
nd_msg:            fcb  'NoData',0
sds_msg:           fcb  'To PC',0
d2m_msg:           fcb  'To Mem',0
LiteOffMsg2:       fcb  'LiteOff Timer:',0
LiteFlagMsg:       fcb  'Backlight:',0
