;*********************************************************************************
;*********************************************************************************
;                       INITIZATION MODULE
;*********************************************************************************
;*********************************************************************************

Init_allAF:
             ;configure the ports
        mov     #$0F,ddra              ;set port A3-A0 as outputs
        mov     #$00,ddrb              ;set port B2-B0 as inputs
        mov     #$34,ddrc              ;set port C4, C5, C2 as outputs
        mov     #$FF,ddrd              ;set port D7-D0 as outputs
        mov     #$05,ddre              ;set port E2, E0 as outputs (and E4 if SEEPROM)
        mov     #$47,ddrf              ;set port F6, F2-F0 as outputs
        mov     #$00,ddrg              ;set port G2-G0 as inputs
        mov     #$00,ddrh              ;set port H1-H0 as inputs

             ;initialize the output ports
        mov     #$FF,porta
        mov     #$FB,portc             ;keep C2 low (Backlite off)
        mov     #$FF,portd
        mov     #$FB,porte             ;keep Piezo (E2) and Pwr Ctl (E5) off
        bset    2,portf
        bset    1,portf
        bset    0,portf

        lda     portf                  ;get initial encoder position
        and     #$30                   ;isolate the encoder bits
        sta     enc_old                ;initialize encoder old value
        sta     enc_new                ;initialize encoder new value

        mov     #$FF,A_flag
        mov     #$FF,B_flag

        mov     #1,LCD_Dat+0           ; init the power-on display to 10,000.000 MHz
        mov     #0,LCD_Dat+1
        mov     #0,LCD_Dat+2
        mov     #0,LCD_Dat+3
        mov     #0,LCD_Dat+4
        mov     #0,LCD_Dat+5
        mov     #0,LCD_Dat+6

        ldhx    #1                      ;SMH  must reconcile if we merge was 3
        sthx    cursor_index            ;set the default digit address for cursor (10 kHz)

        mov     #$70,ADCLK              ;select ADC clock divide of 011

        clra
        sta     EE_dirty                ;dirty flag, used to save Congig settings

        jsr     Init_EEPROM

        jsr     AFInit_EEPROM_vars        ;initialize variables to defaults (normally overwritten by loading EEPROM next)

             ;Load nonvolatile variables from EEPROM

        jsr     LCD_Clear               ;display "Initializing ..."
        ldhx    #init_msg
        jsr     disp_msg
	bclr    BusyLED,portc           ;turn on Busy LED
        jsr     Get_EEPROM              ;get user settings from EEPROM
                                        ; ... if blank EEPROM (addr 0800=FF), then save default valules
	bset    BusyLED,portc           ;turn off Busy LED

             ;set variables and hardware according to saved values

        lda     LiteOffMins
        sta     LiteOffMinCtr           ;init the backlite off counter from the constant from EEPROM

        lda     LiteOnFlag              ;turn lite on/off, per the saved value
        beq     ia1a
        bset    Backlite,portC          ;turn backlight ON
        bra     ia2a                    ;continue
ia1a:   bclr    Backlite,portC          ;turn backlight OFF 

ia2a:   ldhx    #$EA60
        sthx    PwrDwnRegs              ;set PwrDwn Register 1ms counter to 60,000

        jsr     Init_Timer_Module
        jsr     AFInit_Timers           ;init the system timers
        rts

$IFNOT  AA_AF_Both
Init_msg:  fcb  'Initializing...',0
$ENDIF

             ;-----------------------
             ;Init EEPROM Variables
             ;-----------------------
AFInit_EEPROM_vars:
        clra
        sta     Tone_Flag               ;turn off Tone by default

        lda     #$30                    ;init display of LiteOffMins to 1 minute
        sta     lite_tens

        lda     #$31
        sta     lite_units

        lda     #1                      ;init backlight timer to 1 min
        sta     LiteOffMins             ;(stay this way if EEPROM is initially blank
                                        ; otherwise they get overwritten by Get_EEPROM next)
        lda     #1
        sta     LiteOnFlag               ;Lite On is default

        rts

             ;-----------
             ;Init Timers
             ;-----------
AFInit_Timers:
        clr     timeout1               ;init timeouts (0=off)
        clr     timeout2
        clr     timeout2+1             ;(timeout2 is 2-byte value)
        rts

