;*********************************************************************************
;*********************************************************************************
;                       INITIZATION MODULE
;*********************************************************************************
;*********************************************************************************

Init_all:
             ;configure the ports
        mov     #$0F,ddra              ;set port A3-A0 as outputs
        mov     #$00,ddrb              ;set port B2-B0 as inputs
        mov     #$34,ddrc              ;set port C4, C5, C2 as outputs
        mov     #$FF,ddrd              ;set port D7-D0 as outputs
        mov     #$05,ddre              ;set port E2, E0 as outputs (and E4 if SEEPROM)
        mov     #$47,ddrf              ;set port F6, F2-F0 as outputs
        mov     #$00,ddrg              ;set port G2-G0 as inputs
        mov     #$00,ddrh              ;set port H1-H0 as inputs

             ;initialize the output ports
        mov     #$FF,porta
        mov     #$FB,portc             ;keep C2 low (Backlite off)
        mov     #$FF,portd
        mov     #$FB,porte             ;keep Piezo (E2) and Pwr Ctl (E5) off
        bset    2,portf
        bset    1,portf
        bset    0,portf

        lda     portf                  ;get initial encoder position
        and     #$30                   ;isolate the encoder bits
        sta     enc_old                ;initialize encoder old value
        sta     enc_new                ;initialize encoder new value

        mov     #$FF,A_flag
        mov     #$FF,B_flag

        mov     #1,LCD_Dat+0           ; init the power-on display to 10,000.000 MHz
        mov     #0,LCD_Dat+1
        mov     #0,LCD_Dat+2
        mov     #0,LCD_Dat+3
        mov     #0,LCD_Dat+4
        mov     #0,LCD_Dat+5
        mov     #0,LCD_Dat+6

;        mov     F_min+0,LCD_Dat+0
;        mov     F_min+1,LCD_Dat+1
;        mov     F_min+2,LCD_Dat+2
;        mov     F_min+3,LCD_Dat+3
;        mov     F_min+4,LCD_Dat+4
;        mov     F_min+5,LCD_Dat+5
;        mov     F_min+6,LCD_Dat+6

        ldhx    #3
        sthx    cursor_index            ;set the default digit address for cursor (10 kHz)

        mov     #$00,Scan_start+0       ;set default scan START for 1 MHz
        mov     #$01,Scan_start+1
        mov     #$00,Scan_start+2
        mov     #$00,Scan_start+3
        mov     #$00,Scan_start+4
        mov     #$00,Scan_start+5
        mov     #$00,Scan_start+6

        mov     #$01,Scan_end+0         ;set default scan END for 10 MHz
        mov     #$00,Scan_end+1
        mov     #$00,Scan_end+2
        mov     #$00,Scan_end+3
        mov     #$00,Scan_end+4
        mov     #$00,Scan_end+5
        mov     #$00,Scan_end+6

        jsr     Init_Scan_Vars          ;initialize the scan variables DD_min, WW_min, etc.

        mov     #$70,ADCLK              ;select ADC clock divide of 011

        clra                            ;clear DEBUG mode initially
        sta     debug_flag

        clra
        sta     iCalc_flag              ;disallow Calc/Display until we get to dial idle times
        sta     EE_dirty                ;dirty flag, used to save Congig settings
        sta     No_Disp_Flag            ;flag to have silent LCD during X_Sign.asm "freq diddle" processing

        lda     #Impedance_Mode
        sta     Mode_nr                 ;Set Mode initially to Impedance

        ldhx    #0
        sthx    Reactance

        lda     #1
        sta     SWR_Disp_Flag           ;allow display of "SWR" in Process_Dial (Mode.asm)
        sta     R_Disp_Flag             ;allow display of R in Calcs
        sta     X_Disp_Flag             ;allow display of X in Calcs

        lda     #'+'
        sta     sign                    ;init reactance sign to be '+' (inductive)

        lda     #NoData
        sta     DataMode                ;init Data Mode that indicates how to handle Scan data

        jsr     Init_EEPROM

        jsr     Init_EEPROM_vars        ;initialize variables to defaults (normally overwritten by loading EEPROM next)

             ;Load nonvolatile variables from EEPROM.
             ;First time running of AA908 will take longer here (with blank EEPROM),
             ;so display message so the user doesn't get worried

        jsr     LCD_Clear               ;display "Initializing ..."
        ldhx    #init_msg
        jsr     disp_msg

	bclr    BusyLED,portc           ;turn on Busy LED
        jsr     Get_EEPROM              ;get user settings from EEPROM
                                        ; ... if blank EEPROM (addr 0800=FF), then save default valules
	bset    BusyLED,portc           ;turn off Busy LED

             ;set variables and hardware according to saved values

        lda     LiteOffMins
        sta     LiteOffMinCtr           ;init the backlite off counter from the constant from EEPROM

        lda     LiteOnFlag              ;turn lite on/off, per the saved value
        beq     ia1
        bset    Backlite,portC          ;turn backlight ON
        bra     ia2                     ;continue
ia1:    bclr    Backlite,portC          ;turn backlight OFF

ia2:    ldhx    #$EA60
        sthx    PwrDwnRegs              ;set PwrDwn Register 1ms counter to 60,000

        jsr     Init_Timer_Module
        jsr     Init_Timers             ;init the system timers
        rts

Init_msg:  fcb  'Initializing...',0

             ;-----------------------
             ;Init EEPROM Variables
             ;-----------------------
Init_EEPROM_vars:
        mov     #$00,Custom_Start+0     ;set default Custom_Start to 5 Mhz
        mov     #$05,Custom_Start+1
        mov     #$00,Custom_Start+2
        mov     #$00,Custom_Start+3
        mov     #$00,Custom_Start+4
        mov     #$00,Custom_Start+5
        mov     #$00,Custom_Start+6

        mov     #$01,Custom_End+0       ;set default Custom_End to 12 Mhz
        mov     #$02,Custom_End+1
        mov     #$00,Custom_End+2
        mov     #$00,Custom_End+3
        mov     #$00,Custom_End+4
        mov     #$00,Custom_End+5
        mov     #$00,Custom_End+6

        ldhx    #3
        sthx    StepPtr                 ;set default 10 kHz digit used for Scan step size

        clra
        sta     Tone_Flag               ;turn off Tone by default

        lda     #$30                    ;init display of LiteOffMins to 1 minute
        sta     lite_tens

        lda     #$31
        sta     lite_units

        lda     #1                      ;init backlight timer to 1 min
        sta     LiteOffMins             ;(stay this way if EEPROM is initially blank
                                        ; otherwise they get overwritten by Get_EEPROM next)
        lda     #1
        sta     LiteOnFlag               ;Lite On is default

        rts

             ;-------------------------------
             ; Init_Scan_Vars
             ;-------------------------------

Init_Scan_Vars:
        mov     #$01,F_min+0            ;init default min freq registers to 10 MHz (used in Scan.asm)
        mov     #$00,F_min+1
        mov     #$00,F_min+2
        mov     #$00,F_min+3
        mov     #$00,F_min+4
        mov     #$00,F_min+5
        mov     #$00,F_min+6
        mov     #9,WW_min               ;init the SWR min stores
        mov     #9,DD_min
        rts

             ;-----------
             ;Init Timers
             ;-----------
Init_Timers:
        clr     timeout1               ;init timeouts (0=off)
        clr     timeout2
        clr     timeout2+1             ;(timeout2 is 2-byte value)
        rts
