;;---------------- Get_DialSelection
;;---------------- Calc_Batt
;;---------------- Bin2BCD
;;---------------- Utils.asm jump table
;;----AF908------- BCD2to16Bin, BCD4to16Bin
;;----AF908------- String utilities StringCopy,StringCompare,StringNCompare
;;----AF908------- MemSet,MemCopy
;**************************************************************************
; Get_Dial_Selection --

Get_Dial_Selection:
        jmp     um5a                    ;upon initial entry we want to display
                                        ;the first menu item

um2:    lda     #10                     ;dial sensitivity control
        jsr     Read_Encoder            ;process the encoder turns
        bcs     um8                     ;menu item selected (encoder pushbutton pressed)
        bne     um5a                    ;CW turn detected, go display next menu string

             ;----------------
             ;CCW turn detected
             ;Display previous menu string from Message Table
um2c:   jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls     
        ldhx    msg_pointer
        jsr     Comp2MsgTabStart        ;compare HX to MsgTabStart.  At start of table?
        bne     um4a                    ;no, start looking for start of previous string
             ;already at start of table ... must find back-to-back 0's denoting end of table,
             ; *then* back up by one message
um2a:   tst     ,x                      ;encounter 1st 0?
        beq     um2b                    ;yes
        aix     #1                      ;no, keep searching
        bra     um2a
um2b:   inc     msg_cntr                ;bump msg count
        aix     #1                      ;point to next char
        tst     ,x                      ;encounter 2nd 0?
        bne     um2a                    ;nope, go look for 1st one again
        aix     #-1                     ;yes, end of list found! Back up to end of last string
        tst     ,x                      ;this char should also be a 0
        bne     system_error            ;if not, it's a programming error
             ;now look for start of previous string
um3:    aix     #-1                     ;backup again and look for previous string end (another 0)
        tst     ,x                      ;end of string found?
        beq     um3b                    ;yes, continue below
        jsr     Comp2MsgTabStart        ;compare HX to MsgTabStart.  At start of table?
        bne     um3                     ;nope, keep searching backwards for end of previous string
        bra     um4                     ;yes, found start of previous string (i.e., the first one!)
             ;end of previous string found.
um3b:   aix     #1                      ;advance by one to point to start of selected string
             ;end of msg string found
um4:    dec     msg_cntr
        bmi     system_error            ;if msg_cntr goes negative, there is a system programming error
        bra     um7                     ;continue below with processing
                                        ;display of menu selection
             ;look for start of previous string
um4a:   aix     #-1                     ;backup to end of previous string
        tst     ,x                      ;should be a 0
        bne     system_error            ;if not, programming error
        bra     um3                     ;continue looking back for start of current message
             ;----------------
             ;CW turn detected
             ;display previous command message from Message Table
um5a:   jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls     
        ldhx    msg_pointer
um5:    tst     ,x                      ;end of string found?
        beq     um6                     ;yes, incr cntr & test for end of msg list
        aix     #$01                    ;no, just advance to next char in msg string
        bra     um5
             ;end of msg string found
um6:    inc     msg_cntr
        aix     #1                      ;end of msg list?
        tst     ,x                      ;i.e., second 0 encountered?
        bne     um7                     ;no, process this message seletion
        clr     msg_cntr                ;yes, at list end.  Wrap it
        ldhx    MsgTabstart             ;and continue on to display this first menu string

             ;Display selected menu string to the 2nd line of LCD
um7:    sthx    msg_pointer
        jsr     ClearLCDline2
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    msg_pointer             ;get the start address of the selected menu string
        jsr     disp_msg                ;display the menu item on line 2 of the LCD

$IF   AF908    ;;----  SMH  add call back processing
        lda     MSG_instant   ; when this flag set we make a "call-back" each time
                              ; the dial turns to a new message
        beq     um2           ; when not set continue
        pshh
        pshx
        ldhx    msg_callback   ; this is the call back get address of routine
        jsr     ,x             ; go do it
        pulx                   ; restore H:X
        pulh
$ENDIF          ;;-----  Leaving callback bit

        bra     um2                     ;go await another encoder event (turn or press)
             ;------------------
             ;Menu item selected!  (Shaft encoder pressed while menu string displayed)
             ;point to corresponding menu address jump table element and process the command
um8:    jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls     
        ldhx    CmndTabStart
        brclr   ENC_PB,portf,*          ;wait for release of Encoder PB

$IF  AF908    ;;--------  single command routine processing
;;                        not list of command routines
        lda     msg_one                 ; test if single command routine  SMH
        beq     um8a                    ; no go do "normal" processing
        jmp     ,x                      ; yes go directly to command
um8a:
$ENDIF

        pshh
        pshx
        lda     msg_cntr
        clc
        rola                            ;mult x 2 for address offset
        sta     temp
        pula                            ;retrieve X (low byte of #CommandTable)
        add     temp                    ;add 2*msg_cntr to it
        sta     temp                    ;temp save
        pula                            ;retrieve H (high byte of #CommandTable)
        adc     #0                      ;adjust high byte if carry exists from low byte add
        psha
        pulh                            ;get byte byte result into H
        lda     temp
        tax                             ;get low byte result into X
        lda     ,x                      ;get high byte of command addr, pointed to by H:X
        psha
        lda     1,x                     ;get low byte of command addr, pointed to by H:X+1
        tax                             ;put low byte into X
        pulh                            ;put high byte into H
        jmp     ,x                      ;Jump to specific command processor

             ;If get here, there was a programming error. Put breakpoint here and see where
             ;program flow came from.
system_error:
        swi

;------------------------------------------------
; Comp2MsgTabStart -- compare HX to MsgTabStart.

Comp2MsgTabStart:
        txa
        cmp     MsgTabStart+1
        beq     c2ma
        rts
c2ma:   pshh
        pula
        cmp     MsgTabStart
        rts

;*****************************************************************
; Calc_Batt - Calculate the battery voltage
;             5V max of 8-bit ADC = .0195 V/bit = .02 V/bit
;             Voltage divider: 10K/(22K+10K) = (.3125) x Vin

Calc_Batt:
        lda     #Chan_Batt
        jsr     Read_ADC              ;get reading of voltage divider

             ;Multiply reading x .02V/bit
             ;but scale up by 100 to handle denominator, to yield Vbattx100
        sta     intacc1+1
        clr     intacc1+0
        mov     #195,intacc2+1        ;multiply reading by 195, or (reading x (.0195 x 10,000))
        clr     intacc2+0
        jsr     umult16               ;intacc1(0:1) x intacc2(0:1) = intacc1(0:3)

             ;Multiply Vbattx100 by 10,000 (so we can divide by 3125 to get Vin)
        clr     intacc2+0
        clr     intacc2+1
        mov     #$27,intacc2+2        ;10,000. = $2710
        mov     #$10,intacc2+3
        jsr     umult32               ;intacc1(0:3) x intacc2(0:3) = intacc1(0:3)...intacc2(0:3)

             ;Lastly, divide by 3125 to get Vin
        mov     intacc2+0,intacc1+0   ;move big number to intacc1
        mov     intacc2+1,intacc1+1
        mov     intacc2+2,intacc1+2
        mov     intacc2+3,intacc1+3
        mov     #$0C,intacc2+0        ;3125. = $0C35
        mov     #$35,intacc2+1
        jsr     udvd32                ;intacc1(0:3) / intacc2(0:1) = intacc1(0:3)
        jsr     Bin2BCD
$IF  AA908
        jsr     dpr_acc1              ;debug print Vbatt
$ENDIF
        rts

;**************************************************************
; UTILS - Utilities and routines shared from HCmon
;         Just link to routines in HCmon
;         (But remember to adjust jump addresses if HCmon
;         ever changes!)
;         An alternative to linking like this is to include the line:
;             include "../utils.asm"    ; but this approach replicates code!
;**************************************************************

_sp_gets:     jmp   $F4DB
_gets:        jmp   $F4EE
_ahex2b:      jmp   $F53F
_a2nibhex:    jmp   $F54D
_dblchar:     jmp   $F556
_nib_to_asc:  jmp   $F561
_comma_sp:    jmp   $F56C
_pr_letter_x: jmp   $F577
_pr_pound:    jmp   $F57C
_pr_comma:    jmp   $F581
_crlf:        jmp   $F586
_byte_sp:     jmp   $F58F
_space:       jmp   $F592
_pr_nibble:   jmp   $F599
_putch:       jmp   $F59C
_pwordx:      jmp   $F5A5
_pword:       jmp   $F5AD
_pbyte_sp:    jmp   $F5B9
_pbytex:      jmp   $F5BD
_pbyte:       jmp   $F5BE
_ischar:      jmp   $F5CD
_getchne:     jmp   $F5D7
_getch:       jmp   $F5E1
_do_again:    jmp   $F5F6
_puts_lp:     jmp   $F602
_puts:        jmp   $F60A
_gethexbyte:  jmp   $F60E
_get_atbl:    jmp   $F62E
_tohex:       jmp   $F641
_ishex:       jmp   $F64A
_clr_spaces:  jmp   $F660
_next_hex:    jmp   $F66C
_first_hex:   jmp   $F677
_gethex:      jmp   $F67E
_readhex:     jmp   $F686
_numck_lp:    jmp   $F689
_disasm:      jmp   $F6FA


;***************************************************************************
; Bin2BCD - Convert intacc1(2:3) to BCD in bcd100,bcd10,bcd1

Bin2Bcd:
        pshhx
        lda     intacc1+3       ;get 16 bit binary number into HX
        tax
        lda     intacc1+2
        tah
        lda     intacc1+1       ;check next higher byte for a $01 (max we'll see)
        beq     b2b3            ;0 = less than 65,535, so start counting as such
        cmp     #$01
        beq     b2b65           ;1 = start counting at 65,535
        cmp     #$02
        beq     b2b131          ;2 = start counting at 131,070
        cmp     #$03
        beq     b2b196          ;3 = start counting at 196,605
        bra     b2b3            ;should never get here

             ;number is 02xxxx, so start counting bcd at 196,605
b2b196: mov     #1,bcd100000
        mov     #9,bcd10000
        mov     #6,bcd1000
        mov     #6,bcd100
        mov     #0,bcd10
        mov     #5,bcd1
        bra     b2b2
             ;number is 01xxxx, so start counting bcd at 131,070
b2b131: mov     #1,bcd100000
        mov     #3,bcd10000
        mov     #1,bcd1000
        mov     #0,bcd100
        mov     #7,bcd10
        mov     #0,bcd1
        bra     b2b2
             ;number is 01xxxx, so start counting bcd at 65,535
b2b65:  clr     bcd100000
        mov     #6,bcd10000
        mov     #5,bcd1000
        mov     #5,bcd100
        mov     #3,bcd10
        mov     #5,bcd1
        bra     b2b2
             ;number is less than 65,535 (FFFF), so clear out bcd regs
             ;for starting count at 00,000.
b2b3:   clr     bcd100000
        clr     bcd10000
        clr     bcd1000
        clr     bcd100
        clr     bcd10
        clr     bcd1

             ;start counting down HX by 10 while incrementing bcd regs by 10
b2b2:   cphx    #0              ;result =0?
        beq     b2b_exit        ;yes, done
        cphx    #10             ;result < 10?
        bcs     b2b_exit        ;yes, done
b2b1:   aix     #-10            ;bin - 10
        inc     bcd10           ;no, just increment BCD counters
        lda     bcd10
        cmp     #10             ;rollover on the 10s digit?
        bne     b2b2            ;no, do again
        clr     bcd10           ;yes, rollover
        inc     bcd100
        lda     bcd100
        cmp     #10             ;rollover on the 100s digit?
        bne     b2b2            ;no, do again
        clr     bcd100          ;yes, rollover
        inc     bcd1000
        lda     bcd1000
        cmp     #10             ;rollover on the 1000s digit?
        bne     b2b2            ;no, do again
        clr     bcd1000         ;yes, rollover
        inc     bcd10000
        lda     bcd10000
        cmp     #10             ;rollover on the 10000s digit?
        bne     b2b2            ;no, do again
        clr     bcd10000
        inc     bcd100000
        bra     b2b2            ;and do again

b2b_exit:
        txa
        sta     bcd1            ;save "less than 10 remainder" in bcd1
        pulhx
        rts

$IF  AF908   ;;----  for the moment these are only used by AF908

StringCopy:                             ; copys null terminated string from
        ldhx    from_addr               ; address in from_addr to
        lda     ,x                      ; address in to_addr
        aix     #1
        sthx    from_addr
        ldhx    to_addr
        sta     ,x
        aix     #1
        beq     StrCpyEnd
        sthx    to_addr
        bra     StringCopy
StrCpyEnd:
        sthx    to_addr
        rts

StringCompare:                    ; compares null terminated string from
        ldhx    from_addr         ; address in from_addr to
        lda     ,x                ; address in to_addr
        aix     #1                ; return Z set if equal
        sthx    from_addr         ;        Z clear if unequal
        ldhx    to_addr
        cmp     ,x
        bne     StrCompEnd
        aix     #1
        sthx    to_addr
        cmp     #0
        beq     StrCompEnd
        bra     StringCompare
StrCompEnd:
        rts

StringNCompare:           ; compares null terminated string up to N chars
        psha
S_N_C:  ldhx    from_addr         ; from address in from_addr
        lda     ,x                ; to address in to_addr
        aix     #1                ; return Z set if equal
        sthx    from_addr         ;        Z clear if unequal
        ldhx    to_addr           ; Compare stops after N chars N in Acc
        cmp     ,x                ; or Null encountered whichever is first
        bne     StrNCompEnd
        aix     #1
        sthx    to_addr
        cmp     #0
        beq     StrNCompEnd
        dec     1,sp
        beq     StrNCompEnd
        bra     S_N_C
StrNCompEnd:
        pula
        rts

MemCopy:
        psha                            ; copys # bytes in A from
M_C:    ldhx    from_addr               ; address in from_addr to
        lda     ,x                      ; address in to_addr
        aix     #1
        sthx    from_addr
        ldhx    to_addr
        sta     ,x
        aix     #1
        sthx    to_addr
        dec     1,sp
        beq     MemCpyEnd
        bra     M_C
MemCpyEnd:
        pula
        rts

MemSet:
        psha                            ; fills # bytes in A from
        txa                             ; address in from_addr with
        ldhx    from_addr               ; value in X;
M_S:    sta     ,x
        aix     #1
        dec     1,sp
        beq     MemCpyEnd
        bra     M_S

;;----------------- h:x point to 2 BCD digits in sucessive bytes
;;---------------   16 bit binary result intacc1 and intacc1+1
BCD2to16Bin:
        clr     intacc1
        clr     intacc1+1
        sthx    from_addr
        bra     BCD2
;;----------------- h:x point to 4 BCD digits in sucessive bytes
;;---------------   16 bit binary result intacc1 and intacc1+1
BCD4to16Bin:
        clr     intacc1
        clr     intacc1+1
        sthx    from_addr
        ldx     ,x              ; get 1000's digit
        lda     bin10           ;
        mul                     ; multiply by 10 = # of 100's this is
        ldhx    from_addr       ; the result must be <255 so only A is needed!
        aix     #1              ;  point to 100's digit
        add     ,x              ; so adding next 100's digit will also fit in A
        aix     #1              ;  point to 10's digit
        sthx    from_addr       ; save 10's pointer
        ldx     bin100          ; A has # 0f 100's  X has binary 100
        mul
        stx     intacc1         ; save the binary value of 1000's and 100's
        sta     intacc1+1
        ldhx    from_addr       ; get pointer to 10's
BCD2:
        ldx     ,x              ; get # of 10's
        lda     bin10           ; get binary value of 10
        mul                     ; result is in A must be less than 255
        add     intacc1+1       ; add to low byte of result
        sta     intacc1+1       ; and save
        lda     intacc1         ; get high byte and
        adc     #0              ; propagate result
        sta     intacc1
        ldhx    from_addr
        aix     #1              ; now point to 1's digit
        lda     ,x              ; get digit
        add     intacc1+1       ; add to low byte of result
        sta     intacc1+1       ; and save
        lda     intacc1         ; get high byte and
        adc     #0              ; propagate result
        sta     intacc1
        rts

bin100  fcb     100
bin10   fcb     10
bin1    fcb     1

$ENDIF
