;**************************************************************************
;**********************  MODE PROCESSING  *********************************
;**************************************************************************

Mode_Process:
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        brclr   ModePB,porth,*          ;wait until Mode PB released
        tone_off                        ;turn tone off while in Mode menu
        jsr     LCD_Clear
        ldhx    #Mode_msg
        jsr     disp_msg                ;display "Select Mode" on 1st line

        ldhx    #ModeCmndTable          ;set up pointer to ConfigCmndTable
;;drop        sthx    CmndTabPntr
        sthx    CmndTabStart
        ldhx    #ModeMsgTable           ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
;;drop        sthx    MsgTabPntr
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

Mode_msg:  fcb  'Select Mode:',0

;--------------------------
;   Mode JUMP TABLE

ModeCmndTable:
        fdb     Capacitance
        fdb     Impedance
        fdb     VFO
        fdb     Inductance

;---------------------------
;   ModeMESSAGE TABLE

ModeMsgTable:
        fcb     'Capacitance',0           ;CW direction
        fcb     'Impedance',0             ;1st message displayed
        fcb     'VFO',0                   ;CCW direction
        fcb     'Inductance',0
        fcb     0


;***************************************************************************************************
;                      IMPEDANCE MEASUREMENTS
;***************************************************************************************************

Impedance:
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        lda     #1
        sta     DDS_active              ;turn on DDS and Calculations display
        sta     SWR_Disp_Flag           ;and allow display of "SWR"
        sta     R_Disp_Flag             ;and allow display of R
        sta     X_Disp_Flag             ;and allow display of X
        clr     lchar                   ;clear the Link Char

        lda     #Impedance_Mode
        sta     Mode_nr

        lda     Tone_Flag
        beq     imp1
        tone_on                         ;turn on tone if Tone_Flag set

imp1:   jsr     Process_Dial

             ;Determine which pushbutton exited the Process_Dial routine

        brset   BandPB,portH,imp2
        jmp     Band_Process            ;go process Band
imp2:   brset   ScanPB,portG,imp3
        jmp     Scan_Process            ;go process Scan
imp3:   brset   ConfigPB,portG,imp4
        jmp     Config_Process          ;go process Config
imp4:   tst     lchar
	beq     imp5                    ;if Link Command
        jmp     Link_Process            ;go process Link Request
imp5:   jmp     Mode_process            ;else go process Mode again

;***************************************************************************************************
;                      CAPACITANCE MEASUREMENTS
;***************************************************************************************************
Capacitance:
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        jsr     LCD_Clear
        ldhx    #Cap_msg2                ;"Capacitance in pF"
        jsr     Disp_Msg
        jsr     wait500ms

        mov     #1,DDS_active           ;turn on DDS and Calculations display
        clra
        sta     SWR_Disp_Flag           ;and disallow display of "SWR"
        sta     R_Disp_Flag             ;and disallow display of R
        sta     X_Disp_Flag             ;and disallow display of X
        clr     lchar                   ;clear the Link Char

        lda     #Capacitance_Mode
        sta     Mode_nr

cap1:   jsr     Process_Dial

             ;Determine which pushbutton exited the Process_Dial routine

        brset   BandPB,portH,cap2
        jmp     Band_Process            ;go process Band
cap2:   brset   ScanPB,portG,cap3
        jmp     Scan_Process            ;go process Scan
cap3:   brset   ConfigPB,portG,cap4
        jmp     Config_Process          ;go process Config
cap4:   tst     lchar
	beq     cap5                    ;if Link Command
        jmp     Link_Process            ;go process Link Request
cap5:   jmp     Mode_process            ;else go process Mode again

Cap_Msg2: fcb  'Capacitance (pF)',0

;***************************************************************************************************
;                      INDUCTANCE MEASUREMENTS
;***************************************************************************************************
Inductance:
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        jsr     LCD_Clear
        ldhx    #Ind_msg2                ;"Inductance in uH"
        jsr     Disp_Msg
        jsr     wait500ms

        mov     #1,DDS_active           ;turn on DDS and Calculations display
        clra
        sta     SWR_Disp_Flag           ;and disallow display of "SWR"
        sta     R_Disp_Flag             ;and disallow display of R
        sta     X_Disp_Flag             ;and disallow display of X
        clr     lchar                   ;clear the Link Char

        lda     #Inductance_Mode
        sta     Mode_nr

ind1:   jsr     Process_Dial

             ;Determine which pushbutton exited the Process_Dial routine

        brset   BandPB,portH,ind2
        jmp     Band_Process            ;go process Band
ind2:   brset   ScanPB,portG,ind3
        jmp     Scan_Process            ;go process Scan
ind3:   brset   ConfigPB,portG,ind4
        jmp     Config_Process          ;go process Config
ind4:   tst     lchar
	beq     ind5                    ;if Link Command
        jmp     Link_Process            ;go process Link Request
ind5:   jmp     Mode_process            ;else go process Mode again

Ind_Msg2: fcb  'Inductance (uH)',0

;***************************************************************************************************
;                      VFO MEASUREMENTS
;***************************************************************************************************

VFO:
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        jsr     LCD_Clear		;Clear display 01/22/05
	mov     #1,DDS_active           ;turn on DDS and Calculations display
        clra
        sta     SWR_Disp_Flag           ;and disallow display of "SWR"
        sta     R_Disp_Flag             ;and disallow display of R
        sta     X_Disp_Flag             ;and disallow display of X
        clr     lchar                   ;clear the Link Char

        lda     #VFO_Mode
        sta     Mode_nr

vfo1:   jsr     Process_Dial

             ;Determine which pushbutton exited the Process_Dial routine


        brset   BandPB,portH,vfo2
        jmp     Band_Process            ;go process Band
vfo2:   brset   ScanPB,portG,vfo3
        jmp     Scan_Process            ;go process Scan
vfo3:   brset   ConfigPB,portG,vfo4
        jmp     Config_Process          ;go process Config
vfo4:   tst     lchar
	beq     vfo5                    ;if Link Command
        jmp     Link_Process            ;go process Link Request
vfo5:   jmp     Mode_process            ;else go process Mode again

VFO_msg2:  fcb   'VFO',0


;****************************************************************************
;****************************************************************************
; Process_Dial -- Handle display and adjustment of LCD_Dat freq registers.
;                 When DDS_active=1, the DDS is controlled and SWR is calculated and displayed.

Process_Dial:
        mov     F_min+0,LCD_Dat+0       ;Put min SWR freq into display registers
        mov     F_min+1,LCD_Dat+1       ;  Default of 1 MHz set in Init_all,
        mov     F_min+2,LCD_Dat+2       ;  or the min freq found during Scan routine)
        mov     F_min+3,LCD_Dat+3
        mov     F_min+4,LCD_Dat+4
        mov     F_min+5,LCD_Dat+5
        mov     F_min+6,LCD_Dat+6

	jsr	ClearLCDLine1		;Clear line 1 only
        lda     DDS_active
        beq     m1c                     ;bypass display of "SWR" if DDS not active
        lda     SWR_Disp_Flag
        beq     m1c                     ;bypass display of "SWR" if SWR_Disp_Flag not set
        lda     #$80                    ;point to first line
        jsr     Cmnd2LCD
        ldhx    #Man_Line1_msg
        jsr     disp_msg                ;display "SWR" on 1st line

m1c:    jsr     write_data              ;send LCD_dat_n digits to LCD
        jsr     mult                    ;compute new DDS control word from LCD_dat digits

        lda     DDS_active              ;bypass DDS control if flag not set
        beq     m2                      ; (Don't want DDS changing while setting start/end freqs in Config)
        jsr     set_DDS

m2:     lda     #$0E                    ;cursor on
        jsr     cmnd2LCD
        lda     #$87                    ;init cursor to 100 kHz position
        jsr     cmnd2LCD

        lda     DDS_active
        beq     m2a                     ;bypass these updates and DDS control if DDS not active
        clr     iCalc_flag              ;enables Tone to be set in Do_Calcs (MakeTone in drivers.asm)
        jsr     Do_Calcs                ;Calculate SWR,Z,R,X - display as necessary.
                                        ; This is a pre-m3 mainloop call to set display and tone before
                                        ; background (quiet dial) calcs are performed (which don't set Tone
                                        ; or display SWR.)
        lda     Tone_Freq_min           ;Recover min Tone_Freq from Scan ("Check4min")
        sta     Tone_Freq               ;and set it.

m2a:    lda     #6
        sta     sens_sav                ;save entered encoder Sensitivity value
        sta     sens_cw                 ;set CW sensitivity control
        sta     sens_ccw                ;set CCW sensitivity control

             ;Check pushbuttons for exit

m3:     brset   ModePB,portH,m3a        ;if Mode PB pressed
        jmp     pd_exit                 ;  then exit Process_Dial
m3a:    brset   ConfigPB,portG,m3c      ;if Config PB pressed
        jmp     pd_exit                 ;  then exit Process_Dial
m3c:    brset   ScanPB,portG,m3d        ;if Scan PB pressed
        jmp     pd_exit                 ;  then exit Process_Dial
m3d:    brset   BandPB,portH,m3e        ;if Band PB pressed
        jmp     pd_exit                 ;  then exit Process_Dial
m3e:    jsr     _ischar
	beq     m3b                     ;if Link Command
	sta	lchar                   ;save it
        jmp     pd_exit                 ;  then exit Process_Dial

             ;Check the encoder

m3b:    mov     portf,temp2             ;else buffer the input
        lda     temp2
        and     #$30                    ;isolate the encoder bits
        sta     enc_new
        eor     enc_old
        bne     m4                      ;change detected

             ;IDLE TIME ... Update SWR, R & X in timer Interrupt Service Routine

        lda     DDS_active
        beq     m3                      ;bypass updates and DDS control if DDS not active

        lda     debug_flag              ;if DEBUG set, then don't update readings continuously
        bne     m3

        jsr     Set_DDS                 ;else, update the DDS chip from current LCD_dat(0:7) readings
        lda     #1
        sta     iCalc_flag              ;allow calcs to be done and displayed in heartbeat ISR
        bra     m3

             ;Change Detected

m4:     jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        clra
        sta     iCalc_flag              ;turn off Calcs from being done in heartbeat ISR
        bclr    3,enc_old               ;change detected ... which way?
        lda     enc_old
        rola
        eor     enc_new
        sta     enc_old
        brclr   5,enc_old,up

             ;encoder turned down (ccw)
down:
        lda     sens_sav
        sta     sens_cw                 ;Reset CW Sensitivity cntr
        dec     sens_ccw
        beq     dwn05                   ;sensitivity timer to zero, process turn
        mov     enc_new,enc_old         ;else, wait for SENS_CCW transitions to occur before continuing
        bra     m3

dwn05:  brset   3,temp2,dwn1
        ldhx    cursor_index            ;Enc_PB pressed ... adjust cursor over digit
        txa                             ;index in A
        cmp     #$1                     ;already at list end? (i.e., under 1MHz digit?)
        beq     dwn2                    ;yes, don't move it any further
        aix     #-1                     ;else move it left one position
dwn2:   sthx    cursor_index
        lda     digit_posn,X            ;get LCD digit address
        jsr     Cmnd2LCD                ;put cursor there
        mov     enc_new,enc_old
        jmp     m2a                     ;go wait for encoder change

dwn1:   ldhx    cursor_index            ;point to digit to decrement
        jsr     Bcd_Dec                 ;decrement the BCD digit currently pointed to
        jsr     Mult                    ;calc new frequency
        jsr     Write_Data              ;update LCD
        tst     DDS_active
        beq     down3                   ;bypass DDS control if flag not set
        jsr     Set_DDS                 ;update the DDS chip
        jsr     Do_Calcs                ;Calculate SWR,Z,R,X - display as necessary.  Also do L, C, V if in that mode.
down3:  mov     enc_new,enc_old
        jmp     m2a                     ;go wait for encoder change

             ;encoder turned up (cw)
up:
        lda     sens_sav
        sta     sens_ccw                ;Reset CCW Sensitivity cntr
        dec     sens_cw
        beq     up05                    ;sensitivity timer to zero, process turn
        mov     enc_new,enc_old         ;else, wait for SENS_CCW transitions to occur before continuing
        jmp     m3

up05:   brset   3,temp2,up1
        ldhx    cursor_index            ;Enc_PB pressed ... adjust cursor over digit
        aix     #1
        txa                             ;index in A
        cmp     #$7                     ;at list end?
        tax
        bne     up2                     ;no, continue
        aix     #-1                     ;yes, keep cursor at right extreme
up2:    sthx    cursor_index
        lda     digit_posn,X            ;get LCD digit address
        jsr     Cmnd2LCD                ;put cursor there
        mov     enc_new,enc_old
        jmp     m2a                      ;go wait for encoder change

up1:    ldhx    cursor_index            ;point to digit to decrement
        jsr     Bcd_Inc                 ;increment the BCD digit currently pointed to
        jsr     Mult                    ;calc new frequency
        jsr     Write_Data              ;update LCD
        tst     DDS_active
        beq     up3                     ;bypass DDS control if flag not set
        jsr     Set_DDS                 ;update the DDS chip
        jsr     Do_Calcs                ;Calculate SWR,Z,R,X - display as necessary.  Also do L, C, V if in that mode.
up3:    mov     enc_new,enc_old
        jmp     m2a                     ;go wait for encoder change

             ; Exit Process_Dial (one of the pushbuttons pressed)
pd_exit:
        clra
        sta     iCalc_flag              ;turn off Calcs from being done in heartbeat ISR
        rts

Man_Line1_msg:   db 'SWR ',0
digit_posn:      db $84,$85,$87,$88,$89,$8B,$8C  ;digits 10MHz to 10Hz (omit comma and decimal pos'ns)

;****************************************************************************
; Bcd_Inc -- BCD increment subroutine
;
; This routine does a binary coded decimal increment to the values in LCD_Dat.
; On entry, the X register contains the offset (0-6) of the LCD_Dat digit to be incremented.
; The max freq allowed is 30MHz.

Bcd_Inc:
        sthx    start_index    ;save the starting index (points to selected digit)
BCD_I:  lda     LCD_dat+0
        cmp     #3             ;is ms digit already at max?
        beq     exit2          ;yes, just exit without incrementing
        lda     LCD_dat,X      ;get the digit to increment
        add     #1
        sta     temp1
        add     #6
        bhcs    carry          ;the add went past 9
        lda     temp1          ;else just store char and exit
        and     #$0F
        sta     LCD_dat,X
        bra     exit2

carry:  clr     LCD_dat,X      ;clear the current digit
        aix     #-1            ;point to next higher digit
        tstx                   ;pointing to ms digit?
        bne     BCD_I          ;no, just increment & test again
        lda     LCD_dat,X      ;yes, is ms digit already 2 (29.9999MHz)
        cmp     #2
        bne     BCD_I          ;no, increment like normal

        ldhx    #0             ;yes, set display to max value from first digit to start index
        lda     #3             ;max = 30 MHz
set_max:
        sta     LCD_dat,X
        txa                    ;current pointer now in A
        cmp     start_index+1  ;compare to X reg at saved start_index
        beq     exit2          ;then done, just finished max'ing to point of start index
        tax                    ;current pointer back in X
        aix     #1             ;point to next char to the right
        clra                   ;clear data for next char max setting
        bra     set_max

exit2:  rts


;***************************************************************************
; BCD_DEC -- BCD decrement subroutine
;
; This routine does a binary coded decimal decrement to the values in LCD_Dat.
; On entry the INDEX variable must point to the LCD_Dat digit to be decremented,
; INDEX variable = 0 (10mHz) through 6 (10Hz)

BCD_DEC:
        sthx    start_index      ;save the starting index (points to selected digit)
BCD_d1: lda     LCD_dat,X        ;get the digit to increment
        sub     #1
        bcs     carry2           ;borrow occured
        sta     LCD_dat,X        ;no borrow, just store & continue
        bra     exit

carry2: lda     #9
        sta     LCD_dat,X        ;set current digit = 9
        aix     #-1              ;point to next higher digit
        tstx                     ;pointing to ms digit?
        bne     BCD_D1           ;no, ripple upward through next digit
        lda     LCD_dat,X        ;yes, is ms digit already 0?
        tsta
        bne     BCD_D1           ;no, decrement it like others

        ldhx    start_index      ;else, set the display to min value
set_min:                         ;starting at initial digit & going upward
        clr     LCD_dat,X
        tstx                     ;at ms digit? (i.e., pointer X = 0?)
        beq     exit
        aix     #-1              ;point to next higher digit
        bra     set_min

exit:   rts


;***************************************************************************
; WRITE_DATA -- Write contents of LCD_dat+0 through LCD_dat+6 to the LCD

Write_Data:
        jsr        Busy_Check
        lda        #84h               ;LCD position for first Digit
        jsr        cmnd2LCD

        jsr        Busy_check
        lda        LCD_Dat+0
        ora        #$20
        cmp        #$20               ;is digit a zero?
        beq        wd2                ;yes, leave A as space char and go display it
        ora        #$10               ;else, turn char to ascii
wd2:    jsr        data2LCD           ;Send 10MHz digit

        jsr        Busy_check
        lda        LCD_Dat+1
        ora        #$30
        jsr        data2LCD           ;Send 1MHz digit

        jsr        Busy_check
        lda        #','
        jsr        data2LCD           ;Send a comma

        jsr        Busy_check
        lda        LCD_Dat+2
        ora        #$30
        jsr        data2LCD           ;Send 100KHz digit

        jsr        Busy_check
        lda        LCD_Dat+3
        ora        #$30
        jsr        data2LCD           ;Send 10KHz digit

        jsr        Busy_check
        lda        LCD_Dat+4
        ora        #$30
        jsr        data2LCD           ;Send 1KHz digit

        jsr        Busy_check
        lda        #'.'
        jsr        data2LCD           ;Send a period

        jsr        Busy_check
        lda        LCD_Dat+5
        ora        #$30
        jsr        data2LCD           ;Send 100Hz digit

        jsr        Busy_check
        lda        LCD_Dat+6
        ora        #$30
        jsr        data2LCD           ;Send 10Hz digit

        ldhx       cursor_index
        lda        digit_posn,X       ;put cursor back under proper digit
        jsr        cmnd2LCD

        rts

;*****************************************************************************
; MULT -- 7 Digit BCD by 32 bit binary mulitply
;
; The 7 BCD digits stored in LCD_dat are each multiplied by a precalculated digit
; weight, (Digit weight assume a 100 MHz DDS clock). The sum of all 7
; digits times their weight is stored in the DDS_wN registers.

Mult    clr         temp1
        clr         DDS_w4
        clr         DDS_w3
        clr         DDS_w2
        clr         DDS_w1
        clr         DDS_w0

m10:    mov         #$AD,intacc1+3
        mov         #$01,intacc1+2
        clr         intacc1+1
        clr         intacc1+0
        mov         LCD_Dat+6,OP2
        bra         m_go

m100:   mov         #$C7,intacc1+3
        mov         #$10,intacc1+2
        clr         intacc1+1
        clr         intacc1+0
        mov         LCD_Dat+5,OP2
        bra         m_go

m1K:    mov         #$C6,intacc1+3
        mov         #$A7,intacc1+2
        clr         intacc1+1
        clr         intacc1+0
        mov         LCD_Dat+4,OP2
        bra         m_go

m10K:   mov         #$B9,intacc1+3
        mov         #$8D,intacc1+2
        mov         #$06,intacc1+1
        clr         intacc1+0
        mov         LCD_Dat+3,OP2
        bra         m_go

m100K:  mov         #$37,intacc1+3
        mov         #$89,intacc1+2
        mov         #$41,intacc1+1
        clr         intacc1+0
        mov         LCD_Dat+2,OP2
        bra         m_go

m1M:    mov         #$29,intacc1+3
        mov         #$5C,intacc1+2
        mov         #$8F,intacc1+1
        mov         #$02,intacc1+0
        mov         LCD_Dat+1,OP2
        bra         m_go

m10M:   mov         #$9A,intacc1+3
        mov         #$99,intacc1+2
        mov         #$99,intacc1+1
        mov         #$19,intacc1+0
        mov         LCD_Dat+0,OP2
        bra         m_go

m_go:   mov         #4,temp2

digit_loop:
        clc
        ror         OP2
        bcc         noCarry
        jsr         Add32

noCarry:
        clc
        rol         intacc1+3
        rol         intacc1+2
        rol         intacc1+1
        rol         intacc1+0
        dbnz        temp2,digit_loop
        inc         temp1
        lda         temp1
        cmp         #1
        beq         m100
        cmp         #2
        beq         m1K
        cmp         #3
        beq         m10K
        cmp         #4
        beq         m100K
        cmp         #5
        beq         m1M
        cmp         #6
        beq         m10M
        rts

;***************************************************************************
; ADD32 -- 32 bit add
;
; The value in registers intacc1(0:3) is added to the current 32 bit value
; in DDS_w0 [MSB] thru DDS_w4 [LSB]

Add32:  lda     DDS_w4        ;process LS byte first
        add     intacc1+3
        sta     DDS_w4
        bcc     add1
        inc     DDS_w3
        bne     add1
        inc     DDS_w2
        bne     add1
        inc     DDS_w1

add1:   lda     DDS_w3
        add     intacc1+2
        sta     DDS_w3
        bcc     add2
        inc     DDS_w2
        bne     add2
        inc     DDS_w1

add2:   lda     DDS_w2
        add     intacc1+1
        sta     DDS_w2
        bcc     add3
        inc     DDS_w1

add3:   lda     DDS_w1        ;process MS byte
        add     intacc1+0
        sta     DDS_w1
        rts
