;*********************************************************************************
;*********************************************************************************
;                   AF908 PB1 Function
;      this is the principal entry point for AF908
;    It is entered directly on powerup and later when PB1 is pressed
;    Options offered:
;      Download        - Download DSP program from PC
;      Run             - Run DSP program, 1st option displayed
;      Run w/PC        - Run DSP program control from PC
;      Auto Load       - Select DSP rpogram to automatically load on powerup
;      NO Auto Load    - Cancel AutoLoad
;      Save Settings   - Save current Setting to EEPROM
;      Default Settings - Restore EEPROM to orignal default settings
;      Exit            - Rerturn to "Select Function" option
;
;      The bulk of Run and Run w/PC processing is done in DSP_run.asm
;          in routines:  DSP_Run and DSP_PCcmd
;      The rest of the options are processed here
;
;       December 2004 - January 2005 written by Steve Holton N1NB
;*********************************************************************************
;*********************************************************************************

PB1_func:
        brclr   PB1,porth,*            ;await button release
        jsr     ResetAutoTimers        ;Reset LiteOff timer since user touched controls
        jsr     wait100ms
        jsr     LCD_Clear
        ldhx    #PB1_msg
        jsr     disp_msg               ;display "Sel DSP Action" on 1st line
        ldhx    #PB1CmndTable          ;set up pointer to top level Command Table
        sthx    CmndTabStart
        ldhx    #PB1MsgTable           ;set up pointer to top level Msg Table
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

PB1_msg:        fcb     'Sel. DSP Action',0

;   PB1 Top Level Msg Table
PB1MsgTable:
        fcb     'Download  ',0        ;CW item
        fcb     'Run ',0               ;1st item displayed
        fcb     'Run w/PC',0          ;CCW item
        fcb     'Auto Load',0
        fcb     'NO Auto Load',0
        fcb     'Save Settings',0
        fcb     'Default Settings',0
        fcb     'Exit',0
        fcb     0                      ;end of list character
;   PB1 Top level Command table
PB1CmndTable:
        fdb     PB1_Download
        fdb     PB1_Run
        fdb     PB1_RunPC
        fdb     PB1_Startup
        fdb     PB1_NoStartup
        fdb     PB1_Save
        fdb     PB1_Default
        fdb     PB1_Exit

             ; PB1 Top level Command Processors

PB1_Download:           ; Download DSP program image from PC to SEEPROM DSPterm
        jsr     DoDSPdownload
        jmp     ExitPB1

PB1_M_Download:           ; Manual Download DSP program image via serila terminal pgm
        jsr     DoDSP_Mdownload
        jmp     ExitPB1

PB1_Run:                ; Load and run DSP program
                        ; If DSP908 or ASP provide explcit runtime support
        lda     #0                     ; DSP_run will set if actually running
        sta     DSPisdsp908            ; clear dsp908 flag
        sta     DSPisASP               ; clear ASP flag
        jsr     DSP_run                ; load and start DSP porgram
        bne     run3                   ; run failed
        lda     DSPisdsp908            ; if pgm is dsp908
        beq     run1
        jsr     DSP_dsp908             ; go do explicit run support
        bra     ExitPB1
run1:   lda     DSPisASP               ; if pgm is ASP
        beq     run2
        jsr     DSP_ASP                ; go do explicit run support
        bra     ExitPB1
run2:   jsr     PB1_Simple             ; do simple - wait for end
run3:   jmp     ExitPB1

PB1_RunPC:                        ; Run DSP program with PC controlling
        lda     #0                     ; DSP_run will set if actually running
        sta     DSPisdsp908            ; clear dsp908 flag
        sta     DSPisASP               ; clear ASP flag
        jsr     DSP_run
        jsr     DSP_PCcmd
        jsr     DSPreset_banner         ; stop DSP via reset and return
        bra     ExitPB1

PB1_Default:                        ; resets EEPROM to default AF908 settings
        clra                            ; clearing sig will cause
        sta     DSP_EEPROMsig
        jsr     chkDSP_EEPROMsig        ; defaults to be set here (EEPROM Page2)
        bra     ExitPB1

PB1_Startup:                       ; Set AutoLoad for selected program
        jsr     DSP_pgm_select          ; go let user select a program
        psha
        jsr     getDIRentry             ; get h:x pointing to Dir entry
        aix     #PGMNAME                ; yes this is zero now but for future safety
        sthx    from_addr
        ldhx    #DSPdefault_name
        sthx    to_addr
        jsr     StringCopy              ; copy name
        lda     1,sp                    ; get copy of index to entry
        add     #1                      ; saved as entry offset+1
        sta     DSPdefault_load         ; save it
        ldhx    #{$900+DSPdefault_load-Start_P2_data}
        sthx    to_addr
        ldhx    #DSPdefault_load
        sthx    from_addr
        bclr    BusyLED,portc           ;turn on Busy LED
        lda     #9
        sta     EEbyte_count           ; write index and name to EEPROM
        jsr     ec2                    ; do write to EEPROM
        bset    BusyLED,portc           ;turn off Busy LED
        bra     ExitPB1

PB1_NoStartup:                   ; "cancel" Auto Load
        bclr    BusyLED,portc              ;turn on Busy LED
        clra
        sta     DSPdefault_load
        ldhx    #{$900+DSPdefault_load-Start_P2_data} ;address of this setting
        jsr     Write_EEPROM_Byte          ; save this setting only
        bset    BusyLED,portc              ;turn off Busy LED
        bra     ExitPB1

PB1_Save:                        ; save AF908 settings (EEPROM page 2)
        jsr     Say_BusySave        ; turn on/off busy LED, msg on LCD & save P2
        bra     ExitPB1

PB1_Exit:                        ; Break out of PB1 Level go to very top level
        brclr   ENC_pb,portf,*        ;wait here until Encoder PB released
        jmp     StartMainAF           ; go very top AF908 Select Function

PB1_End:
        jsr     DSPreset_banner       ; stop DSP via reset
ExitPB1:
        brclr   ENC_pb,portf,*        ;wait here until Encoder PB released
        jmp     PB1_func              ; go back to PB1 top level menu

PB1_simple:                     ; Run "simple" DSP program - only option
                                ;  is End DSP program
        jsr     LCD_Clear
        ldhx    #PB1_msg
        jsr     disp_msg                ;display "Sel DSP Acrtion" on 1st line
        ldhx    #simpleCmndTable        ;set up pointer to Simple CMD Table
        sthx    CmndTabStart
        ldhx    #simpleMsgTable           ;set up pointer to Simple MsgT able
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

simpleCmndTable:
        fdb     PB1_End

simpleMsgTable:
        fcb     'End DSP pgm  ',0
        fcb     0                       ;end of list characte

;--------------  Support Subroutines
DoDSP_Mdownload:
        lda     #7                   ; byte count for what we read here
        psha                         ; save for use as counter
        sta     ManualDl             ; set flag for manual DL
        jsr     SPI_Init            ; initialize SPI module
        ldhx    #manual_msg
        jsr     _puts
        ldhx    #SPI_buffer1
M_DL_1:
        jsr     _getchne
        sta     ,x
        aix     #1
        dec     1,sp
        bne     M_DL_1
        ais     #1                  ; clear counter off stack
        lda     SPI_buffer1+6
        add     #3                  ; real byte count is 3 more than in image
        sta     SPI_count+1         ; save true byte count
        sta     SPI_Ptr+1
        lda     SPI_buffer1+5
        adc     #0
        sta     SPI_count
        sta     SPI_Ptr
                                    ; compute nearest page boundary
                                    ; where DSP programs will fit at top of SEEPROM
        clra                        ;  $0000 == 64K == Top
        sub     SPI_count+1         ;  subtract low byte
        and     #$80                ; round down to page boundary
        sta     DSP_offset+1
        sta     SPI_offset+1        ; pointer for receive
        clra
        sbc     SPI_count           ; subtract high byte
        sta     SPI_offset          ; pointer for receive
        sta     DSP_offset          ; remember pointer for future
        lda     #'a'                 ; type a
        jsr     MdataRcvCommon
        clra                        ; clear manual DL flag
        sta     ManualDL
        bra     finishDL

manual_msg:
        fcb     CR,LF,'***waiting for DSP pgm ***',CR,LF,0
DoDSPdownload:    ;  download program file to SEEPROM




        lda     #'a'                 ; type a

;;       this is like the fron end of SPI_DataRcvNC
        psha                        ; save type for now
        jsr     SPI_Init            ; initialize SPI module

        ldhx    #SPImsg_rcv         ; Send 'zZ'for Receive
        jsr     _puts
        pula                        ; recover type
        jsr     _putch              ; send type from A (via stack)
        ldhx    #SPImsg_NC          ; Send '??'to get count
        jsr     _puts

        jsr     getCount            ; get count from PC
                                    ; compute nearest page boundary
                                    ; where DSP programs will fit at top of SEEPROM
        clra                        ;  $0000 == 64K == Top
        sub     SPI_count+1         ;  subtract low byte
        and     #$80                ; round down to page boundary
        sta     DSP_offset+1
        sta     SPI_offset+1        ; pointer for receive
        clra
        sbc     SPI_count           ; subtract high byte
        sta     SPI_offset          ; pointer for receive
        sta     DSP_offset          ; remember pointer for future

        jsr     dataRcvCommon       ; no pickup with main line of SPI_DataRcvNC

;--                                   on return last page still in buffer
;        jsr     SPI_DeviceReady     ; read back last page
;        ldhx    #SPI_buffer1
;        sthx    SPI_RamAddr
;        mov     #$ff,SPI_Offset
;        mov     #$80,SPI_Offset+1
;        ldhx    #SPI_Args
;        lda     #$80
;        jsr     SPI_Read
finishDL:
         lda     DSP_offset         ; put DSP_offset in last 2 bytes of SEEPROM
         sta     SPI_buffer1+$7E     ; May overlay check sum (OK for now) CKsum not used
         lda     DSP_offset+1
         sta     SPI_buffer1+$7F
         jsr     SPI_DeviceReady     ; wait for any write to complete
         ldhx    #SPI_buffer1
         sthx    SPI_RamAddr
         mov     #$ff,SPI_Offset
         mov     #$80,SPI_Offset+1
         ldhx    #SPI_Args
         jsr     SPI_WriteEnable
         lda     #$80
         jsr     SPI_WritePAge
        rts

        ; check EEPROM is not initialized - set with default starting values
chkDSP_EEPROMsig:                   ; check if eeprom page 2 has been written
        lda     #4                  ; previously with DSP data
        ldhx    #DSP_EEPROMsig
        sthx    from_addr
        ldhx    #DSP_sig
        sthx    to_addr
        jsr     StringNCompare      ; check for DSP signature
        bne     EEPROMsigfail
        rts
EEPROMsigfail:                       ; no so set default and write eeprom
        ldhx    #DSP_sig
        sthx    from_addr
        ldhx    #DSP_EEPROMsig
        sthx    to_addr
        lda     #4
        jsr     MemCopy                  ; set signature
        clra
        sta     DSPdefault_load          ; set global value
        jsr     dsp908_EEPROM_Defaults   ; set dsp908 values
        jsr     ASP_EEPROM_Defaults      ; set ASP values
        jsr     Say_BusySave        ; turn on/off busy LED, msg on LCD & save P2
        rts

Say_BusySave:                 ; turn on busy, save EEPROM p2, turn off busy
        bclr    BusyLED,portc           ;turn on Busy LED
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #Save_Settings
        jsr     disp_msg                ;"Saving settings.."
        jsr     Put_EEPROM_P2
        bset    BusyLED,portc           ;turn off Busy LED
        rts
