;*********************************************************************************
;*********************************************************************************
;                       PB4 = Config Process for Micro908
;  made save and defaults just go to AF908 routines in PB1.asm
;  2/15/2005 Steve Holton N1NB
;*********************************************************************************
;*********************************************************************************

PB4_func:
        brclr   PB4,portG,*             ;wait until PB4 released
        jsr     ResetAutoTimers         ;Reset LiteOff timer since user touched controls
        tone_off                        ;turn tone off while in Config menu
        jsr     LCD_Clear
        ldhx    #Config_msg
        jsr     disp_msg                ;display "Setup Selection" on 1st line

        ldhx    #ConfigCmndTableAF        ;set up pointer to ConfigCmndTable
        sthx    CmndTabStart
        ldhx    #ConfigMsgTableAF         ;set up pointer to ConfigMsgTable
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection


;-----------------------------
; AFSaveUserSettings  -- Saves users settings to EEPROM (DSP P2).

AFSaveUserSettings:
                        ; save AF908 settings (EEPROM page 2)
        jsr     Say_BusySave        ; turn on/off busy LED, msg on LCD & save P2
        rts


;--------------------------
;   Config JUMP TABLE

ConfigCmndTableAF:
        fdb     Monitor
        fdb     ExitConfig
        fdb     AFDispBatt
        fdb     AFDefault_Settings
        fdb     PB1_m_Download          ; actual code is located in PB1.asm
        fdb     TurnLightOn
        fdb     TurnLightOff
        fdb     LightOffTime
$IF  AA_AF_Both
        fdb     RunAA908
$ENDIF
;---------------------------
;   Config MESSAGE TABLE

ConfigMsgTableAF:
        fcb     'Software Load',0       ;CW direction
        fcb     'Exit',0                ;1st message displayed
        fcb     'Battery Voltage',0     ;CCW direction
        fcb     'Default_Settings',0
        fcb     'Manual Download',0
        fcb     'Turn Lite On',0
        fcb     'Turn Lite Off',0
        fcb     'Light Off Time',0
$IF  AA_AF_Both
        fcb     'Run AA908',0
$ENDIF
        fcb     0


$IF  AA_AF_Both
RunAA908:
        sei             ; mask interrupts
        jmp     Start
$ENDIF

;********************************************************************************
; Default Settings
; Returns user-customizable settings to default values in EEPROM

AFDefault_Settings:
                        ; resets EEPROM to default AF908 settings
        clra                            ; clearing sig will cause
        sta     DSP_EEPROMsig
        jsr     chkDSP_EEPROMsig        ; defaults to be set here (EEPROM Page2)
        jmp     PB4_func                ;and exit



;******************************************************************************
; DispBatt -- Display the battery voltage

AFDispBatt:
        jsr     LCD_Clear
        ldhx    #Batt_msg               ;"Battery Voltage:"
        jsr     Disp_Msg
             ;Read battery voltage and display it
AFdispb1: jsr     Calc_Batt
        lda     #$89                    ;point to mid-1st line
        jsr     Cmnd2LCD
        jsr     dispb2
        brset   ENC_PB,portf,AFdispb1   ;do again if Encoder PB not pressed
        brclr   ENC_PB,portf,*          ;wait here until Encoder PB released
        jmp     PB4_Func                ;then exit

$IFNOT  AA_AF_Both

             ;Display Battery Voltage
dispb2:
        lda     bcd100000
        add     #$30
        jsr     data2lcd
        lda     bcd10000
        add     #$30
        jsr     data2lcd
        lda     #'.'
        jsr     data2lcd
        lda     bcd1000
        add     #$30
        jsr     data2lcd
        lda     bcd100
        add     #$30
        jsr     data2lcd
        rts

Batt_msg:  fcb  'Battery:',0

             ;Save all Config settings in EEPROM if EE_dirty flag is set, then exit
exitconfig:
        lda     EE_dirty
        beq     cp_exit                 ;just exit if no settings changes made
        bclr    BusyLED,portc           ;turn on Busy LED
        lda     #$C0
        jsr     Cmnd2LCD                ;write to 2nd line next
        ldhx    #Save_Settings
        jsr     disp_msg                ;"Saving settings.."
        jsr     AFSaveUserSettings
        bset    BusyLED,portc           ;turn off Busy LED
        brclr   ENC_PB,portf,*          ;wait here until Encoder PB released
cp_exit:
        clra
        sta     EE_dirty
        jmp     StartMainAF

Config_msg:     fcb     'Config Selection',0
Save_Settings:  fcb     '    (saving)    ',0  ;SMH Added 1 more blank to clear
                                              ;SMH line completely


;********************************************************************************
;TurnLightOn

TurnLightOn:
        jsr     TurnLiteOn
        lda     #$FF
        sta     LiteOnFlag              ;indicates that LCD backlight should be turned on when checked
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     PB4_func                ;and exit

;********************************************************************************
;TurnLightOff

TurnLightOff:
        jsr     TurnLiteOff
        clra
        sta     LiteOnFlag              ;indicates that LCD backlight should be turned off when checked
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     PB4_func                ;and exit

;********************************************************************************
; Light Off Time -- Set the number of minutes until backlight turned off

LightOffTime:
        jsr     LCD_clear
        ldhx    #LiteOffMsg1             ;"Set LiteOff Time"
        jsr     Disp_Msg
        lda     lite_units
        sta     enc_units
        lda     lite_tens
        sta     enc_tens
        jsr     GetDialReading
        sta     LiteOffMins              ;save binary result
        lda     enc_units                ;and save ascii results
        sta     lite_units
        lda     enc_tens
        sta     lite_tens
        lda     #1
        sta     EE_dirty                ;set flag to save settings upon exit
        jmp     PB4_func                ;and exit

LiteOffMsg1:       fcb  'LiteOff Time',0


             ;----------------------------------------------------------------
             ; GetDialNumbers -- Get dial reading 00-99
             ;                   return binary in A
             ;                   and ascii in enc_tens, enc_units
             ;----------------------------------------------------------------

GetDialReading:
        jsr     ClearLCDLine2
        lda     #$C0                     ;point to 2nd line on LCD
        jsr     Cmnd2LCD
        ldhx    #EncoderMsg
        jsr     Disp_Msg
        jsr     Update_Digits            ;display current numbers for enc_tens,enc_units
se1:    lda     #4                       ;dial sensitivity
        jsr     Read_Encoder             ;process the encoder turns
        bcs     se3                      ;exit if ENC PB pressed
        bne     se2
        jsr     increase_digit_display   ;CW turn detected
        bra     se1
se2:    jsr     decrease_digit_display   ;CCW turn detected
        bra     se1

             ;Convert (enc_tens:enc_units) to binary in A

se3:    lda     enc_tens
        and     #$0F                     ;strip off upper ascii nibble
        sta     intacc1+1                ;and place into intacc1(0:1)
        clr     intacc1+0
        mov     #10,intacc2+1            ;load 10 into intacc2(0:1)
        clr     intacc2+0
        jsr     umult16                  ;multiply tens by 10
        lda     enc_units
        and     #$0F                     ;strip off upper ascii nibble
        add     intacc1+3                ;add to multiplication result and return in A
        brclr   ENC_PB,portf,*           ;wait here until Encoder PB released
        rts

EncoderMsg:
        fcb     'Minutes = 00',0

             ;-------------------------
             ;encoder turned down (ccw)

Decrease_Digit_Display:
        lda     enc_units
        deca                          ;decrement units digit
        sta     enc_units
        cmp     #$2F                  ;units at min?
        beq     dwn1a                 ;yes, roll down
        bra     dwn1b                 ;no,just continue

dwn1a:  lda     #$39
        sta     enc_units             ;roll down units to 9
        lda     enc_tens
        deca                          ;and decr tens
        sta     enc_tens
        cmp     #$2F                  ;tens at min?
        beq     dwn1c                 ;yes, reset cntr to 00
        bra     dwn1b                 ;no, just continue

dwn1c:  lda     #$39
        sta     enc_units             ;reset tens to zero
        sta     enc_tens              ; and units to zero

dwn1b:  jsr     update_digits         ;update counter digits
        rts
             ;----------------------
             ;encoder turned up (cw)

Increase_Digit_Display:
        lda     enc_units             ;increment units digit
        inca
        sta     enc_units
        cmp     #$3A                  ;units at max?
        beq     up1a                  ;yes, roll over
        bra     up1b                  ;no, just continue

up1a:   lda     #$30
        sta     enc_units             ;roll over units to zero
        lda     enc_tens              ;and bump tens
        inca
        sta     enc_tens
        cmp     #$3A                  ;tens at max?
        beq     up1c                  ;yes, reset cntr to 00
        bra     up1b                  ;no, just continue

up1c:   lda     #$30
        sta     enc_units             ;reset tens to zero
        sta     enc_tens              ; and units to zero

up1b:   jsr     update_digits         ;update counter digits
        rts
             ;--------------
             ;update Digits
update_digits:
        lda     #$CA                  ;point to tens digit on 2nd line
        jsr     Cmnd2LCD
        lda     enc_tens              ;send the tens digit there
        jsr     Data2LCD
        lda     #$CB                  ;point to units digit on 2nd line
        jsr     Cmnd2LCD
        lda     enc_units             ;send the units digit there
        jsr     Data2LCD
        rts


;********************************************************************************
; Debug Monitor
Monitor:
;;;__start:
;;;_bootreset:
                ; Launch the bootloader from power-on reset.

        sei                             ; disable all interupts
        ldhx    #$450    ;;;  <===> ldhx  #_mon_stack+1      ; initialize
        txs                             ;  the stack pointer
        fcb     $CD,$f2,$00  ;;; <===> jsr     PLLset ; change bus speed

                ; Initialize the SCI.

        mov      #init_scbr,scbr        ; initialize baud rate
        mov      #init_scc1,scc1        ; initialize SCI Control Register 1
        mov      #init_scc2,scc2        ; initialize SCI Control Register 2
        mov      #init_config1,config1  ; init Configuration Register 1

                ; Test Port F7 for "Monitor Jumper"
                ;   Entry to Monitor  = jumper OFF ==> F7 = 1
                ;   Entry to User Pgm = jumper ON  ==> F7 = 0

        mov     #$40,ddrf               ;set portf bits 7,5,4,3,2,1,0 as inputs, bit 6 (LED) as output
        mov     #$B8,prtfpu             ;assign pullups to Port F bits 7,5,4,3
        bclr    6,portf                 ;turn LED on continuously while in monitor
 ;;;    brclr   7,portf,MonStart        ;go start Monitor if "Monitor Jumper" is in place
        jmp     $ef2a                    ; $EF2A == MonStart  DON'T Test it just do it!!!


$ENDIF

