;*********************************************************************************
;*********************************************************************************
;                          PB1 Function: DSP Load
;*********************************************************************************
;*********************************************************************************

; DSP  simple serial passthru
;  "wired" to Push Button 5
; On pushing PB5 a start msg is issued and the DSP ports are configured
; The SCI is disabled and it's ports configured
; the DSP is then reset by bringing /RST low for 100ms
; then pass-thru loop starts
; When PB pushed again the loop ends, and the SCI is restored and the test exits

PB5_func:
        brclr   PB5,portG,*        ;wait until PB released
        jsr     wait100ms
        jsr     LCD_Clear
        ldhx    #PB5_msg
        jsr     disp_msg           ;display "DSP Load" on 1st line
        ldhx    #PB5CmndTable          ;set up pointer to DiagCmndTable
        sthx    CmndTabStart
        ldhx    #PB5MsgTable           ;set up pointer to DiagMsgTable
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr                ;counter used as index into Msg Table
        jmp     get_dial_selection

PB5_msg:  fcb  'Sel. Direct Mode',0
PB5CmndTable:
        fdb     Direct_NOreset
        fdb     Direct_reset

PB5MsgTable:
        fcb     'Direct No Reset',0
        fcb     'Direct w/Reset',0
        fcb     0

Direct_reset:
        lda     #1
        sta     PB5_reset
        bra     do_Direct
Direct_NOreset:
        lda     #0
        sta     PB5_reset
do_Direct:
        jsr     DSP_Direct
        jmp     StartMainAF

DSP_Direct:

        jsr      wait32ms           ; let the serial port drain to be safe

; Set DSP port direction
        bset     DSP_RST,portA     ; set /RST High
        bclr     DSP_TXD,ddrA      ; set DSP's tXd as input to us
        bset     DSP_RXD,ddrA      ; set DSP's RxD as output from us
        bset     DSP_RST,ddrA      ; set DSP's reset as output from us

;         jsr      wait500ms        ; TEST
;         jsr      wait500ms        ; TEST
; Disable SCI
        bclr     ENSCI,scc1        ; clear sci enable

; Set "SCI" port direction
        bclr     RxData,ddrE       ; set RxD as input from PC
        bset     TxData,ddrE       ; set TxD as output to PC

        lda      PB5_reset         ; do reset if selected
        beq      BitPassLoop

; Reset DSP
        bclr     DSP_RST,portA     ; bring /rst low
        jsr      wait100ms
;       jsr      wait500ms               ; TEST
;       jsr      wait500ms             ; TEST
;       jsr      wait500ms              ; TEST
;       jsr      wait500ms             ; TEST
        bset     DSP_RST,portA     ; bring /rst hi again

BitPassLoop:
        brclr   ENC_pb,portF,breakout     ; check for Encoder PB pushed
        brset    DSP_TXD,portA,setTxD     ; check for incoming '1' from DSP
        bclr     TxData,portE             ; if not set '0' to PC
BitPassLoop1:
        brset    RxData,portE,setDSP_RXD  ; check for incoming '1' from PC
        bclr     DSP_RXD,portA            ; if not set '0' to DSP
        bra      BitPassLoop

setTxD: bset     TxData,portE             ; DSP set '1' set '1' to PC
        bra      BitPassLoop1
setDSP_RXD:
        bset     DSP_RXD,portA            ; PC set '1' set '1' to DSP
        bra      BitPassLoop

breakout:                           ; PB5 pushed again to stop passthru
        brclr    PB5,portG,*        ; wait until PB5 released
;        jsr      wait100ms          ; let the serial port drain to be safe

        bclr     DSP_RST,ddrA       ; set DSP's reset as input makes it HiZ
; reinitialize the SCI
        mov      #init_scbr,scbr    ; initialize baud rate
        mov      #init_scc1,scc1    ; initialize SCI Control Register 1
        mov      #init_scc2,scc2    ; initialize SCI Control Register 2

;
        rts



