;**************************************************************************
;   Runtime support for the ASP DSP program
;    provides operational control of the ASP program
;    entered from PB1_Run in PB1.asm after ASP has been loaded and started
;
;    Written February 2005 by Steve Holton N1NB
;**************************************************************************

DSP_ASP:
           ; restore last saved mode and filter parms, gains,taps,betas
        jsr     restoreASP
ASP_top_start:                         ; loops from  here until "End" selected
        jsr     asp_showStatus         ; format and display status line
        ldhx    #ASPtopCmdTable        ; set up pointer to DiagCmndTable
        sthx    CmndTabStart
        ldhx    #DSPtopMsgTable        ;for now we share this with dsp908
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection

;---  ASP Top level messages
;****  Same choices as dsp908 in DSP DSPtopMsgTable that is:
;****DSPtopMsgTable:
;****DSP_End_msg        fcb     'End DSP pgm',0
;****DSP_Change_msg:    fcb     'Change',0    ; 1st item displayed
;****;;;DSP_Query_msg:  fcb     'Query',0     ; CW item
;****                   fcb     0             ; end of list

;.... ASPtop level commands
ASPtopCmdTable:
        fdb     ASP_End
        fdb     ASP_Change
        fdb     ASP_Save
;        fdb     DSP_Query

;************* ASP top level command processors

ASP_End:                    ; End ASP
        clra                         ; show ASP no longer running
        sta    DSPisASP
        jsr    DSPreset_banner       ; stop DSP via reset and return
        rts

ASP_Save:                    ; Save current settings (saves all including dsp908)
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Save_Settings
        jsr     disp_msg
        bclr    BusyLED,portc
        jsr     put_EEPROM_p2
        bset    BusyLED,portc
        jmp     ASP_top_start

ASP_Change:                             ; Change Something in ASP
        ldhx    #ASPchangeCmdTable
        sthx    CmndTabStart
        ldhx    #ASPchangeMsgTable
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection

;-----  Change level MSG table
ASPchangeMsgTable:
        fcb     'Mode',0
        fcb     'Adjust BFO',0
        fcb     'Taps',0
        fcb     0
;-----  Change level CMD table
ASPchangeCmdTable:
        fdb     ASP_mode
        fdb     ASP_changeBFO
        fdb     ASP_taps

;*********** change level command processor for Mode
ASP_mode:                   ; Change Mode
        lda     ASP_status      ; find current mode
        cmp     #6
        blo     ASP_cwMode
        cmp     #9
        blo     ASP_ssbMode
        ldx     #$02            ; index to status & BFO value
        bra     ASP_Modecom
ASP_ssbMode:
        ldx     #$01            ; index to status & BFO value
        bra     ASP_Modecom
ASP_cwMode:
        ldx     #$00            ; index to status & BFO value

ASP_Modecom:               ; Common Mode processing
        clrh               ; h:x used to index into mode & BFO settings for old mode
        sta     ASPcwStatus,x   ; save current status to old mode
        lda     ASP_status2     ; save current status2 Nt/Nr
        sta     ASPcwStatus2,x
        lslx                    ; index times 2 for 16 bit BFO values
        lda     ASP_BFO
        sta     ASPcwBFO,x      ; save current BFO to old BFO probably unneeded
        lda     ASP_BFO+1
        sta     ASPcwBFO+1,x
                                  ; we have saved old mode now find out new mode
        ldhx    #ASPmodeCmdTable
        sthx    CmndTabStart
        ldhx    #ASPmodeMsgTable     ;
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection

;-----  Mode level MSG table
ASPmodeMsgTable:
        fcb     'CW',0
        fcb     'SSB',0
        fcb     'RTTY',0
        fcb     0
;-----  Mode level CMD table
ASPmodeCmdTable:
        fdb     ASP_domode
        fdb     ASP_domode
        fdb     ASP_domode
;---------    Mode level command processor (for all mode choices)
ASP_domode:
        ldx     msg_cntr          ; get index for mode selected
        clrh                      ; into h:x
        lda     ASPcwStatus,x     ; get new mode's status to curent
        sta     ASP_status
        lda     ASPcwStatus2,x
        sta     ASP_status2
        lslx                      ; now index*2 for 16 bit BFO
        lda     ASPcwBFO,x        ; get new mode's BFO to curent
        sta     ASP_BFO
        lda     ASPcwBFO+1,x
        sta     ASP_BFO+1
        jsr     restoreASP        ; set DSP to the new values
        jmp     ASP_top_start

;*********** Change level command processor for Adjust BFO
ASP_changeBFO:
        clr     intacc1
        clr     intacc1+1
        lda     ASP_status      ; determine current mode
        cmp     #5
        blo     ASP_cwBFO
        cmp     #9
        blo     ASP_ssbBFO
        ldx     #$04            ; index to BFO value for RTTY
        bra     ASP_BFOcom
ASP_ssbBFO:
        ldx     #$02            ; index to BFO value for SSB
        bra     ASP_BFOcom
ASP_cwBFO:
        ldx     #$00            ; index to BFO value for CW

ASP_BFOcom:             ; common BFO adjust processing
        clrh            ; h:x has index to BFO for this mode
        pshx                    ; save copy of index for later
        lda     ASPcwBFO,x      ; get current BFO value for mode
        sta     intacc1+2
        lda     ASPcwBFO+1,x
        sta     intacc1+3
        jsr     bin2bcd           ; convert to BCD
        mov     BCD1000,LCD_dat   ; move BCD to dial buffer
        mov     BCD100,LCD_dat+1
        mov     BCD10,LCD_dat+2   ; move BCD to dial buffer
        mov     BCD1,LCD_dat+3
        lda     #4
        sta     AFcursor_end    ; position cursor under 100's digit
        clra
        sta     AFTwo_Digits    ; 4 digit value 0-9999
        lda     #$C0
        jsr     Cmnd2LCD        ;write to 2nd line
        jsr     ClearLCDLine2
        jsr     Process_DialAF  ; Display Current BFO and let user set new value
        ldhx    #lcd_dat
        jsr     BCD4to16Bin     ; convert to binary
        clrh
        pulx                    ; recover index
        lda     intacc1         ; get resulting new BFO value
        sta     ASP_BFO         ; save in current mode BFO
        sta     ASPcwBFO,x      ; and in mode's BFO
        lda     intacc1+1
        sta     ASP_BFO+1
        sta     ASPcwBFO+1,x
        jsr     ASPadjBFO       ; now adjust bfo in dsp to new setting
        lda     #$0C
        jsr     Cmnd2LCD        ;turn off cursor
        jmp     ASP_top_start

;*********** Change level command processor for Taps
ASP_taps:
        ldhx    #ASPdoTapsCmd           ; Now get tap value
        sthx    CmndTabStart
        ldhx    #ASPdoTapsMsg           ; 
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection
ASPdoTapsMsg:
  fcb '31',0,'51',0,'71',0,'91',0,0


;-----  Taps level CMD table
ASPdoTapsCmd:
  fdb ASPdo_taps,ASPdo_taps,ASPdo_taps,ASPdo_taps

;-----  Taps level command processor (common for all values)
ASPdo_taps:
        clrh
        ldx     msg_cntr             ; get index for tap
        lda     ASPdoTapsValues,x    ; get corresponding command value
        sta     ASPtaps              ; save for restore at startup
        jsr     ASP_cmd              ; send command
        lda     ASP_status2          ; setting taps turns on denoiser
        bit     #$02                 ; if denoiser is on then we're done
        bne     A_T_E        
        lda     #'d'                 ; if not we need to
        jsr     ASP_cmd              ; send command to turn it off 
A_T_E:  jmp     ASP_top_start

ASPdoTapsValues:
  fcb 'qwas'            ; taps one byte command 'q', 'w', 'a', 's'


;;**************  ASP support subroutines

asp_showStatus:                 ; clear status line, format new status & display
        ldhx    #DSP_status_msg
        sthx    from_addr
        lda     #16                    ; clear status msg
        ldx     #' '
        jsr     MemSet
        lda     #$80                   ; make sure we're on 1st line of LCD
        jsr     Cmnd2LCD
        jsr     ASPformat_status       ; format status in buffer
        ldhx    #DSP_status_msg
        jsr     disp_msg               ;display "DSP status" on 1st line
        rts

;--               sets DSP to current status reflected in
;--                ASP_Status, ASP_status2, ASP_BFO, ASPTaps
restoreASP:
        lda     #'r'             ; ASP Reset: BFO, denoiser, autonotch, CW
        jsr     ASP_cmd         ; send command
        lda     ASP_Status      ; get status
        and     #$0F            ; get filter in low nibble
        ora     #$30            ; make if ascii digit which ==> command
        jsr     ASP_cmd         ; send command
        jsr     ASPadjBFO       ; adjust the BFO
        lda     ASPTaps         ; set to last taps value
        jsr     ASP_cmd         ; send command
        lda     #'d'            ; setting taps turns on denoiser
        jsr     ASP_cmd         ; send command to turn it off
        lda     ASP_Status
        cmp     #6              ; for CW no notch or denoiser
        blo     rex             ; ignore state
        lda     ASP_status2
        cmp     #$01            ; Notch?
        beq     rNo
rTN:    cmp     #$02            ; deNoiser?
        beq     rdN
        rts
rNo:    lda     #'n'
        jsr     ASP_cmd         ; send command toggle Notch
        lda     ASP_status2
        bra     rTN
rdN:    lda     #'d'
        jsr     ASP_cmd         ; send command toggle deNoiser
rex:    rts

;---             set BFO for current filter to value in ASP_BFO
;                  resets BFO to zero offset from center
;                  computes difference new value to zero offset value for mode
;                  adjusts BFO by that ammount
ASPadjBFO:
        lda     #'c'            ; reset BFO Command to offset of zero
        jsr     ASP_cmd         ; send command
        lda     ASP_status      ; get current status
        cmp     #6
        blo     BFO_cw
        cmp     #9
        blo     BFO_ssb
        ldhx    #rttyCenter     ; Address of default center value for RTTY == offset 0
        bra     BFOadjCom

BFO_ssb:
        ldhx   #ssbCenter       ; Address of default center value for SSB == offset 0
        bra     BFOadjCom

BFO_cw:
        ldhx   #cwCenter        ; Address of default center value for CW == offset 0

BFOadjCom:
        ldhxx                   ; get center value in H:X
        sthx   from_addr        ; save it
        ldhx   #ASP_BFO         ;
        ldhxx                   ; get current BFO in h:X
        cmphx  from_addr        ; which way to go to set it?
        beq    B_A_X            ; no change
        blo    BFOdown          ; we need to go lower
;--                        adjust BFO up from Center
        lda     ASP_BFO+1       ; going up compute how much
        sub     from_addr+1     ; subtract center from target
        sta     from_Addr+1
        lda     ASP_BFO
        sbc     from_addr
        sta     from_addr       ; change amount now in from_addr
        ldhx    from_addr       ; change amount in h:x
        txa                     ; now in h:a
        ldx     #100
        div                     ; how many steps of 100?
        beq     B_U10           ; none
        psha                    ; save # of steps on stack
B_U100: lda     #">"            ; up 100Hz command
        jsr     ASP_Cmd
        lda     from_addr+1
        sub     #100            ; reduce remaining
        sta     from_Addr+1
        lda     from_addr
        sbc     #0
        sta     from_addr       ; change amount now in from_addr
        dec     1,sp
        bne     B_U100          ; repeat until change less than 100
        pula
B_U10:  ldhx    from_addr       ; change amount in h:x
        txa                     ; now in h:a
        ldx     #10
        div                     ; how many steps of 10?
        pshh                    ; save  remainder
        beq     B_U1            ; none
        psha
B_U10a: lda     #")"            ; up 10Hz command
        jsr     ASP_Cmd
        dec     1,sp
        bne     B_U10a          ; repeat until change less than 10
        pula                    ; toss # of 10s counter to expose remainder on stack
B_U1:   lda     #0              ; remainder is number of 1s and is on stack
        cmp     1,sp            ; any 1's ?
        beq     B_A_Xa          ; no we're done
B_U1a:  lda     #"+"            ; up 1Hz command
        jsr     ASP_Cmd
        dec     1,sp
        bne     B_U1a           ; repeat 'til done
B_A_Xa: pula                    ; toss # of 1s counter
B_A_X:  rts
;--                        adjust BFO down from Center
BFOdown:
        lda     from_addr+1   ; going down compute how much
        sub     ASP_BFO+1     ; subtract target from center
        sta     from_addr+1
        lda     from_Addr
        sbc     ASP_BFO
        sta     from_addr       ; change amount now in from_addr
        ldhx    from_addr       ; change amount in h:x
        txa                     ; now in h:a
        ldx     #100
        div                     ; how many steps of 100?
        beq     B_D10           ; none
        psha                    ; save # of steps on stack
B_D100: lda     #"<"             ; up 100Hz command
        jsr     ASP_Cmd
        lda     from_addr+1
        sub     #100            ; reduce remaining
        sta     from_Addr+1
        lda     from_addr
        sbc     #0
        sta     from_addr       ; change amount now in from_addr
        dec     1,sp
        bne     B_D100
        pula
B_D10:  ldhx    from_addr       ; change amount in h:x
        txa                     ; now in h:a
        ldx     #10
        div                     ; how many steps of 10?
        pshh                    ; save  remainder
        beq     B_D1            ; none
        psha
B_D10A: lda     #"("             ; up 10Hz command
        jsr     ASP_Cmd
        dec     1,sp
        bne     B_D10A
        pula
B_D1:   lda     #0             ; remainder is number of 1s and is on stack
        cmp     1,sp           ; any 1's ?
        beq     B_D1b          ; no we're done
B_D1a:  lda     #"-"           ; up 1Hz command
        jsr     ASP_Cmd
        dec     1,sp
        bne     B_D1a
B_D1b:  pula
BFOadjXit:
         rts

;-------       format status line with current status
ASPformat_status:
        lda     ASP_status      ; determine mode and put correct character
        cmp     #6              ; in first position
        blo     ASP_cw
        cmp     #9
        blo     ASP_ssb
        lda     #'R'
        sta     DSP_status_msg
        bra     ASPformatCom
ASP_ssb:
        lda     #'S'
        sta     DSP_status_msg
        bra     ASPformatCom
ASP_cw:
        lda     #'C'
        sta     DSP_status_msg
ASPformatCom:
        ldhx    #DSP_status_msg+2   ; put width in next part of status
        sthx    to_addr
        ldhx    #ASP_widths      ; source 4 digit ASCII widths table
        sthx    from_addr
        lda     ASP_status      ; get filter
        deca                    ; make mode a zero based index
        lsla                    ; times 4 for index into ASCII widths table
        lsla
        add     from_addr+1     ; add index to start address
        sta     from_addr+1
        clra
        adc     from_addr
        sta     from_addr
        lda     #4              ; copy 4 bytes
        jsr     MemCopy         ; from current width to status buffer+2
        clr     intacc1         ; Now display curretn BFO value
        clr     intacc1+1       ; put BFO value in intacc1
        lda     ASP_BFO
        sta     intacc1+2
        lda     ASP_BFO+1
        sta     intacc1+3
        jsr     bin2bcd          ; convert to BFO binary value to BCD
        ldhx    #DSP_status_msg+7
        sthx    to_addr
        ldhx    #bcd1000        ; 4 digit ASCII widths
        sthx    from_addr
        lda     #4              ; copy 4 bytes of converted BCD value
        jsr     MemCopy         ; from current bcd1000 to status buffer+7
        lda     DSP_status_msg+7
        cmp     #$00
        bne     afc1
        ora     #$20            ; make leading 0 a space
        bra     afc2
afc1:   ora     #$30            ; make it ascii
afc2:   sta     DSP_status_msg+7
        lda     DSP_status_msg+8
        ora     #$30            ; make it ascii
        sta     DSP_status_msg+8
        lda     DSP_status_msg+9
        ora     #$30            ; make it ascii
        sta     DSP_status_msg+9
        lda     {DSP_status_msg+10}
        ora     #$30            ; make it ascii
        sta     {DSP_status_msg+10}
        lda     ASP_status2        ; get Notch deNoiser state
        bit     #$01               ; Notch on?
        bne     f_no               ; yes
f_nr:   bit     #$02               ; denoiser on?
        beq     f_out              ; no
        lda     #'N'
        sta     {DSP_status_msg+12}
        lda     #'r'
        sta     {DSP_status_msg+13}
f_out:  rts

f_no:   lda     #'N'                 ; Show Notch on
        sta     {DSP_status_msg+14}
        lda     #'t'
        sta     {DSP_status_msg+15}
        lda     ASP_status2
        bra     f_nr                 ; now check denoiser on

;--------   Set EEPROM (page 2) to default values related to ASP
ASP_EEPROM_Defaults:
        lda     #2              ; 2=700 Hz 200 Hz wide CW filter
        sta     ASP_status      ; is current filter
        sta     ASPcwStatus     ; is CW filter
        lda     #8              ; 8=1500 Hz 2400 Hz wide SSB filter
        sta     ASPssbStatus    ; is SSB filter
        lda     #9              ; 9=2210 Hz 200 Hz wide RTTY filter
        sta     ASPrttyStatus   ; is RTTY filter
        clra                    ; notch  & denosier off
        sta     ASP_status2     ; in current filter
        sta     ASPcwStatus2    ; and in all modes
        sta     ASPssbStatus2
        sta     ASPrttyStatus2
        ldhx    #cwCenter
        sthx    from_addr
        ldhx    #ASPcwBFO
        sthx    to_addr
        lda     #6
        jsr     MemCopy         ; set default values for BFOs in each mode
        lda     ASPcwBFO        ; default is CW
        ldx     ASPcwBFO+1
        sta     ASP_BFO         ; so set current to default CW BFO
        stx     ASP_BFO+1
        lda     #'s'            ; 91 taps default
        sta     ASPTaps

        rts

cwCenter:    fdb  700
ssbCenter:   fdb  1500
rttyCenter:  fdb  2210
ASP_widths:  fcb ' 100',' 200',' 300',' 400',' 500','1200','1800','2400',' 200'



;------------- send 1 byte command in Acc response to status_buffer1
;                carry clear if OK, carry set for invalid command

ASP_cmd:
        ldx     #8
        pshx
        ldhx    #0
        jsr     DSPputch           ; send to DSP
ll1:    jsr     DSPgetchne         ; get reply & prompt
        sta     SPI_buffer1,x
        aix     #1
        dec     1,sp
        bne     ll1
        pula
;---DEBUG   lda     #0                 ; DEBUG see what we tell DSP
;---DEBUG   sta     SPI_buffer1+8      ; DEBUG and what it replies
;---DEBUG   ldhx    #SPI_buffer1       ; DEBUG
;---DEBUG   jsr     _puts              ; DEBUG
        lda     #'?'
        cmp     SPI_Buffer1+2
        beq     A_Cerr
        clc
        rts
A_Cerr: sec
        rts



;---        called from Read_Encoder when PB2 or PB3 pushed and ASP is running
CheckASP_PB:            ; check for PBs when running ASP

        brset   PB3,portg,CA_PB2     ; check button 3 pushed
                                     ; button set advance NR & Notch
        brclr   PB3,portg,*          ; await button release
        lda     ASP_status           ; determine current mode
        cmp     #6
        blo     CA_exit              ; CW do nothing
                                     ; only SSB and RTTY can do this
        lda     ASP_status2          ; get current state
        inca                         ; advance by adding 1
        and     #$03                 ;  modulo 4
        sta     ASP_status2          ; save new
        cmp     #1                   ; toggle Notch (on)?
        beq     cNo
        cmp     #2                   ; toggle deNoiser (on) & Notch (off)?
        beq     cdN_No
        cmp     #3                   ; toggle Notch (on) ?
        beq     cNo
        cmp     #0                   ; toggle both (off) ?
        beq     cdN_No
        rts
cdN_No:
        lda     #'d'
        jsr     ASP_cmd              ; send command toggle deNoiser
cNo:    lda     #'n'
        jsr     ASP_cmd              ; send command toggle Notch
        bra     CA_tell

cdN:    lda     #'d'
        jsr     ASP_cmd              ; send command toggle deNoiser
CA_tell:
        jsr     asp_showStatus       ; tell the world
CA_exit:
        rts


CA_PB2:
        brset   PB2,porth,CA_ret      ; if PB2 not pushed exit
                                     ; PB2 button set advance width
        brclr   PB2,porth,*          ; await button release
        lda     ASP_status           ; determine current mode
        cmp     #6
        blo     CA_cw
        cmp     #9
        blo     CA_ssb
CA_ret: rts                          ; RTTY has only one width do nothing
CA_ssb:
        cmp     #8                   ; if widest
        beq     CA_ssb1              ; wrap to low
        inca                         ; else widen current
CA_tellExit:
        sta     ASP_status           ; save new
        ora     #$30                 ; make it ascii for command
        jsr     ASP_cmd              ; tell DSP
        bra     CA_tell

CA_ssb1:
        lda     #6                   ; set ssb narrowest
        bra     CA_tellExit

CA_cw:  cmp     #5                   ; if widest
        beq     CA_cw1               ; wrap to low
        inca                         ; else widen current
        bra     CA_tellExit

CA_cw1: lda     #1                   ; set cw narrowest
        bra     CA_tellExit







