;**************************************************************************
;   Runtime support for the DSP908 DSP program
;    provides oprational control of the DSP908 program
;    entered from PB1_Run in PB1.asm after DSP908 has been loaded and started
;
;    Written January 2005 by Steve Holton N1NB
;**************************************************************************

DSP_dsp908:
          ; initialize volatile variables
        lda     #1                     ; next step is mid
        sta     DSPfilter_next
        lda     #$04                   ; next is Notch
        sta     DSPtoggle_next
           ; restore last saved mode and filter parms, gains,taps,betas
        jsr     restore908

DSP_top_start:                          ; loops from here until "End" selected
        jsr     dsp908_showStatus

        ldhx    #DSPtopCmdTable         ;set up pointer to Top level CMD Table
        sthx    CmndTabStart
        ldhx    #DSPtopMsgTable        ;set up pointer to Top level MSG Table
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection


;********  Top level messages
DSPtopMsgTable:
DSP_End_msg     fcb     'End DSP pgm',0
DSP_Change_msg: fcb     'Change',0    ; 1st item displayed
DSP_Save_msg:   fcb     'Save Settings',0      ; ccw item
                fcb     'Exit',0
;;;DSP_Query_msg:  fcb     'Query',0     ; CW item
                fcb     0                 ; end of list
;***** top level commands
DSPtopCmdTable:
        fdb     DSP_End
        fdb     DSP_Change
        fdb     DSP_Save
        fdb     DSP_exit
 ;;       fdb     DSP_Query


DSP_Save:                       ; Save all DSP setting to EEPROM page 2
        lda     #$C0
        jsr     Cmnd2LCD
        ldhx    #Save_Settings
        jsr     disp_msg
        bclr    BusyLED,portc
        jsr     put_EEPROM_p2
        bset    BusyLED,portc
        jmp     DSP_top_start        ; go get next action at this level

DSP_End:
        clra                         ; dsp908 no longer running
        sta    DSPisdsp908
        jsr    DSPreset_banner       ; stop DSP via reset and return
        rts                          ; to DSP_Run

DSP_Change:
        ldhx    #DSPchangeCmdTable    ; pointer to change level commands
        sthx    CmndTabStart
        ldhx    #DSPchangeMsgTable    ; pointer to change level messages
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr              ; counter used as index into Msg Table
        jmp     get_dial_selection

;---  change level messages
DSPchangeMsgTable:
       fcb     'Set Widths',0 ; CW item
       fcb     'Filter',0    ; 1st item displayed
       fcb     'Mode',0
       fcb     'Gain',0
       fcb     'Taps',0
       fcb     'Beta/Decay',0
       fcb     'Exit',0
       fcb     0                 ; end of list
;--- change level commands
DSPchangeCmdTable:
        fdb     DSP_widths
        fdb     DSP_filter
        fdb     DSP_mode
        fdb     DSP_gain
        fdb     DSP_taps
        fdb     DSP_BetaDecay
        fdb     DSP_Exit

;---- change level command processors
;-----   change -> Exit
DSP_Exit:                       ; We were entered via call - we leave here by JMP
        ais    #2               ;  we must remove return address on stack
        clra                    ; dsp908 no longer running flag BUT we're leaving
        sta    DSPisdsp908      ; it running. We must clear flag because once we leave
        jmp    PB1_Exit         ; we can never return to dsp908 controls nor can
                                ; we leave it set since next dsp to run might be
                                ; a different dsp program
                                
;-----   change->Mode
DSP_mode:
        ldhx    #DSPmodeCmdTable    ;set up pointer to
        sthx    CmndTabStart
        ldhx    #DSPmodeMsgTable    ;set up pointer to
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection

;...  mode level messages
DSPmodeMsgTable:
        fcb     'Soft CW',0     ; CW item
        fcb     'CW',0          ; 1st item displayed
        fcb     'RTTY',0
        fcb     'SSB',0
        fcb     0                ; end of list
;... mode level commands
DSPmodeCmdTable:
        fdb     DSP_softCWset
        fdb     DSP_CWset
        fdb     DSP_RTTYset
        fdb     DSP_SSBset
;... mode level command processors
DSP_softCWset:
        jsr     saveModeStatus       ; save current status for old mode
        lda     DSPcw_mode           ; get CW  status
        and     #$FC                 ; remove mode (0 = soft cw)
        ora     #$80                 ; indicate 2 byte command  in A
        ldx     DSPcw_fil            ; put current cw filter in A
        bra     mode_com
DSP_CWset:
        jsr     saveModeStatus       ; save current status for old mode
        lda     DSPcw_mode           ; get CW  status
        and     #$FC                 ; remove mode
        ora     #$81                 ; indicate 2 byte command + CW in A
        ldx     DSPcw_fil            ; put current cw filter in X
        bra     mode_com
DSP_RTTYset:
        jsr     saveModeStatus       ; save current status for old mode
        lda     DSPrtty_mode         ; get rtty status
        ora     #$80                 ; indicate 2 byte command in A
        ldx     DSPrtty_fil          ; put current cw filter in X
        bra     mode_com
DSP_SSBset:
        jsr     saveModeStatus       ; save current status for old mode
        lda     DSPssb_mode          ; get ssb status
        ora     #$80                 ; indicate 2 byte command  in A
        ldx     DSPssb_fil           ; put current cw filter in X
mode_com:
        jsr     send_cmd2            ; send command
        lda     1                    ; make next step mid width
        sta     DSPfilter_next
        jmp     DSP_top_start

;-----   change->Filter
DSP_filter:
        lda     DSP_statusbyte      ; get current status
        and     #$03                ; isolate current mode
        cmp     #$02                ; is it SSB
        beq     ssb_fil
        cmp     #3
        beq     rtty_fil
        ldhx    #DSPfilCmdTable     ;set up pointer to CW,RTTY,SoftCW changes
        sthx    CmndTabStart
        ldhx    #DSPfilmsgTable     ;set up pointer to CW,RTTY,SoftCW changes
dialcom:
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr            ;counter used as index into Msg Table
        jmp     get_dial_selection
ssb_fil:
        ldhx    #DSPssbfilCmdTable   ;set up pointer to SSB filter changes
        sthx    CmndTabStart
        ldhx    #DSPssbfilmsgTable   ;set up pointer to SSB filter changes
        bra     dialcom
rtty_fil:
        ldhx    #DSPRTTYfilCmdTable   ;set up pointer to RTTY filter changes
        sthx    CmndTabStart
        ldhx    #DSPRTTYfilmsgTable   ;set up pointer to RTTY filter changes
        bra     dialcom
;...  filter level messages
DSPfilMsgTable:                  ; for CW, CW Soft
        fcb     'Center',0 ; CW item
        fcb     'Width',0        ; 1st item displayed
        fcb     'Toggle Noise',0
;        fcb     'Toggle Notch',0

        fcb     0                ; end of list
DSPRTTYfilMsgTable:                  ; for  RTTY
        fcb     'Center',0 ; CW item
        fcb     'Width',0        ; 1st item displayed
;        fcb     'Toggle Noise',0
;        fcb     'Toggle Notch',0

        fcb     0                ; end of list

DSPssbfilMsgTable:               ; for SSB
        fcb     'Low',0 ; CW item
        fcb     'High',0         ; 1st item displayed

        fcb     'Toggle Noise',0
        fcb     'Toggle Notch',0
        fcb     0                ; end of list

;.... filter level commands
DSPfilCmdTable:                 ; for CW,  CW Soft
        fdb     DSP_Center
        fdb     DSP_Width
        fdb     DSP_Noise
;        fdb     DSP_Notch
DSPRTTYfilCmdTable:                 ; for  RTTY  Soft
        fdb     DSP_Center
        fdb     DSP_Width
;        fdb     DSP_Noise
;        fdb     DSP_Notch
DSPssbfilCmdTable:              ; for SSB
        fdb     DSP_Low
        fdb     DSP_High
        fdb     DSP_Noise
        fdb     DSP_Notch
;.... filter level command processors

DSP_Notch:
        lda     DSP_statusbyte       ; in current status
        and     #$04                 ; is autonotch on?
        beq     setnotch             ; no
        lda     DSP_statusbyte       ; in current status
        and     #$FB                 ; turn autonotch off
        sta     DSP_statusbyte
        jmp     filcom
setnotch:
        lda     DSP_statusbyte       ; in current status
        ora     #$04                 ; turn autonotch on
        sta     DSP_statusbyte
        jmp     filcom
DSP_Noise:
        lda     DSP_statusbyte       ; in current status
        and     #$08                 ; is denoiser on ?
        beq     setnoise             ; no
        lda     DSP_statusbyte       ; in current status
        and     #$F7                 ; turn denoiser off
        sta     DSP_statusbyte
        jmp     filcom
setnoise:
        lda     DSP_statusbyte       ; in current status
        ora     #$08                 ; turn denoiser on
        sta     DSP_statusbyte
        jmp     filcom

;;------- These command processors are different than the others -----------------
;;          (DSP_Width, DSP_Center, DSP_low, DSP_high)
;;
;;       1. Each time the dial turns to a new value a call-back is made to proceess
;;          the new value and tell the DSP so the user can hear the effect.
;;          For now the callback is hard coded in Get_Dial_Selection. In a future
;;          release this will all be cleaned up in a more general interfece to
;;          Get_Dial_Selection.
;;       2. When the user pushe the dial PB to "commit" the new value then the
;;          normal command processor is invoked
;;


DSP_Width:
        lda    #0
        sta    DSP_fil_nibble   ; width updates low nibble of filter parameters
        lda    DSP_filterparms  ; get current value
        and    #$0F             ; from low nibble
        deca                    ; subtract 1
        and    #$0F             ; modulo $10
        sta    msg_cntr         ; save index for dial processing
        ldx    #5               ; all msgs are 5 bytes long
        mul                     ; compute offset in bytes to msg
        psha                    ; save for a moment
        lda    DSP_statusbyte
        and    #$03             ; isolate mode to do mode specific part
        cmp    #3               ; DATA
        pula                    ; recover index to msg
        beq    ld_rtty_width

;; --- CW width
        ldhx   #CW_width_msg    ; base address
        bra    fil_cmd_common

ld_rtty_width:
        ldhx   #RTTY_width_msg    ; base address
        bra    fil_cmd_common



fil_cmd_common:
        sthx    MsgTabStart     ; save for msg processing
        aax                     ; add offset to first msg
        sthx    msg_pointer     ; save for msg processing
        lda    #1
        sta    MSG_instant      ; instant processing - set flag
        ldhx   #do_nibbleNOW    ; and set pointer to callback
        sthx   msg_callback
        ldhx    #do_nibble             ;set up pointer to single command
        sthx    CmndTabStart
        lda     #1                     ; set flag for single command
        sta     MSG_one
        jmp     get_dial_selection

DSP_Center:
        lda    #1
        sta    DSP_fil_nibble   ; center updates hi nibble of filter parameters
        lda    DSP_filterparms  ; get current value
        nsa                     ; into low nibble
        and    #$0F             ; isolate low nibble
        deca                    ; subtract 1
        and    #$0F             ; modulo $10
        sta    msg_cntr         ; save index for dial processing
        ldx    #5               ; all msgs are 5 bytes long
        mul                     ; compute offset in bytes to msg
        psha                    ; save for a moment
        lda    DSP_statusbyte
        and    #$03             ; isolate mode to do mode specific part
        cmp    #3               ; DATA
        pula                    ; recover index to msg
        beq    ld_rtty_center

;; --- CW center
        ldhx   #CW_center_msg    ; base address
        bra    fil_cmd_common

ld_rtty_center:
        ldhx   #RTTY_center_msg    ; base address
        bra    fil_cmd_common


DSP_High:
        lda    #1
        sta    DSP_fil_nibble   ; high updates hi nibble of filter parameters
        lda    DSP_filterparms  ; get current value
        nsa                     ; into low nibble
        and    #$0F             ; isolate low nibble
        deca                    ; subtract 1
        and    #$0F             ; modulo $10
        sta    msg_cntr         ; save index for dial processing
        ldx    #5               ; all msgs are 5 bytes long
        mul                     ; compute offset in bytes to msg
        ldhx   #SSB_high_msg    ; base address
        bra    fil_cmd_common

DSP_Low:
        lda    #0
        sta    DSP_fil_nibble   ; low updates low nibble of filter parameters
        lda    DSP_filterparms  ; get current value
        and    #$0F             ; isolate low nibble
        sub    #1               ; subtract 1
        bcc    S_L_0            ; if not at zero
        lda    #$0B             ; otherwise we wrap
S_L_0:  sta    msg_cntr         ; save index for dial processing
        ldx    #5               ; all msgs are 5 bytes long
        mul                     ; compute offset in bytes to msg
        ldhx   #SSB_low_msg     ; base address
        jmp    fil_cmd_common


;,,,,  change->filter->values to change messages
CW_center_msg: fcb '400 ',0,'450 ',0,'500 ',0,'550 ',0,'600 ',0,'650 ',0
               fcb '700 ',0,'750 ',0,'800 ',0,'850 ',0,'900 ',0,'950 ',0
               fcb '1000',0,'1100',0,'1200',0,'1300',0,0
CW_width_msg:  FCB ' 50 ',0,'100 ',0,'150 ',0,'200 ',0,'250 ',0,'300 ',0
               FCB '350 ',0,'400 ',0,'450 ',0,'500 ',0,'550 ',0,'600 ',0
               FCB '650 ',0,'700 ',0,'750 ',0,'800 ',0,0
SSB_high_msg:  fcb '1500',0,'1600',0,'1700',0,'1800',0,'1900',0,'2000',0
               fcb '2100',0,'2200',0,'2300',0,'2400',0,'2500',0,'2600',0
               fcb '2700',0,'2800',0,'2900',0,'3000',0,0
SSB_low_msg:   fcb '  0 ',0,'100 ',0,'150 ',0,'200 ',0,'250 ',0,'300 ',0,'350 ',0
               fcb '400 ',0,'450 ',0,'500 ',0,'550 ',0,'600 ',0,0
RTTY_center_msg:
               fcb '1000',0,'1100',0,'1200',0,'1300',0,'1400',0,'1500',0,'1600',0
               fcb '1700',0,'1800',0,'1900',0,'2000',0,'2100',0,'2200',0,'2300',0
               fcb '2400',0,'2500',0,0
RTTY_width_msg:
               fcb ' 50 ',0,'100 ',0,'200 ',0,'300 ',0,'400 ',0,'500 ',0,'600 ',0
               fcb '700 ',0,'800 ',0,'900 ',0,'1000',0,'1200',0,'1400',0,'1600',0
               fcb '1800',0,'2000',0,0

;;-- this command processor is called when user pushes dial PB to "committ"
;---- new value that was selected via call back mechanism all that's left
;---- is to clear flags
; used by any callback user

do_nibble:
do_gain:
do_AICgain:
do_taps:
        clra
        sta     MSG_instant        ; turn of instant update flag
        sta     MSG_one            ; turn off single command flag
        jmp     DSP_top_start      ; go to top of loop format and display new value

;; This is the "call back" called from Get_Dial_Selection every time a new value is
;;    displayed when user turns the dial
;;  It updates the status and informs the DSP to make the change so the user
;;    can hear the effect as the dial is turned
;;  Some of the code is also common starting at filcom:  to other functions
;;    so a test is made at the end to see how that code was entered to determine
;;    how the exit is to be made-  Ugly bit works will be changed in future
;;    release when Get_Dial_Selection is reworked

do_nibbleNOW:
        lda     DSP_filterparms
        ldx     DSP_fil_nibble    ; get which nibble to update
        beq     its_low
        and     #$0F              ; remove old value
        sta     DSP_filterparms
        lda     msg_cntr           ; get selection
        and     #$0F               ; isolate value
        nsa                        ; into high nibble
        ora     DSP_filterparms    ; combine
        sta     DSP_filterparms
        bra     filcom
its_low:
        and     #$F0              ; remove old value
        sta     DSP_filterparms
        lda     msg_cntr           ; get selection
        and     #$0F               ; isolate value
        ora     DSP_filterparms    ; combine
        sta     DSP_filterparms

;;--                  this code is used by several functions
;;                    we test at end how we got here to see how we're supposed to leave
filcom:
        lda     DSP_statusbyte
        and     #$03
        cmp     #$02
        beq     updatessb
        cmp     #$03
        beq     updatertty
        lda     DSP_statusbyte
        sta     DSPcw_mode
        lda     DSP_filterparms
        sta     DSPcw_fil
        bra     fil_cmd2
updatessb:
        lda     DSP_statusbyte
        sta     DSPssb_mode
        lda     DSP_filterparms
        sta     DSPssb_fil
        bra     fil_cmd2
updatertty:
        lda     DSP_statusbyte
        sta     DSPrtty_mode
        lda     DSP_filterparms
        sta     DSPrtty_fil
fil_cmd2:
        lda     DSP_statusbyte       ; get current status
        ora     #$80                 ; mark as set status
        ldx     DSP_filterparms      ; put current filter in X
        jsr     send_cmd2            ; send command
        lda     MSG_instant          ; if we're here for instant width dial
        bne     fil_now              ; we're a subroutine - return
        jmp     DSP_top_start        ; all others to top of loop
fil_now:
        rts

;-----   change->Beta/Decay
DSP_BetaDecay:
        ldhx    #DSPbetaCmdTable        ; Now select which to change
        sthx    CmndTabStart
        ldhx    #DSPbetaMsgTable
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection

;...   beta/decay level messages
DSPbetaMsgTable:
        fcb     'Notch Beta',0     ; CW item
        fcb     'Noise Beta',0     ; 1st item displayed
        fcb     'Noise Decay',0
        fcb     'Notch Decay',0
        fcb     0                ; end of list
;... beta/decay level commands
DSPbetaCmdTable:
        fdb     doNotchBeta
        fdb     doNoiseBeta
        fdb     doNoiseDecay
        fdb     doNotchDecay
;...  beta/decay level commandprocessors
doNoiseBeta:
        lda     #$C1            ; command for 1st byte of command
        ldx     #0              ; our index into betas and decays
        bra     doComBetaDecay

doNoiseDecay:
        lda     #$C2            ; command for 1st byte of command
        ldx     #1              ; our index into betas and decays
        bra     doComBetaDecay

doNotchBeta:
        lda     #$C4            ; command for 1st byte of command
        ldx     #2              ; our index into betas and decays
        bra     doComBetaDecay

doNotchDecay:
        lda     #$C8            ; command for 1st byte of command
        ldx     #3              ; our index into betas and decays
        bra     doComBetaDecay

;--- doComBetaDecay  -- common processing for setting decays and betas
; invoker puts in X the index into list of betas & decays
;         order:      0              1          2           3
;                 DenoiserBeta,DenoiserDecay,NotchBeta,NotchDecay
; invoker puts in A 1st Command byte for proper command
;     computes value 0-255  from value supplied in range 0-99
;        sends value to DSP as 2nd byte of 2 byte command

doComBetaDecay:
        sta     DSP_cmd2        ; save command byte
        clr     intacc1
        clr     intacc1+1
        clr     intacc1+2
        clrh
        pshx                    ; save copy of index for later
        lda     DenoiserBeta,x  ; and current user value load it
        sta     intacc1+3
        jsr     bin2bcd         ; convert to BCD
        mov     BCD10,LCD_dat   ; move BCD to dial buffer
        mov     BCD1,LCD_dat+1
        lda     #2
        sta     AFcursor_end    ; position cursor under 1's digit
        sta     AFTwo_Digits    ; 2 digit value 0-99
        lda     #$C0
        jsr     Cmnd2LCD        ;write to 2nd line
        jsr     ClearLCDLine2
        jsr     Process_DialAF  ; get new value
        ldhx    #lcd_dat
        jsr     BCD2to16Bin     ; convert to binary
        lda     intacc1+1       ; can only be one byte (0-99)
        clrh
        pulx                    ; index of the one we are changing
        sta     DenoiserBeta,x  ; and save it
        ldx     #$ff            ; spread 0-99 beta to 0-255 command value
        mul                     ; compute beta*255 a*x result in x:a
        pshx                    ; move so
        pulh                    ; result now in h:a
        ldx     #100
        div                     ; compute (beta*255)/100
        sta     DSP_cmd2+1      ; put quotient in command
        lda     DSP_statusbyte  ; before sending command
        psha                    ; we must save current
        lda     DSP_filterparms ; status as these commands
        psha                    ; send back different type of status
        jsr     send_cmd2a      ; send command
        pula                    ; restore status
        sta     DSP_filterparms ; so it'll format correctly
        pula
        sta     DSP_statusbyte
        lda     #$0C
        jsr     Cmnd2LCD        ;turn off cursor
        jmp     DSP_top_start   ; go to user select next command

;-----   change->Gain
DSP_Gain:
        ldhx    #DSPGainCmd          ; Now get gain type
        sthx    CmndTabStart
        ldhx    #DSPGainMsg
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table
        jmp     get_dial_selection

;.... which gain to change messages
DSPGainMsg:
  fcb 'DeNoiser',0,'AutoNotch',0,'CW',0,'SSB',0,'Data',0,'CODEC',0,0
;.... which gain to change commands
DSPGainCmd:
  fdb DSP_Gain2,DSP_Gain2,DSP_Gain2,DSP_Gain2,DSP_Gain2,DSP_GainCODEC
;.... which gain to change values (value == command byte)
DSPGainValues:
  fcb $D0,$D1,$D2,$D3,$D4,$D5

;.... which gain to change command processor (for all but CODEC gain)
DSP_Gain2:
        clrh
        ldx     msg_cntr             ; get index for selected gain type
        lda     DSPGainValues,x      ; get corresponding command value
        sta     DSP_cmd2             ; in 1st byte of 2 byte command
        ldhx    #do_gain             ; Now get gain value
        sthx    CmndTabStart         ; save single command to do so
        lda     #1                   ; and set flag
        sta     msg_one              ; for single command processing
        sta     msg_instant          ; and for call back processing
        ldhx    #do_gainNOW
        sthx    msg_callback
        lda     #49                  ; # of gains
        psha                         ; as counter on stack
        clrh
        ldx     msg_cntr             ; get index for selected gain type
        lda     DeNoisergain,x       ; get current setting for this gain
        clrx
D_G_a:  cmpa    DSPdoGainValues,x    ; find index into list fo current gain
        beq     D_G_b
        dec     1,sp
        aix     #1
        bne     D_G_a
        ldx     #38                  ; shouldn't get - if so set to default
D_G_b:  pula                         ; clear counter from stack
        txa                          ; put index in A
        tsta                         ; see if we are a start of list
        beq     D_G_c                ; if so skip
        deca                         ; subtracting one
        bra     D_G_d
D_G_c:  lda     #48                  ; at start of list must point to end of list
D_G_d:  sta     msg_cntr             ; counter used as index into Msg Table save starting value
        clr     intacc1
        sta     intacc1+1            ; set up 16x16bit multiply (index * 7) -won;t fit in 8 bits!
        clr     intacc2
        ldx     #7                   ; # of chars each text list entry
        stx     intacc2+1
        jsr     UMULT16
        ldhx    #DSPdoGainMsg        ; save text list start
        sthx    MsgTabStart
        txa                          ; get low bits of address to A
        add     intacc1+3            ; so we can add result of multiply
        tax                          ; put back in X
        tha                          ; now get hi bits in A
        adc     intacc1+2            ; add hi bits w/ carry
        tah                          ; back to H
                                     ; h:x now offset to entry for initial display == current value
        sthx    msg_pointer

        jmp     get_dial_selection


;,,,, change gain value messages
DSPdoGainMsg:
  fcb  '-5.5dB',0,'-6.0dB',0,'18.0dB',0,'17.5dB',0,'17.0dB',0,'16.5dB',0,'16.0dB',0,'15.5dB',0
  fcb  '15.0dB',0,'14.5dB',0,'14.0dB',0,'13.5dB',0,'13.0dB',0,'12.5dB',0,'12.0dB',0,'11.5dB',0

  fcb  '11.0dB',0,'10.5dB',0,'10.0dB',0,' 9.5dB',0,' 9.0dB',0,' 8.5dB',0,' 8.0dB',0,' 7.5dB',0
  fcb  ' 7.0dB',0,' 6.5dB',0,' 6.0dB',0,' 5.5dB',0,' 5.0dB',0,' 4.5dB',0,' 4.0dB',0,' 3.5dB',0

  fcb  ' 3.0dB',0,' 2.5dB',0,' 2.0dB',0,' 1.5dB',0,' 1.0dB',0,' 0.5dB',0,' 0.0dB',0,'-0.5dB',0
  fcb  '-1.0dB',0,'-1.5dB',0,'-2.0dB',0,'-2.5dB',0,'-3.0dB',0,'-3.5dB',0,'-4.0dB',0,'-4.5dB',0,'-5.0dB',0,0

;,,,, change gain values  == 2nd byte of the 2 byte command
DSPdoGainValues:
   fcb  $01,$00,$3c,$3b,$3a,$39,$38,$37
   fcb  $36,$35,$34,$33,$32,$31,$30,$2b

   fcb  $2a,$29,$28,$27,$26,$25,$24,$23
   fcb  $22,$21,$20,$1b,$1a,$19,$18,$17

   fcb  $16,$15,$14,$13,$12,$11,$10,$0b
   fcb  $0a,$09,$08,$07,$06,$05,$04,$03,$02


;,,,, change gain value command processor callback
do_gainNOW:                         ; setup above put which gain in
        clrh                         ;  command's 1st byte
        ldx     msg_cntr             ; get index for selected gain
        lda     DSPdoGainValues,x    ; get corresponding gain value
        sta     DSP_cmd2+1           ; in 2nd byte of 2 byte command
do_g_c: clrh
        lda     DSP_cmd2             ; get command value tells which gain
        and     #$0F                 ; isolate index from command prefix
        tax                          ; as index in h:x
        lda     DSP_cmd2+1           ; get actual gain value
        sta     DeNoisergain,x       ; and save in correct slot for later
        jsr     send_cmd2a           ; send command
        rts                          ; return to get_dial_selection

;... change CODEC gain command processor
DSP_GainCODEC:
        clrh
        ldx     msg_cntr             ; get index for selected gain type
        lda     DSPGainValues,x      ; get corresponding command value
        sta     DSP_cmd2             ; in 1st byte of 2 byte command
        ldhx    #do_AICgain          ; Now get gain value
        sthx    CmndTabStart
        lda     #1                   ; set flag for single command
        sta     msg_one
        sta     msg_instant          ; and for call back processing
        ldhx    #do_AICgainNOW
        sthx    msg_callback
        lda     #$7F                 ; max value
        sub     CODECgain            ; subtract current value as index to 1st msg
        sta     msg_cntr             ;counter used as index into Msg Table
        ldx     #7                   ; 7 chars/msg
        mul                          ; compute start of msg
        ldhx    #DSPdoAICGainMsg
        sthx    MsgTabStart          ; save start of msg table
        aax                          ; compute 1st msg
        sthx    msg_pointer          ; and start here
        jmp     get_dial_selection

;... change CODEC gain messages
DSPdoAICGainMsg:
  fcb '-6.0dB',0,'+6.0dB',0,'+4.5dB',0,'+3.0dB',0,'+1.5dB',0,'0.0dB',0,'-1.5dB',0,'-3.0dB',0,'-4.5dB',0,0


;... change CODEC gain command processor callback
do_AICgainNOW:
        lda     msg_cntr
        beq     d_Ag_a
        lda     #$7F                 ; Max gain +6.0dB is 7F
        sub     msg_cntr             ; subtract index-1 for correct value
        inca
        bra     d_Ag_b
d_Ag_a  lda     #$77
d_Ag_b: sta     DSP_cmd2+1           ; in 2nd byte of 2 byte command
        jmp     do_g_c

;-----   change->Taps
DSP_taps:
        ldhx    #do_taps             ; Now get tap value
        sthx    CmndTabStart
        lda     #1                   ; single command operation
        sta     msg_one
        sta     msg_instant          ; and for call back processing
        ldhx    #do_tapsNOW
        sthx    msg_callback
        lda     DSP_statusbyte       ; get current status
        and     #$30                 ; isolate current taps
        nsa                          ; to low nibble
        bit     #$01                 ; msg table not in order of tap value get the right value
        beq     d_t_1                ; no change needed (00 and 20)
        eor     #$02                 ; flip bit 1( 01->11 and 11->01 )
d_t_1:  deca
        and     #$03
        ldx     #$03
        sta     msg_cntr             ;counter used as index into Msg Table save starting value
        mul                          ; times 3 as index
        ldhx    #DSPdoTapsMsg
        sthx    MsgTabStart          ; save start of table
        aax                          ; add index to current value
        sthx    msg_pointer          ; have code start here
        jmp     get_dial_selection

;....   change->Taps messages
DSPdoTapsMsg:
  fcb '31',0,'91',0,'71',0,'51',0,0

DSPdoTapsValues:

  fcb  $00,$30,$20,$10        ; taps value for status flag & command 1st byte
;....   change->Taps command processor
do_tapsNOW:
        lda     DSP_statusbyte       ; get current status
        and     #$cf                 ; remove current taps
        clrh
        ldx     msg_cntr             ; get index for new taps value
        ora     DSPdoTapsValues,x    ; insert new taps value
        sta     DSP_statusbyte       ; save new status
        ora     #$80                 ; indicate 2 byte command
        ldx     DSP_filterparms      ; get current DSP_filterparms
        jsr     send_cmd2            ; send command
        rts

;-----   change->Widths
DSP_widths:
        ldhx    #DSPWidthsMsg
        sthx    MsgTabStart
        sthx    msg_pointer
        clr     msg_cntr               ;counter used as index into Msg Table

        lda     DSP_statusbyte
        and     #$03
        cmp     #$02
        beq     ssbwidth
        cmp     #$03
        beq     rttywidth
        ldhx    #DSPWidthsCWCmd           ; Now get CW commands
w_e:    sthx    CmndTabStart
        jmp     get_dial_selection
ssbwidth:
        ldhx    #DSPWidthsSSBCmd           ; Now get SSB commands
        bra     w_e
rttywidth:
        ldhx    #DSPWidthsRTTYCmd          ; Now get RTTY commands
        bra     w_e

;-----   change->Widths messages
DSPWidthsMsg:
  fcb    'narrow',0,'mid',0,'wide',0,0
;-----   change->Widths commands when in CW mode
DSPWidthsCWCmd:
  fdb     is_CWnarrow,is_CWmid,is_CWwide
;-----   change->Widths commands when in RTTY mode
DSPWidthsRTTYCmd:
  fdb     is_RTTYnarrow,is_RTTYmid,is_RTTYwide
;-----   change->Widths commands when in SSB mode
DSPWidthsSSBCmd:
  fdb     is_SSBnarrow,is_SSBmid,is_SSBwide



;-----   change->Widths commands processors
is_CWnarrow:
        lda    #1
        bra    is_width_com_c
is_CWmid:
        lda    #2
        bra    is_width_com_c
is_CWwide:
        lda    #3
is_width_com_c:
        ldhx   #CW_width_msg    ; base address
        bra    is_width_com

is_RTTYnarrow:
        lda    #9
        bra    is_width_com_r
is_RTTYmid:
        lda    #10
        bra    is_width_com_r
is_RTTYwide:
        lda    #11
is_width_com_r
        ldhx   #RTTY_Width_msg

is_width_com:
        pshx                    ; save h:x from msg addr h:x til later
        pshh
        sta    DSPwidth_set     ; this is offset from cw_fil to selected width
        clrh                    ;  set offset in H:X
        tax                     ; this is offset from cw_fil to selected width
        lda    DSPcw_fil,x      ; get current value
        and    #$0F             ; from low nibble
        deca                    ; subtract 1
        and    #$0F             ; modulo $10
is_width_com_2:
        sta    msg_cntr         ; save index for dial processing
        ldx    #5               ; all msgs are 5 bytes long
        mul                     ; compute offset in bytes to msg
        pulh
        pulx                    ; recover X part of msg start
        sthx    MsgTabStart     ; save start of msg table
        aax                     ; add index to current
        sthx    msg_pointer
        ldhx    #do_widths             ; Now get  value
        sthx    CmndTabStart
        lda     #1                     ; indicate single command
        sta     msg_one
        jmp     get_dial_selection

is_SSBnarrow:
        lda    #5
        bra    is_width_com_s
is_SSBmid:
        lda    #6
        bra    is_width_com_s
is_SSBwide:
        lda    #7
is_width_com_s:
        sta    DSPwidth_set     ; this is offset from cw_fil to selected width
        clrh                    ;  set offset in H:X
        tax                     ; this is offset from cw_fil to selected width
        lda    DSPcw_fil,x      ; get current value
        and    #$0F             ; from low nibble
        sub    #1               ; subtract 1
        bcc    i_w_c_s          ; if not at zero
        lda    #$0B             ; otherwise we wrap
i_w_c_s: ldhx   #SSB_Low_msg
        pshx
        pshh
        bra    is_width_com_2

;.....   change->Widths select value messages use basic width messgaes

;...   change->Widths select value command processor
do_widths:                   ; does lower nibble= width CW&RTTY  SSB (lower)
        lda    DSP_statusbyte
        and    #$03
        cmp    #$02             ; is it ssb
        beq    needmore_w       ; yes we need to get lower & upper
        clra
        sta    msg_one          ; clear single command flag
        clrh
        ldx    DSPwidth_set     ; this is offset from cw_fil to selected width
        lda    DSPcw_fil,x
        and    #$F0             ; remove lower nibble from current
        psha
        lda    msg_cntr         ; get new value
        ora    1,sp             ; combine
        sta    DSPcw_fil,x
        pula
        jmp    DSP_top_start

needmore_w:                     ; for SSB we need to get two values get 2nd
        clrh
        ldx    DSPwidth_set     ; this is offset from cw_fil to selected width
        lda    DSPcw_fil,x
        and    #$F0             ; remove lower nibble from current
        psha
        lda    msg_cntr         ; get new value
        ora    1,sp             ; combine
        sta    DSPcw_fil,x
        pula
        ldhx    #do_widths2           ; Now get high value
        sthx    CmndTabStart
        lda     #1
        sta     msg_one
        clrh
        ldx    DSPwidth_set     ; this is offset from cw_fil to selected width
        lda    DSPcw_fil,x      ; get current value
        and    #$F0             ; from high nibble
        nsa                     ; to low nibble
        deca                    ; subtract 1
        and    #$0F             ; modulo $10
        sta    msg_cntr         ; save index for dial processing
        ldx    #5               ; all msgs are 5 bytes long
        mul                     ; compute offset in bytes to msg
        ldhx    #SSB_high_Msg
        sthx    MsgTabStart     ; save start of msg table
        aax                     ; add index to current
        sthx    msg_pointer
        jmp     get_dial_selection


;....   change->Widths select value for SSB High Value command processor
do_widths2:                     ; does upper nibble for SSB
        clra
        sta    msg_one          ; clear single command flag
        clrh
        ldx    DSPwidth_set     ; this is offset from cw_fil to selected width
        lda    DSPcw_fil,x
        and    #$0F             ; remove upper nibble from current
        psha
        lda    msg_cntr         ; get new value
        nsa                     ; into hi nibble
        ora    1,sp             ; combine
        sta    DSPcw_fil,x      ; save as new
        pula                    ; clear stack
        jmp    DSP_top_start

;;------------------ Format DSP908 current status in status line
format_status:               ; format status bytes
        lda    DSP_statusbyte
        and    #$03         ; isolate mode to do mode specific part
        cmp    #0           ; soft cw
        beq    format_softCW
        cmp    #1           ; cw
        beq    format_CW
        cmp    #2           ; SSB
        beq    format_SSB_jmp
        cmp    #3           ; DATAcw
        beq    format_data
format_SSB_jmp:
        jmp    format_SSB

format_softCW:
        lda    #'c'
        bra    formatCW_com

format_CW:
        lda    #'C'
formatCW_com:
        sta    DSP_status_msg
        lda    DSP_filterparms
        and    #$0F
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        clrh
        tax
        pula
        lda    CW_width_msg,x
        sta    DSP_status_msg+2
        aix    #1
        lda    CW_width_msg,x
        sta    DSP_status_msg+3
        aix    #1
        lda    CW_width_msg,x
        sta    DSP_status_msg+4
        lda    DSP_filterparms
        and    #$f0             ; get width nibble
        nsa                     ; into low nibble
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        clrh
        tax
        pula
        lda    CW_center_msg,x
        sta    DSP_status_msg+6
        aix    #1
        lda    CW_center_msg,x
        sta    DSP_status_msg+7
        aix    #1
        lda    CW_center_msg,x
        sta    DSP_status_msg+8
        aix    #1
        lda    CW_center_msg,x
        cmp    #' '
        beq    f_cw1
        sta    DSP_status_msg+9
f_cw1:  jmp    format_noise

format_data:
        lda    #'R'
        sta    DSP_status_msg
        lda    DSP_filterparms
        and    #$0f             ; get center freq. nibble
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        clrh
        tax
        pula
        lda    RTTY_width_msg,x
        sta    DSP_status_msg+2
        aix    #1
        lda    RTTY_width_msg,x
        sta    DSP_status_msg+3
        aix    #1
        lda    RTTY_width_msg,x
        sta    DSP_status_msg+4
        aix    #1
        lda    RTTY_width_msg,x
        cmp    #' '
        beq    fdata2
        sta    DSP_status_msg+5
        lda    DSP_filterparms
        and    #$f0             ; get width nibble
        nsa                     ; into low nibble
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        clrh
        tax
        pula
fdata1:
        lda    RTTY_center_msg,x
        sta    DSP_status_msg+7
        aix    #1
        lda    RTTY_center_msg,x
        sta    DSP_status_msg+8
        aix    #1
        lda    RTTY_center_msg,x
        cmp    #' '
        beq    fdata_ret
        sta    DSP_status_msg+9
        aix    #1
        lda    RTTY_center_msg,x
        cmp    #' '
        beq    fdata_ret
        cmp    #0
        beq    fdata_ret
        sta    {DSP_status_msg+10}    ; assembler bug without {} thinks it's +1
fdata_ret:
        jmp    format_noise
fdata2:
        lda    DSP_filterparms
        and    #$f0             ; get width nibble
        nsa                     ; into low nibble
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        clrh
        tax
        pula
        lda    RTTY_center_msg,x
        sta    DSP_status_msg+6
        aix    #1
        bra    fdata1

format_SSB:
        lda    #'S'
        sta    DSP_status_msg
        lda    DSP_filterparms
        and    #$0f             ; get center freq. nibble
        psha                    ; save
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus original == 5 times
        clrh
        tax
        pula
        lda    SSB_low_msg,x
        sta    DSP_status_msg+2
        aix    #1
        lda    SSB_low_msg,x
        sta    DSP_status_msg+3
        aix    #1
        lda    SSB_low_msg,x
        sta    DSP_status_msg+4
        aix    #1
        lda    SSB_low_msg,x
        cmp    #' '
        beq    sdata2
        sta    DSP_status_msg+5
        lda    DSP_filterparms
        and    #$f0             ; get width nibble
        nsa                     ; into low nibble
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        tax
        clrh
        pula
sdata1:
        lda    SSB_high_msg,x
        sta    DSP_status_msg+7
        aix    #1
        lda    SSB_high_msg,x
        sta    DSP_status_msg+8
        aix    #1
        lda    SSB_high_msg,x
        cmp    #' '
        beq    sdata_ret
        sta    DSP_status_msg+9
        aix    #1
        lda    SSB_high_msg,x
        cmp    #' '
        beq    sdata_ret
        cmp    #0
        beq    sdata_ret
        sta    {DSP_status_msg+10}
sdata_ret:
        jmp    format_noise
sdata2:
        lda    DSP_filterparms
        and    #$f0             ; get width nibble
        nsa                     ; into low nibble
        psha
        lsla                    ; times 4
        lsla
        add    1,sp             ; plus itself = times 5
        tax
        clrh
        pula
        lda    SSB_high_msg,x
        sta    DSP_status_msg+6
        aix    #1
        bra    sdata1

format_noise:                   ; common part: format denoiser,notch and exit
        lda    DSP_statusbyte
        and    #$08             ; isolate denoiser
        beq    format_notch
        lda    #'N'
        sta    {DSP_status_msg+12}
        lda    #'r'
        sta    {DSP_status_msg+13}
format_notch:
        lda    DSP_statusbyte
        and    #$04               ; isolate notch
        beq    format_exit
        lda    #'N'
        sta    {DSP_status_msg+14}
        lda    #'t'
        sta    {DSP_status_msg+15}
format_exit:
        rts


;------------- send 2 byte command in A, 2nd byte in X,   response to DSP_response
;                carry clear if OK, carry set for invalid command
send_cmd2:
        jsr     DSPputch           ; send command to DSP
        txa                        ; 2nd byte from X to A
        bra     s_c2c              ; send it
;------------- send 2 byte command DSP_cmd2 response to DSP_response
;                carry clear if OK, carry set for invalid command

send_cmd2a:
        lda     DSP_cmd2
        jsr     DSPputch           ; send to DSP
        lda     DSP_cmd2+1
s_c2c:  jsr     DSPputch           ; send to DSP
        jsr     DSPgetchne         ; get reply
        sta     DSP_statusbyte
        jsr     DSPgetchne
        sta     DSP_filterparms
        and     DSP_statusbyte
        cmp     #$ff               ; $FFFF response for bad command
        beq     cmderr2
        clc
        rts
cmderr2:
        sec
        rts

;;--------  save current status and filter parms to current mode's status
;;             used to save current setting when switching to new mode
saveModeStatus:
        lda    DSP_statusbyte
        and    #$03         ; isolate mode to do mode specific part
        cmp    #0           ; soft cw
        beq    save_CW
        cmp    #1           ; cw
        beq    save_CW
        cmp    #2           ; SSB
        beq    save_SSB
        cmp    #3           ; DATAcw
        beq    save_rtty
save_CW:
        lda    DSP_statusbyte
        sta    DSPcw_mode
        lda    DSP_filterparms
        sta    DSPcw_fil
        rts
save_SSB:
        lda    DSP_statusbyte
        sta    DSPSSB_mode
        lda    DSP_filterparms
        sta    DSPSSB_fil
        rts
save_rtty:
        lda    DSP_statusbyte
        sta    DSPrtty_mode
        lda    DSP_filterparms
        sta    DSPrtty_fil
        rts

;;-----------  clear status msg, format status and display it
dsp908_showStatus:
        ldhx    #DSP_status_msg
        sthx    from_addr
        lda     #16                    ; clear status msg
        ldx     #' '
        jsr     MemSet
        lda     #$80                   ; make sure we're on 1st line of LCD
        jsr     Cmnd2LCD
        jsr     format_status          ; format status in buffer
        ldhx    #DSP_status_msg
        jsr     disp_msg               ;display "DSP status" on 1st line
        rts

Checkdsp908_PB:
        brset   PB3,portg,C_PC_PB2   ; check button 3 pushed
                                     ; button set advance NR & Notch
        brclr   PB3,portg,*          ; await button release

        lda     DSP_statusbyte       ; in current status
        and     #$03                 ; isolate mode for mode specific action
        cmp     #2                   ; SSB cycle all 4 states
        beq     C_PC_PB1a
        cmp     #3
        beq     C_PC_exit            ; exit silently doing nothing
        lda     DSP_statusbyte       ; All other modes toggle denoiser only
        and     #$08                 ; is denoiser on ?
        beq     C_PC_PB1             ; no
        lda     DSP_statusbyte       ; in current status
        and     #$F7                 ; turn denoiser off
        sta     DSP_statusbyte
        jmp     C_PC_4               ; go tell DSP what we've done
C_PC_PB1:
        lda     DSP_statusbyte       ; in current status
        ora     #$08                 ; turn denoiser on
        sta     DSP_statusbyte
        jmp     C_PC_4               ; go tell DSP what we've done

C_PC_PB1a:
        lda     DSP_statusbyte
        and     #$F3                 ; remove NR/Notch
        ora     DSPtoggle_next       ; put in next value
        sta     DSP_statusbyte
        lda     DSPtoggle_next       ; advance to next
        add     #$04                 ; by adding 1
        and     #$0C                 ; in right place modulo 4
        sta     DSPtoggle_next
        jmp     C_PC_4               ; go tell DSP what we've done

C_PC_PB2:
        brset   PB2,porth,C_PC_exit  ; if PB2 not pushed exit
                                     ; button set advance width
        brclr   PB2,porth,*          ;await button release
        lda     DSP_statusbyte
        and    #$03         ; isolate mode to do mode specific part
        cmp    #2           ; SSB
        beq    adv_SSB
        cmp    #3           ; rtty
        beq    adv_rtty
        lda    DSPfilter_next
        cmp    #0
        bne    C_cw1
        ldx    #2            ; make next wide
        lda    DSPcw_narrow  ; make current narrow
        bra    C_cw3
C_cw1:  cmp    #1
        bne    C_cw2
        lda    DSPcw_mid     ; make current mid
        clrx                 ; make next narrow
        bra    C_cw3
C_cw2:  lda    DSPcw_wide    ; make current wide
        ldx    #1            ; make next mid

C_cw3:
        psha
        lda    DSPcw_fil     ; get current filter
        and    #$F0          ; clear width
        ora    1,sp          ; put in new width
        sta    DSPcw_fil             ; save new setting in cw filter
        sta    DSP_filterparms       ; and in current filter
        pula
        bra    C_PC_3

C_PC_exit:
        rts
adv_rtty:
        lda    DSPfilter_next
        cmp    #0
        bne    C_rt1
        ldx    #2            ; make next wide
        lda    DSPrtty_narrow  ; make current narrow
        bra    C_rt3
C_rt1:  cmp    #1
        bne    C_rt2
        lda    DSPrtty_mid     ; make current mid
        clrx                 ; make next narrow
        bra    C_rt3
C_rt2:  lda    DSPrtty_wide    ; make current wide
        ldx    #1            ; make next mid
C_rt3:
        psha
        lda    DSPrtty_fil     ; get current filter
        and    #$F0          ; clear width
        ora    1,sp          ; put in new width
        sta    DSPrtty_fil             ; save new setting in rtty filter
        sta    DSP_filterparms       ; and in current filter
        pula
        bra    C_PC_3

adv_SSB:
        lda    DSPfilter_next
        cmp    #0
        bne    C_ssb1
        ldx    #2            ; make next wide
        lda    DSPssb_narrow  ; make current narrow
        bra    C_ssb3
C_ssb1: cmp    #1
        bne    C_ssb2
        lda    DSPssb_mid     ; make current mid
        clrx                 ; make next narrow
        bra    C_ssb3
C_ssb2: lda    DSPssb_wide    ; make current wide
        ldx    #1            ; make next mid
C_ssb3:
        sta    DSPssb_fil             ; save new setting in ssb filter
        sta    DSP_filterparms       ; and in current filter

C_PC_3:           ; tell DSP what we have done
        stx    DSPfilter_next        ; advance to next setting in rotation
C_PC_4:                              ; PB2 enters here to exit
        ldx     DSP_filterparms      ; put current filter in command
        lda     DSP_statusbyte       ; get current status in command
        ora     #$80                 ; indicate command is 2 byte command
        jsr     send_cmd2            ; send command
        jsr     dsp908_showStatus    ; tell the world
        rts                          ; go back to where we were

dsp908_EEPROM_Defaults:
           ;  Set default current mode & filter parms
           ;               mode and filter by mode & default widths/mode
;;  CW and default filter is CW
        lda     #$31                   ; mode CW , 91 taps
        sta     DSP_statusbyte
        sta     DSPcw_mode
        lda     #$69                   ; CW 500 wide center 600
        sta     DSPcw_fil
        sta     DSP_filterparms
        lda     #$0F                   ; CW  800 wide
        sta     DSPcw_wide
        lda     #$04                   ; CW 250 mid
        sta     DSPcw_mid
        lda     #$01                   ; CW 100 narrow
        sta     DSPcw_narrow
;;;RTTY
        lda     #$33                   ; mode RTTY , 91 taps
        sta     DSPrtty_mode
        lda     #$CA                   ; rtty  1000 wide center 2200
        sta     DSPrtty_fil
        lda     #$0A                   ; rtty  1000 wide
        sta     DSPrtty_wide
        lda     #$03                   ; rtty  300 mid
        sta     DSPrtty_mid
        lda     #$02                   ; rtty  200 narrow
        sta     DSPrtty_narrow
;;SSB
        lda     #$32                   ; mode SSB , 91 taps
        sta     DSPssb_mode
        lda     #$A3                   ; ssb  200-2500
        sta     DSPssb_fil
        sta     DSPssb_wide
        lda     #$45                   ; ssb 300-1900
        sta     DSPssb_mid
        lda     #$15                   ; ssb 300-1600
        sta     DSPssb_narrow
;;Gains
        lda     #$20                    ; 6dB gain
        sta     DeNoisergain
        lda     #$10                    ; 0dB gain
        sta     AutoNotchgain
        sta     CWgain
        sta     SSBgain
        sta     RTTYgain
        lda     #$7F                    ; 6dB gain CODEC is different!
        sta     CODECgain
;; betas and decays                     ; values stored as 0-99
        lda     #89                     ; closest match to 1:15 format
        sta     DenoiserBeta            ; binary defaults in DSP code
        lda     #71
        sta     DenoiserDecay
        lda     #39
        sta     NotchBeta
        lda     #7
        sta     NotchDecay
        rts

Restore908:                      ; sets DSP to last saved values on start up
        lda     DSP_statusbyte   ; save the saved status until after all other
        psha                     ;   commands are sent as the response to
        lda     DSP_filterparms  ;    the command reponses will overwrite them
        psha
;;Gains
        lda     #$D0
        ldx     DeNoisergain
        jsr     send_cmd2
        lda     #$D1
        ldx     AutoNotchgain
        jsr     send_cmd2
        lda     #$D2
        ldx     CWgain
        jsr     send_cmd2
        lda     #$D3
        ldx     SSBgain
        jsr     send_cmd2
        lda     #$D4
        ldx     RTTYgain
        jsr     send_cmd2
        lda     #$D5
        ldx     CODECgain
        jsr     send_cmd2

;; betas and decays
        lda     #$C1
        ldx     DenoiserBeta
        jsr     send_cmd2
        lda     #$C2
        ldx     DenoiserDecay
        jsr     send_cmd2
        lda     #$C4
        ldx     NotchBeta
        jsr     send_cmd2
        lda     #$C8
        ldx     NotchDecay
        jsr     send_cmd2

;; mode and filter parms
        pulx                           ; recover saved DSP_filterparms
        pula                           ; recover saved DSP_statusbyte
        ora     #$80
        jsr     send_cmd2
        rts
