;****************************************************************************
;                         REACTANCE SIGN MODULE
;
;                Display the sign of the reactance display
;
;****************************************************************************

;------------------------------------------------------------------------------
;Disp_Sign - Display the sign of the reactance display when the SLEEP PB is pressed
;
;            LCD_Dat+0, LCD_Dat+1, LCD_Dat+2, LCD_Dat+3, LCD_Dat+4, LCD_Dat+5, LCD_Dat+6
;              10 MHz     1 MHz     100 kHz     10 kHz     1 kHz     100 Hz      10 Hz
;

Disp_Sign:
        brclr   SleepPB,portG,dsgn1     ;only do this function when the SLEEP PB is pressed
        lda     #' '                    ;else display blank for sign
        sta     sign
        rts                             ;and exit
        
dsgn1:  ldhx    Reactance
        sthx    Current_React
;        jsr     print_HX
;        jsr     _space
        jsr     SaveFreqDisplay         ;save current freq dial setting of LCD_Dat(0:6)

            ;determine if dial is currently above or below 20 MHz

        lda     LCD_Dat+0
        beq     check_higher            ;if dial = 0x,xxx.xx, then check higher
        cmp     #3
        beq     check_lower             ;if dial = 3x,xxx.xx, then check lower
        cmp     #2
        beq     check_lower             ;if dial = 2x,xxx.xx, then check lower
        nop                             ;else dial = 1x,xxx.xx, so fall thru to check higher

         ;--------------------------------
         ; Check reactance at a higher frequency if dial currently set below 20 MHz
         ;        SIGN DETERMINATION ALGORITHM:
         ;        increasing X with increasing F ==> inductive reactance = +j
         ;        decreasing X with increasing F ==> capacitive reactance = -j

check_higher:
        ldhx    #1                      ;point to the 01 digit in LCD_Dat array
        jsr     BCD_INC                 ;increment freq by 1MHz
        ldhx    #1                      ;point to the 01 digit in LCD_Dat array
        jsr     BCD_INC                 ;increment freq by 1MHz
        ldhx    #1                      ;point to the 01 digit in LCD_Dat array
        jsr     BCD_INC                 ;increment freq by 1MHz
        jsr     Mult                    ;calc new frequency
        jsr     Set_DDS                 ;update the DDS chip
        jsr     wait10ms
        lda     #1
        sta     No_Disp_Flag            ;set the No Display Flag so we don't change anything on LCD
        jsr     Do_Calcs                ;calculate SWR, R and X

        ldhx    Reactance               ;get X reading from higher freq
;        jsr     print_HX
        txa
        sub     Current_React+1         ;compare to X reading from current (lower) freq
        tha
        sbc     Current_react+0
        bcc     dsgn3                   ;Higher-freq Reactance is larger than current (lower-freq) Current_React

            ; X(f_high) < X(f_low) ==> CAPACITIVE Reactance ==> -j
        lda     #'-'
        sta     sign
;       jsr     print_A
        bra     dsgn_exit

            ; X(f_high) > X(f_low) ==> INDUCTIVE Reactance ==> +j
dsgn3:  lda     #'+'
        sta     sign
;        jsr     print_A
        jmp      dsgn_exit

        ;--------------------------------
        ; Check reactance at a lower frequency if dial currently set above 20 MHz
        ;        SIGN DETERMINATION ALGORITHM:
        ;        increasing X with decreasing F ==> capacitive reactance = -j
        ;        decreasing X with decreasing F ==> inductive reactance = +j

check_lower:
        ldhx    #1                      ;point to the 01 digit in LCD_Dat array
        jsr     BCD_DEC                 ;decrement freq by 1MHz
        ldhx    #1                      ;point to the 01 digit in LCD_Dat array
        jsr     BCD_DEC                 ;decrement freq by 1MHz
        ldhx    #1                      ;point to the 01 digit in LCD_Dat array
        jsr     BCD_DEC                 ;decrement freq by 1MHz
        jsr     Mult                    ;calc new frequency
        jsr     Set_DDS                 ;update the DDS chip
        jsr     wait10ms
        lda     #1
        sta     No_Disp_Flag            ;set the No Display Flag so we don't change anything on LCD
        jsr     Do_Calcs                ;calculate SWR, R and X

        ldhx    Reactance               ;get X reading from lower freq
;        jsr     print_HX
        txa
        sub     Current_React+1         ;compare to X reading from current (higher) freq
        tha
        sbc     Current_react+0
        bcc     dsgn4                   ;Higher-freq Reactance is lower than current (lower-freq) Current_React

            ; X(f_high) > X(f_low) ==> INDUCTIVE Reactance ==> +j
        lda     #'+'
        sta     sign
;       jsr     print_A
        bra     dsgn_exit

            ; X(f_high) < X(f_low) ==> CAPACITIVE Reactance ==> -j
dsgn4:  lda     #'-'
        sta     sign
;        jsr     print_A

            ;--------------------
            ;exit
dsgn_exit:
        jsr     RestoreFreqDisplay      ;point the frequency dial back at the current point
        jsr     Mult                    ;calc new frequency
        jsr     Set_DDS                 ;update the DDS chip
        clra
        sta     No_Disp_Flag            ;and display again to the LCD
        ldhx    Current_React
        sthx    Reactance               ;restore reactance to what it was at specific freq
        rts

;---------------------------------
;Save Frequency Display

SaveFreqDisplay:
        ldhx    #LCD_Dat
        lda     0,x
        sta     FreqSave+0
        lda     1,x
        sta     FreqSave+1
        lda     2,x
        sta     FreqSave+2
        lda     3,x
        sta     FreqSave+3
        lda     4,x
        sta     FreqSave+4
        lda     5,x
        sta     FreqSave+5
        lda     6,x
        sta     FreqSave+6
        rts

;---------------------------------
;Restore Frequency Display

RestoreFreqDisplay:
        ldhx    #FreqSave
        lda     0,x
        sta     LCD_Dat+0
        lda     1,x
        sta     LCD_Dat+1
        lda     2,x
        sta     LCD_Dat+2
        lda     3,x
        sta     LCD_Dat+3
        lda     4,x
        sta     LCD_Dat+4
        lda     5,x
        sta     LCD_Dat+5
        lda     6,x
        sta     LCD_Dat+6
        rts

;------------------------------------------------------------------------------
; GetFreq - read LCD_Dat(0:6) frequency registers
;           and convert 10M+1M+100K+10K+1K digits to integer in HX
;            LCD_Dat+0, LCD_Dat+1, LCD_Dat+2, LCD_Dat+3, LCD_Dat+4, LCD_Dat+5, LCD_Dat+6
;              10 MHz     1 MHz     100 kHz     10 kHz     1 kHz     100 Hz      10 Hz

GetFreq:
        ldhx   #0                 ;clear "accumulator"

             ;check 10M digit
        lda    LCD_Dat+0          ;get 1M ascii digit
        beq    db0a               ;zero already, so nothing to add
db0:    jsr    add10000
        deca
        bne    db0

             ;check 1M digit
db0a:   lda    LCD_Dat+1          ;get 1M ascii digit
        beq    db1a               ;zero already, so nothing to add
db1:    jsr    add1000
        deca
        bne    db1

            ;check 100K digit
db1a:   lda    LCD_Dat+2          ;get 100K ascii digit
        beq    db3                ;zero already, so nothing to add
db2:    aix    #100
        deca
        bne    db2

            ;check 10K digit
db3:    lda    LCD_Dat+3          ;get 10K digit
        beq    db5                ;zero already, so nothing to add
db4:    aix    #10
        deca
        bne    db4

            ;check 1K digit
db5:    lda    LCD_Dat+4          ;get 1K digit
        beq    db7                ;already zero, so nothing to add
db6:    aix    #1
        deca
        bne    db6

db7:    sthx   Freq
        rts

            ;-------------
            ;Add1000 - Adds 1,000 to HX
add1000:
        psha
        txa
        add    #$e8
        tax
        tha
        adc    #$03
        tah
        pula
        rts

            ;-------------
            ;Add10000 - Adds 10,000 to HX
add10000:
        psha
        txa
        add    #$10
        tax
        tha
        adc    #$27
        tah
        pula
        rts
